package datavisualizer;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import java.io.IOException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.InputSource;

import java.io.FileInputStream;
import java.io.InputStream;
import com.ice.tar.TarInputStream;
import com.ice.tar.TarEntry;

import javax.swing.JInternalFrame;
import java.util.Vector;

import javawrapper.DataVisualizerManager;
import javawrapper.MainWindowStateSetter;
import javawrapper.ModelException;
import javawrapper.ErrorGUI;
import javawrapper.DetailedGridSettings;
import javawrapper.DetailedTreeSettings;
import javawrapper.OutputBehaviors;

/**
 * Controls the data visualization for a single detailed output file.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.3
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>June 28, 2004:  Added support for snags (LEM)
 * <br>March 9, 2005:  Added line graph for volume and made version 1.1 (LEM)
 * <br>March 29, 2005:  Added histograms for grids and made version 1.2 (LEM)
 * <br>March 20, 2006:  Added package support and made version 1.3 (LEM)
 * <br>January 3, 2007: Added better subplot support including specific area
 * calculation (LEM)
 * <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
 * <br>October 20, 2010: Added the ability to see a histogram for all species 
 * at once for a grid (LEM)
 * <br>March 3, 2011: Added support for dead trees (LEM) 
 */

public class DetailedOutputFileManager
    extends DataFileManager {
  
  /**The collection of all data requests currently open*/
  private Vector<DataRequest> mp_oDataRequests = new Vector<DataRequest>(0);

  /**Species names - the order determines the species codes*/
  private Vector<String> mp_sSpeciesNames = new Vector<String>(0);

  /**Collection of DetailedTreeSettings objects*/
  private Vector<DetailedTreeSettings> mp_oTreeSettings = new Vector<DetailedTreeSettings>(0);

  /**Collection of DetailedGridSettings objects*/
  private Vector<DetailedGridSettings> mp_oGridSettings = new Vector<DetailedGridSettings>(0);

  /**Whether each histogram option is float (true) or int (false)*/
  private boolean[] mp_bHistogramIsFloat;

  /**Histogram names - so we can figure out which value in mp_bHistogramIsFloat
   * value to use*/
  private String[] mp_sHistogramNames;
  
  /**Dead reason codes for each tree data member*/
  private int[] mp_iDeadCodes;

  /**Display string of filename - should have the last 20 chars.*/
  private String m_sFileDisplayName;
  private XMLReader m_oParser; /**<File parser*/
  private String m_sTempRoot, /**<Root directory to put temp files*/
      m_sTempDir, /**<Where we actually extract to*/
      m_sFileRoot; /**<Root file name of all detailed output files*/
  private float
      m_fPlotArea = 0, /**<Area of plot (or subplot) in hectares*/
      m_fXPlotLength = 0, /**<Plot X length*/
      m_fYPlotLength = 0, /**<Plot Y length*/
      /**X length of subplot cells, in m, if this is a subplot*/
      m_fSubplotXCellLength = 8,  
      /**Y length of subplot cells, in m, if this is a subplot*/
      m_fSubplotYCellLength = 8,
      m_fNumYearsPerTimestep; /**<Number of years per timestep*/
  private int m_iNumTimesteps, /**<Number of actual timesteps of data*/
      m_iNumParFileTimesteps, /**<Number of timestes from the par file*/
      m_iNumTypes = 7, /**<Number of tree types*/
      m_iLastTimestepRead; /**<When updating charts, this is the file parsed*/

  /**
   * Sets the number of timesteps for the detailed output file parameter file.
   * @param i Number of timesteps.
   */
  public void SetParFileTimesteps(int i) {
    m_iNumParFileTimesteps = i;
  }

  /**
   * Gets the number of actual data timesteps contained in this detailed output
   * file (as opposed to how many the par file says there are).
   * @return int Number of timesteps for this detailed output file.
   */
  public int GetNumberOfActualTimesteps() {
    return m_iNumTimesteps;
  }
  
  /**
   * Sets the subplot X cell length.
   * @param f X cell length, in m.
   */
  public void SetSubplotXCellLength(float f) {
  	m_fSubplotXCellLength = f;
  }
  
  /**
   * Sets the subplot Y cell length.
   * @param f Y cell length, in m.
   */
  public void SetSubplotYCellLength(float f) {
  	m_fSubplotYCellLength = f;
  }
  
  /**
   * Checks to see if a subplot name indicates that the detailed output file 
   * being managed is in fact for that subplot. If it were, the subplot name
   * would be the last part of the tarball name preceded by an underscore.
   * @param sSubplotName Name of possible subplot for this file.
   * @return True if this is a subplot with that name, false otherwise.
   */
  public boolean IsThisSubplot(String sSubplotName) {
  	if (m_sFileRoot.endsWith("_" + sSubplotName))
  		return true;
  	return false;
  }
  
  /**
   * Gets the area of this plot or subplot, in ha.
   * @return Plot area in ha.
   */
  public float GetPlotArea() {
  	return m_fPlotArea;
  }

  /**
   * Handles the request to draw a chart for this file.  The request will be
   * passed on to an appropriate DataRequest object.  If the chart is already
   * open, the open chart will be returned.
   * @param sGraphName Name of chart to draw.
   * @return The drawn chart, or NULL if it could not be drawn.
   * @throws ModelException If anything goes wrong.
   */
  public JInternalFrame DrawChart(String sGraphName) throws ModelException {

    DataRequest oRequest = null;
    boolean bValidChart = false;

    //See if the chart wanted is already open
    int i;
    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (sGraphName.equals(oRequest.GetChartName())) {
        return oRequest.GetChart();
      }
    }

    if (sGraphName.equalsIgnoreCase("XY DBH Tree Map")) {
      oRequest = new TreeMapDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.equalsIgnoreCase("Timestep Tree Writer")) {
      oRequest = new TreeListWriter(sGraphName, this);
      bValidChart = true;
    }
    else if (sGraphName.equalsIgnoreCase("Crown Radius Tree Map")) {
      oRequest = new TreeMapCrownRadDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.endsWith("Histogram")) {
      //Find the index of the histogram
      int iIndex = 0;
      for (i = 0; i < mp_sHistogramNames.length; i++) {
        if (mp_sHistogramNames[i].equals(sGraphName)) {
          iIndex = i;
          break;
        }
      }
      if (sGraphName.startsWith("Grid")) {
        if (sGraphName.indexOf(" All ") > -1) {
          String sGridName = mp_sHistogramNames[i].substring(mp_sHistogramNames[i].
              indexOf(":") + 2, mp_sHistogramNames[i].indexOf(" - "));
          String sMemberName = mp_sHistogramNames[i].substring(mp_sHistogramNames[
              i].indexOf(" All ") + 5, mp_sHistogramNames[i].indexOf("Histogram") - 1);
          oRequest = new GridAllSpeciesHistogramDataRequest(sMemberName,
                                              mp_bHistogramIsFloat[iIndex], this,
                                              sGraphName, sGridName);
        } else {
          String sGridName = mp_sHistogramNames[i].substring(mp_sHistogramNames[i].
              indexOf(":") + 2, mp_sHistogramNames[i].indexOf(" - "));
          String sMemberName = mp_sHistogramNames[i].substring(mp_sHistogramNames[
              i].indexOf(" - ") + 3, mp_sHistogramNames[i].indexOf("Histogram") - 1);
          oRequest = new GridHistogramDataRequest(sMemberName,
                                              mp_bHistogramIsFloat[iIndex], this,
                                              sGraphName, sGridName);
        }
      } else {
        if (sGraphName.indexOf("Dead") > -1) {
          String sDead = sGraphName.substring(0, sGraphName.indexOf(" Dead"));
          sGraphName = sDead.substring(0, sDead.lastIndexOf(" "));
          sDead = sDead.substring(sDead.lastIndexOf(" ")+1);
          for (int j = 0; j < mp_sDeadCodeNames.length; j++) {
            if (sDead.equals(mp_sDeadCodeNames[j])) {
              oRequest = new HistogramDataRequest(sGraphName,
                  mp_bHistogramIsFloat[iIndex], this, sGraphName, j);
            }
          }       
        } else {
          oRequest = new HistogramDataRequest(sGraphName.substring(0,
              sGraphName.indexOf(" Histogram")),
              mp_bHistogramIsFloat[iIndex], this, sGraphName, 
              OutputBehaviors.NOTDEAD);
        }
      }
      bValidChart = true;
    }

    //Grid map object
    else if (sGraphName.startsWith("Map")) {

      //Parse out the grid name and data member name
      String sGridName = sGraphName.substring(sGraphName.indexOf(" - ") + 3,
                                              sGraphName.lastIndexOf(" - "));
      String sDataMember = sGraphName.substring(sGraphName.lastIndexOf(" - ") +
                                                3);
      oRequest = new GridDataRequest(sGraphName, sGridName, sDataMember, this);
      bValidChart = true;
    }

    //Tables
    else if (sGraphName.equals("Whole plot: Table")) {
      oRequest = new OverviewTableDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.equals("BA Stand Table")) {
      oRequest = new StandTableDataRequest(this, sGraphName,
                                           StandTableDataRequest.BASAL_AREA);
      bValidChart = true;
    }
    else if (sGraphName.equals("Density Stand Table")) {
      oRequest = new StandTableDataRequest(this, sGraphName,
                                           StandTableDataRequest.DENSITY);
      bValidChart = true;
    }
    else if (sGraphName.equals("Stock Table")) {
      oRequest = new StockTableDataRequest(this, sGraphName);
      bValidChart = true;
    }
    else if (sGraphName.endsWith("Harvest Results") ||
             sGraphName.endsWith("Mortality Episode Results")) {
      oRequest = new HarvestDataRequest(sGraphName, this);
      bValidChart = true;
    }

    else if (sGraphName.endsWith("Windstorm Results")) {
      oRequest = new WindstormDataRequest(sGraphName, this);
      bValidChart = true;
    }

    else if (sGraphName.endsWith("Merchantable Timber Value")) {
      oRequest = new MerchValueDataRequest(sGraphName, this);
      bValidChart = true;
    }

    else if (sGraphName.endsWith("Carbon Value")) {
      oRequest = new CarbonValueDataRequest(sGraphName, this);
      bValidChart = true;
    }

    else if (sGraphName.endsWith("Partitioned Biomass") &&
             !sGraphName.endsWith("Storm Killed Partitioned Biomass")) {
      oRequest = new PartitionedBiomassTableDataRequest(sGraphName, this);
      bValidChart = true;
    }
    
    else if (sGraphName.endsWith("Foliar Chemistry")) {
      oRequest = new FoliarChemistryTableDataRequest(sGraphName, this);
      bValidChart = true;
    }
    
    else if (sGraphName.endsWith("Dispersed Seeds")) {
      oRequest = new SeedTableDataRequest(sGraphName, this);
      bValidChart = true;
    }

    else if (sGraphName.endsWith("Storm Killed Partitioned Biomass")) {
      oRequest = new StormKilledPartitionedBiomassDataRequest(sGraphName, this);
      bValidChart = true;
    }
    else if (sGraphName.endsWith("Storm Summary")) {
    	oRequest = new StormDataRequest(sGraphName, this);
    	bValidChart = true;
    }
    else if (sGraphName.endsWith("State Variables")) {
      oRequest = new StateVariableTableDataRequest(sGraphName, this); 
      bValidChart = true; 
    }


    //Line graphs
    else if (sGraphName.equals("Whole plot: Seedling absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SEEDLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Seedling relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SEEDLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SAPLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SAPLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.SAPLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Sapling relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.SAPLING,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.ADULT,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.ADULT,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.ADULT,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Adult relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.ADULT,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag absolute density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_DENSITY,
                                          javawrapper.TreePopulation.SNAG,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag relative density")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_DENSITY,
                                          javawrapper.TreePopulation.SNAG,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag absolute basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.ABSOLUTE_BASAL_AREA,
                                          javawrapper.TreePopulation.SNAG,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.equals("Whole plot: Snag relative basal area")) {
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          LineGraphDataRequest.RELATIVE_BASAL_AREA,
                                          javawrapper.TreePopulation.SNAG,
                                          OutputBehaviors.NOTDEAD);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("abs den") > -1) {
      int iType = javawrapper.TreePopulation.SAPLING,
      iDeadCode = OutputBehaviors.NOTDEAD;

      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }

      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("seedling") > -1)
        iType = javawrapper.TreePopulation.SEEDLING;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;

      oRequest = new LineGraphDataRequest(this, sGraphName,
          LineGraphDataRequest.ABSOLUTE_DENSITY,
          iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("rel den") > -1) {
      int iType = javawrapper.TreePopulation.SAPLING,
      iDeadCode = OutputBehaviors.NOTDEAD;

      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }

      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("seedling") > -1)
        iType = javawrapper.TreePopulation.SEEDLING;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;

      oRequest = new LineGraphDataRequest(this, sGraphName,
          LineGraphDataRequest.RELATIVE_DENSITY,
          iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("abs ba") > -1) {
      int iType = javawrapper.TreePopulation.SAPLING,
      iDeadCode = OutputBehaviors.NOTDEAD;

      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }

      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;

      oRequest = new LineGraphDataRequest(this, sGraphName,
          LineGraphDataRequest.ABSOLUTE_BASAL_AREA,
          iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("rel ba") > -1) {
      int iType = javawrapper.TreePopulation.SAPLING,
      iDeadCode = OutputBehaviors.NOTDEAD;

      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }

      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;

      oRequest = new LineGraphDataRequest(this, sGraphName,
          LineGraphDataRequest.RELATIVE_BASAL_AREA,
          iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("volume") > -1) {
      int iChartType = LineGraphDataRequest.ABSOLUTE_VOLUME,
          iType = javawrapper.TreePopulation.SAPLING,
          iDeadCode = OutputBehaviors.NOTDEAD;
      if (sGraphName.toLowerCase().indexOf("relative") > -1 ||
          sGraphName.toLowerCase().indexOf("rel") > -1)
        iChartType = LineGraphDataRequest.RELATIVE_VOLUME;
      
      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }
      
      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;
      
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          iChartType, iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.toLowerCase().indexOf("biomass") > -1 &&
             (sGraphName.toLowerCase().indexOf("rel") > -1 ||
              sGraphName.toLowerCase().indexOf("abs") > -1)) {
      int iChartType = LineGraphDataRequest.ABSOLUTE_BIOMASS,
          iType = javawrapper.TreePopulation.SAPLING,
          iDeadCode = OutputBehaviors.NOTDEAD;
      if (sGraphName.toLowerCase().indexOf("rel") > -1)
        iChartType = LineGraphDataRequest.RELATIVE_BIOMASS;
      
      if (sGraphName.toLowerCase().indexOf("dead") > -1) {
        String sDead = sGraphName.substring(0, sGraphName.indexOf(' '));
        int k;
        for (k = 0; k < mp_sDeadCodeNames.length; k++) 
          if (sDead.equalsIgnoreCase(mp_sDeadCodeNames[k]))
            iDeadCode = k;
      }
      
      if (sGraphName.toLowerCase().indexOf("adult") > -1) 
        iType = javawrapper.TreePopulation.ADULT;
      else if (sGraphName.toLowerCase().indexOf("snag") > -1)
        iType = javawrapper.TreePopulation.SNAG;
      
      oRequest = new LineGraphDataRequest(this, sGraphName,
                                          iChartType, iType, iDeadCode);
      bValidChart = true;
    }
    else if (sGraphName.equals("Ripley's K")) {
    	oRequest = new RipleysKDataRequest(sGraphName, this);
    	bValidChart = true;
    }
    else if (sGraphName.equals("Relative Neighborhood Density")) {
    	oRequest = new RelativeNeighborhoodDensityDataRequest(sGraphName, this);
    	bValidChart = true;
    }
    else if (sGraphName.startsWith("Foliar Chemistry")) {
      oRequest = new FoliarChemistryLineGraphDataRequest(sGraphName, this);
      bValidChart = true;
    }
    else if (sGraphName.startsWith("Biomass")) {
    	oRequest = new PartitionedBiomassLineGraphDataRequest(sGraphName, this);
    	bValidChart = true;
    }

    if (bValidChart) {
      mp_oDataRequests.add(oRequest);

      //Parse our file
      ReadFile(m_iLastTimestepRead);
      return oRequest.DrawChart(m_oLegend,
                                sGraphName + " - " + m_sFileDisplayName);
    }
    else {
      return null;
    }
  }

  /**
   * Updates charts for a current run.  This re-inflates the tarball and gets
   * the last timestep file for parsing and processing.
   * @throws ModelException if there is a problem reading the file or
   * drawing the charts.
   */
  public void UpdateCurrentRunCharts() throws ModelException {
    if (mp_oCharts.size() == 0) {
      return;
    }

    //Get the new number of timesteps
    CountTimesteps();

    //Extract the tarball again
    javawrapper.Tarball.ExtractTarball(m_sFilename, m_sTempRoot);

    //Advance through the legend
    DetailedOutputLegend oLegend = (DetailedOutputLegend) m_oLegend;
    oLegend.SetNumberOfTimesteps(m_iNumTimesteps);

    ( (DetailedOutputLegend) m_oLegend).GoToTimestep(m_iNumTimesteps);
  }

  /**
   * Gets the tables that can be drawn for this file.
   * @return String of tables, or NULL if no tables can be drawn.
   */
  public String[] GetTableOptions() {

    Vector<String> p_oOptions = new Vector<String>(0);
    DetailedGridSettings oSettings;
    int i, j;
    
    p_oOptions.add("Timestep Tree Writer");

    //Check our grids for specialized grid table options
    for (i = 0; i < mp_oGridSettings.size(); i++) {

      oSettings = (DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals("Harvest Results")) {
        p_oOptions.add("Harvest Results");
      }
      else if (oSettings.GetName().equals("Mortality Episode Results")) {
        p_oOptions.add("Mortality Episode Results");
      }
      else if (oSettings.GetName().equals("Windstorm Results")) {
        p_oOptions.add("Windstorm Results");
      }
      else if (oSettings.GetName().equals("Merchantable Timber Value")) {
        p_oOptions.add("Merchantable Timber Value");
      }
      else if (oSettings.GetName().equals("Carbon Value")) {
        p_oOptions.add("Carbon Value");
      }
      else if (oSettings.GetName().equals("Foliar Chemistry")) {
        p_oOptions.add("Foliar Chemistry");
      }
      else if (oSettings.GetName().equals("Dispersed Seeds")) {
        p_oOptions.add("Dispersed Seeds");
      }
      else if (oSettings.GetName().equals("Partitioned Biomass")) {
        p_oOptions.add("Partitioned Biomass");
      }
      else if (oSettings.GetName().equals("Storm Killed Partitioned Biomass")) {
        p_oOptions.add("Storm Killed Partitioned Biomass");
      }
      else if (oSettings.GetName().equals("Storm Damage")) {
      	for (j = 0; j < oSettings.GetNumberOfPackageFloats(); j++) {
      	  if (oSettings.GetPackageFloat(j).GetCodeName().equals("1dmg_index")) {
      	  	p_oOptions.add("Storm Summary");
      	  	break;
      	  }
      	}	
      }	
      else if (oSettings.GetName().equals("State Variables")) {
        p_oOptions.add("State Variables");
      }
    }

    //See if there are tree DBH or diam10 datamembers
    DetailedTreeSettings oTreeSetting;

    //Can we do diameter (overview table)?
    boolean bHasDiam = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          bHasDiam = true;
          break;
        }
        else if (sLabel.equalsIgnoreCase("diam10") &&
                 oTreeSetting.GetType() == javawrapper.TreePopulation.SEEDLING) {
          bHasDiam = true;
          break;
        }
      }
      if (bHasDiam) {
        p_oOptions.add("Whole plot: Table");
        break;
      }
    }

    //Can we do DBH (stand tables)?
    boolean bHasDBH = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          bHasDBH = true;
          break;
        }
      }
      if (bHasDBH) {
        p_oOptions.add("BA Stand Table");
        p_oOptions.add("Density Stand Table");
        break;
      }
    }

    //If we have volume, we can do a stock table
    boolean bHasVolume = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("volume")) {
          bHasVolume = true;
          break;
        }
      }
      if (bHasVolume) {
        p_oOptions.add("Stock Table");
        break;
      }
    }

    //Transfer the options list to an array
    if (p_oOptions.size() > 0) {
      String[] p_sOptions = new String[p_oOptions.size()];
      for (i = 0; i < p_sOptions.length; i++) {
        p_sOptions[i] = (String) p_oOptions.elementAt(i);
      }
      return p_sOptions;
    }
    return null;
  }

  /**
   * Gets the histograms that can be drawn for this file.
   * @return String of tables, or NULL if no histograms can be drawn.
   */
  public String[] GetHistogramOptions() {
    Vector<String> oOptions = new Vector<String>(0);
    Vector<Boolean> oIsFloat = new Vector<Boolean>(0);
    Vector<Integer> oDeadCode = new Vector<Integer>(0);
    DetailedTreeSettings oSetting;
    DetailedGridSettings oGridSetting;
    String sLabel, sTestLabel;
    int i, j, k, iDeadCode;
    boolean bFound;

    for (i = 0; i < mp_oTreeSettings.size(); i++) {

      oSetting = (DetailedTreeSettings) mp_oTreeSettings.elementAt(i);

      //Go through the floats - we could histogram any of them
      for (j = 0; j < oSetting.GetNumberOfFloats(); j++) {

        //But we'll skip X and Y
        sLabel = oSetting.GetFloat(j).GetCodeName();
        if (!sLabel.equalsIgnoreCase("x") && !sLabel.equalsIgnoreCase("y")) {
          iDeadCode = oSetting.GetDeadCode();

          //Make sure we haven't already added this float
          bFound = false;
          for (k = 0; k < oOptions.size(); k++) {
            sTestLabel = (String) oOptions.elementAt(k);
            if (sLabel.equals(sTestLabel) && 
                oDeadCode.elementAt(k) == iDeadCode) {
              bFound = true;
              break;
            }
          }
          if (!bFound) {
            oOptions.add(sLabel);
            oIsFloat.add(new Boolean(true));
            oDeadCode.add(new Integer(iDeadCode));
          }
        }
      }
      //Go through the ints - we could histogram any of them
      for (j = 0; j < oSetting.GetNumberOfInts(); j++) {

        sLabel = oSetting.GetInt(j).GetCodeName();
        iDeadCode = oSetting.GetDeadCode();

        //Make sure we haven't already added this int
        bFound = false;
        for (k = 0; k < oOptions.size(); k++) {
          sTestLabel = (String) oOptions.elementAt(k);
          if (sLabel.equals(sTestLabel) && 
              oDeadCode.elementAt(k) == iDeadCode) {
            bFound = true;
            break;
          }
        }
        if (!bFound) {
          oOptions.add(sLabel);
          oIsFloat.add(new Boolean(false));
          oDeadCode.add(new Integer(iDeadCode));
        }
      }
    }

    //At the top of the list, put all species histograms for the grids that 
    //support it 
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oGridSetting = (DetailedGridSettings) mp_oGridSettings.elementAt(i);
      String sName = oGridSetting.GetName();
      if (sName.equals("Carbon Value") ||
          sName.equals("Competition Harvest Results") ||
          sName.equals("Dispersed Seeds") ||
          sName.equals("Foliar Chemistry") ||
          sName.equals("Merchantable Timber Value") ||
          sName.equals("Neighborhood Seed Predation") ||
          sName.equals("Partitioned Biomass") ||
          sName.equals("Storm Killed Partitioned Biomass") ||
          sName.equals("Substrate Favorability") ||
          sName.equals("Temperature Dependent Neighborhood Survival") ||
          sName.equals("Weibull Climate Quadrat Growth")) {
      
        //Go through the floats - look for species specific 
        for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {

          sLabel = oGridSetting.GetFloat(j).GetCodeName();
          if (sLabel.matches(".*_\\d.*")) {
            sLabel = sLabel.substring(0, sLabel.lastIndexOf("_"));
            sLabel = "Grid: " + oGridSetting.GetName() + " - All " + sLabel;
            
            //Make sure we haven't already added this float
            bFound = false;
            for (k = 0; k < oOptions.size(); k++) {
              sTestLabel = (String) oOptions.elementAt(k);
              if (sLabel.equals(sTestLabel)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              oOptions.add(sLabel);
              oIsFloat.add(new Boolean(true));
              oDeadCode.add(OutputBehaviors.NOTDEAD);
            }
          }
        }
        //Repeat with ints
        for (j = 0; j < oGridSetting.GetNumberOfInts(); j++) {
          sLabel = oGridSetting.GetInt(j).GetCodeName();
          if (sLabel.matches(".*_\\d.*")) {
            sLabel = sLabel.substring(0, sLabel.lastIndexOf("_"));
            sLabel = "Grid: " + oGridSetting.GetName() + " - All " + sLabel;
            bFound = false;
            for (k = 0; k < oOptions.size(); k++) {
              sTestLabel = (String) oOptions.elementAt(k);
              if (sLabel.equals(sTestLabel)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              oOptions.add(sLabel);
              oIsFloat.add(new Boolean(false));
              oDeadCode.add(OutputBehaviors.NOTDEAD);
            }
          }
        }
      }
    }
    
    //LEM 03-29-05
    for (i = 0; i < mp_oGridSettings.size(); i++) {

      oGridSetting = (DetailedGridSettings) mp_oGridSettings.elementAt(i);
      
      //Skip some grids, because they just swamp the options list without 
      //producing useful maps
      if (oGridSetting.GetName().equals("Ripley's K") == false &&
      		oGridSetting.GetName().equals("Relative Neighborhood Density") == false &&
      		oGridSetting.GetName().equals("Harvest Results") == false &&
      		oGridSetting.GetName().equals("Partitioned Biomass") == false &&
      		oGridSetting.GetName().equals("Foliar Chemistry") == false &&
      		oGridSetting.GetName().equals("Storm Killed Partitioned Biomass") == false &&
      		oGridSetting.GetName().equals("Mortality Episode Results") == false) {

        //Go through the floats - we could histogram any of them
        for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {

          sLabel = oGridSetting.GetFloat(j).GetCodeName();

          //Make sure we haven't already added this float
          bFound = false;
          for (k = 0; k < oOptions.size(); k++) {
            sTestLabel = (String) oOptions.elementAt(k);
            if (sLabel.equals(sTestLabel)) {
              bFound = true;
              break;
            }
          }
          if (!bFound) {
            oOptions.add("Grid: " + oGridSetting.GetName() + " - " + sLabel);
            oIsFloat.add(new Boolean(true));
            oDeadCode.add(OutputBehaviors.NOTDEAD);
          }
        }
        //Go through the ints - we could histogram any of them
        for (j = 0; j < oGridSetting.GetNumberOfInts(); j++) {

          sLabel = oGridSetting.GetInt(j).GetCodeName();

          //Make sure we haven't already added this int
          bFound = false;
          for (k = 0; k < oOptions.size(); k++) {
            sTestLabel = (String) oOptions.elementAt(k);
            if (sLabel.equals(sTestLabel)) {
              bFound = true;
              break;
            }
          }
          if (!bFound) {
            oOptions.add("Grid: " + oGridSetting.GetName() + " - " + sLabel);
            oIsFloat.add(new Boolean(false));
            oDeadCode.add(OutputBehaviors.NOTDEAD);
          }
        }
      }
    }

    //Transform our Vector into an array of strings
    if (oOptions.size() == 0) {
      return null;
    }
    mp_sHistogramNames = new String[oOptions.size()];
    mp_iDeadCodes = new int[oOptions.size()];
    mp_bHistogramIsFloat = new boolean[oOptions.size()];
    for (i = 0; i < mp_sHistogramNames.length; i++) {
      mp_sHistogramNames[i] = oOptions.elementAt(i);
      mp_bHistogramIsFloat[i] = oIsFloat.elementAt(i).booleanValue();
      mp_iDeadCodes[i] = oDeadCode.elementAt(i).intValue();
    }
    
    //Add the name "Histogram" to each name
    for (i = 0; i < mp_sHistogramNames.length; i++) {
      if (OutputBehaviors.NOTDEAD == mp_iDeadCodes[i]) {
        mp_sHistogramNames[i] = mp_sHistogramNames[i] + " Histogram";  
      } else {
        mp_sHistogramNames[i] = mp_sHistogramNames[i] + " " + 
        mp_sDeadCodeNames[mp_iDeadCodes[i]] + " Dead Histogram";
      }
      
    }

    return mp_sHistogramNames;
  }

  /**
   * Gets the maps that can be drawn for this file.
   * @return String of maps, or NULL if no maps can be drawn.
   */
  public String[] GetMapOptions() {
    //This will hold all our options as we collect them
    Vector<String> oOptions = new Vector<String>(0);
    DetailedTreeSettings oTreeSetting;
    DetailedGridSettings oGridSetting;
    int i, j;

    //Can we do X, Y, and DBH?
    boolean bHasX = false, bHasY = false, bHasDBH = false, bHasCrownRad = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("x")) bHasX = true;
        else if (sLabel.equalsIgnoreCase("y")) bHasY = true;
        else if (sLabel.equalsIgnoreCase("dbh")) bHasDBH = true;
      }

      if (bHasX && bHasY && bHasDBH) {
        oOptions.add("XY DBH Tree Map");
        break;
      } else {
        bHasX = false;
        bHasY = false;
        bHasDBH = false;
      }
    }
    //Can we do X, Y, and crown radius?
    bHasX = false; bHasY = false; bHasDBH = false; bHasCrownRad = false;
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("x")) bHasX = true;
        else if (sLabel.equalsIgnoreCase("y")) bHasY = true;
        else if (sLabel.equalsIgnoreCase("crown radius")) bHasCrownRad = true;
      }
      if (bHasX && bHasY && bHasCrownRad) {
        oOptions.add("Crown Radius Tree Map");
        break;
      } else {
        bHasX = false;
        bHasY = false;
        bHasCrownRad = false;
      }
    }

    //We can map any non-char, non-package grid data members
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oGridSetting = (DetailedGridSettings) mp_oGridSettings.
          elementAt(i);
      String sGridName = oGridSetting.GetName();
      
      //Skip some grids, because they just swamp the options list without 
      //producing useful maps
      if (sGridName.equals("Ripley's K") == false &&
      		oGridSetting.GetName().equals("Relative Neighborhood Density") == false &&
      		oGridSetting.GetName().equals("Harvest Results") == false &&
      		oGridSetting.GetName().equals("Mortality Episode Results") == false &&
      		oGridSetting.GetName().equals("Partitioned Biomass") == false &&
      		oGridSetting.GetName().equals("Foliar Chemistry") == false &&
      		oGridSetting.GetName().equals("Storm Killed Partitioned Biomass") == false) {
        for (j = 0; j < oGridSetting.GetNumberOfBools(); j++) {
          String sLabel = oGridSetting.GetBool(j).GetCodeName();
          oOptions.add("Map - " + sGridName + " - " + sLabel);
        }
        for (j = 0; j < oGridSetting.GetNumberOfInts(); j++) {
          String sLabel = oGridSetting.GetInt(j).GetCodeName();
          oOptions.add("Map - " + sGridName + " - " + sLabel);
        }
        for (j = 0; j < oGridSetting.GetNumberOfFloats(); j++) {
          String sLabel = oGridSetting.GetFloat(j).GetCodeName();
          oOptions.add("Map - " + sGridName + " - " + sLabel);
        }
      }
    }

    if (oOptions.size() == 0) {
      return null;
    }

    String[] p_sOptions = new String[oOptions.size()];
    for (i = 0; i < p_sOptions.length; i++) {
      p_sOptions[i] = (String) oOptions.elementAt(i);
    }

    return p_sOptions;

  }

  /**
   * Gets the line graphs that can be drawn for this file.
   * @return String of tables, or NULL if no line graphs can be drawn.
   */
  public String[] GetLineGraphOptions() {
    Vector<String> p_oOptions = new Vector<String>(0);
    DetailedTreeSettings oTreeSetting;
    DetailedGridSettings oGridSetting;
    int i, j;
    boolean[] p_bSeedling = new boolean[OutputBehaviors.NUMCODES],
              p_bSaplingDiam = new boolean[OutputBehaviors.NUMCODES], 
              p_bAdultDiam = new boolean[OutputBehaviors.NUMCODES],
              p_bSnagDiam = new boolean[OutputBehaviors.NUMCODES], 
              p_bSaplingVolume = new boolean[OutputBehaviors.NUMCODES], 
              p_bAdultVolume = new boolean[OutputBehaviors.NUMCODES],
              p_bSnagVolume = new boolean[OutputBehaviors.NUMCODES], 
              p_bSaplingBiomass = new boolean[OutputBehaviors.NUMCODES], 
              p_bAdultBiomass = new boolean[OutputBehaviors.NUMCODES],
              p_bSnagBiomass = new boolean[OutputBehaviors.NUMCODES];
    
    for (i = 0; i < OutputBehaviors.NUMCODES; i++) {
      p_bSeedling[i] = false;
      p_bSaplingDiam[i] = false; 
      p_bAdultDiam[i] = false;
      p_bSnagDiam[i] = false; 
      p_bSaplingVolume[i] = false; 
      p_bAdultVolume[i] = false;
      p_bSnagVolume[i] = false; 
      p_bSaplingBiomass[i] = false; 
      p_bAdultBiomass[i] = false;
      p_bSnagBiomass[i] = false;
    }

    //Check for which tree types we have diameter information
    for (i = 0; i < mp_oTreeSettings.size(); i++) {
      oTreeSetting = (DetailedTreeSettings) mp_oTreeSettings.
          elementAt(i);
      for (j = 0; j < oTreeSetting.GetNumberOfFloats(); j++) {
        String sLabel = oTreeSetting.GetFloat(j).GetCodeName();
        if (sLabel.equalsIgnoreCase("dbh")) {
          if (oTreeSetting.GetType() == javawrapper.TreePopulation.SAPLING) {
            p_bSaplingDiam[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.ADULT) {
            p_bAdultDiam[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.SNAG) {
            p_bSnagDiam[oTreeSetting.GetDeadCode()] = true;
          }
        }
        else if (sLabel.equalsIgnoreCase("diam10") &&
                 oTreeSetting.GetType() == javawrapper.TreePopulation.SEEDLING) {
          p_bSeedling[oTreeSetting.GetDeadCode()] = true;
        }
        else if (sLabel.equalsIgnoreCase("volume")) {
          if (oTreeSetting.GetType() == javawrapper.TreePopulation.SAPLING) {
            p_bSaplingVolume[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.ADULT) {
            p_bAdultVolume[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.SNAG) {
            p_bSnagVolume[oTreeSetting.GetDeadCode()] = true;
          }
        }
        else if (sLabel.equalsIgnoreCase("biomass")) {
          if (oTreeSetting.GetType() == javawrapper.TreePopulation.SAPLING) {
            p_bSaplingBiomass[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.ADULT) {
            p_bAdultBiomass[oTreeSetting.GetDeadCode()] = true;
          }
          else if (oTreeSetting.GetType() == javawrapper.TreePopulation.SNAG) {
            p_bSnagBiomass[oTreeSetting.GetDeadCode()] = true;
          }
        }
      }
    }

    if (p_bSeedling[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Whole plot: Seedling absolute density");
      p_oOptions.add("Whole plot: Seedling relative density");
    }
    if (p_bSaplingDiam[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Whole plot: Sapling absolute basal area");
      p_oOptions.add("Whole plot: Sapling relative basal area");
      p_oOptions.add("Whole plot: Sapling absolute density");
      p_oOptions.add("Whole plot: Sapling relative density");
    }
    if (p_bAdultDiam[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Whole plot: Adult absolute basal area");
      p_oOptions.add("Whole plot: Adult relative basal area");
      p_oOptions.add("Whole plot: Adult absolute density");
      p_oOptions.add("Whole plot: Adult relative density");
    }
    if (p_bSnagDiam[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Whole plot: Snag absolute basal area");
      p_oOptions.add("Whole plot: Snag relative basal area");
      p_oOptions.add("Whole plot: Snag absolute density");
      p_oOptions.add("Whole plot: Snag relative density");
    }
    if (p_bSaplingVolume[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Sapling absolute volume");
      p_oOptions.add("Sapling relative volume");
    }
    if (p_bAdultVolume[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Adult absolute volume");
      p_oOptions.add("Adult relative volume");
    }
    if (p_bSnagVolume[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Snag absolute volume");
      p_oOptions.add("Snag relative volume");
    }
    if (p_bSaplingBiomass[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Sapling absolute biomass");
      p_oOptions.add("Sapling relative biomass");
    }
    if (p_bAdultBiomass[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Adult absolute biomass");
      p_oOptions.add("Adult relative biomass");
    }
    if (p_bSnagBiomass[OutputBehaviors.NOTDEAD]) {
      p_oOptions.add("Snag absolute biomass");
      p_oOptions.add("Snag relative biomass");
    }
    for (i = 1; i < OutputBehaviors.NUMCODES; i++) {
      if (p_bSeedling[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Seedling Abs Den");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Seedling Rel Den");
      }
      if (p_bSaplingDiam[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Abs BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Rel BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Abs Den");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Rel Den");
      }
      if (p_bAdultDiam[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Abs BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Rel BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Abs Den");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Rel Den");
      }
      if (p_bSnagDiam[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Abs BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Rel BA");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Abs Den");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Rel Den");
      }
      if (p_bSaplingVolume[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Abs Volume");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Rel Volume");
      }
      if (p_bAdultVolume[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Abs Volume");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Rel Volume");
      }
      if (p_bSnagVolume[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Abs Volume");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Rel Volume");
      }
      if (p_bSaplingBiomass[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Abs Biomass");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Sapling Rel Biomass");
      }
      if (p_bAdultBiomass[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Abs Biomass");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Adult Rel Biomass");
      }
      if (p_bSnagBiomass[i]) {
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Abs Biomass");
        p_oOptions.add(mp_sDeadCodeNames[i] + " Dead Snag Rel Biomass");
      }
    }

    //Grid options
    for (i = 0; i < mp_oGridSettings.size(); i++) {
    	oGridSetting = (DetailedGridSettings) mp_oGridSettings.elementAt(i);
    	if (oGridSetting.GetName().equals("Ripley's K")) {
    		p_oOptions.add("Ripley's K");
    	} else if (oGridSetting.GetName().equals("Relative Neighborhood Density")) {
    		p_oOptions.add("Relative Neighborhood Density");
    	}
    	else if (oGridSetting.GetName().equals("Partitioned Biomass")) {
    		p_oOptions.add("Biomass - Leaf");
    		p_oOptions.add("Biomass - Branch");
    		p_oOptions.add("Biomass - Bole");
    	}
    	else if (oGridSetting.GetName().equals("Foliar Chemistry")) {
        p_oOptions.add("Foliar Chemistry - P");
        p_oOptions.add("Foliar Chemistry - N");
        p_oOptions.add("Foliar Chemistry - Lignin");
        p_oOptions.add("Foliar Chemistry - SLA");
        p_oOptions.add("Foliar Chemistry - Fiber");
        p_oOptions.add("Foliar Chemistry - Cellulose");
        p_oOptions.add("Foliar Chemistry - Tannins");
        p_oOptions.add("Foliar Chemistry - Phenolics");
      }
    }

    //Transfer the options list to an array
    if (p_oOptions.size() > 0) {
      String[] p_sOptions = new String[p_oOptions.size()];
      for (i = 0; i < p_sOptions.length; i++) {
        p_sOptions[i] = (String) p_oOptions.elementAt(i);
      }
      return p_sOptions;
    }
    return null;

  }

  /**
   * Sets the plot X length.
   * @param f Plot X length, in meters.
   */
  public void SetXPlotLength(float f) {
    m_fXPlotLength = f;
  }

  /**
   * Sets the plot Y length.
   * @param f Plot Y length, in meters.
   */
  public void SetYPlotLength(float f) {
    m_fYPlotLength = f;
  }

  /**
   * Sets the number of years per timestep.
   * @param f Number of years per timestep.
   */
  public void SetNumberOfYearsPerTimestep(float f) {
    m_fNumYearsPerTimestep = f;
  }

  /**
   * Gets the number of years per timestep.
   * @return Number of years per timestep.
   */
  public float GetNumberOfYearsPerTimestep() {
    return m_fNumYearsPerTimestep;
  }

  /**
   * Gets the plot X length.
   * @return Plot X length, in meters.
   */
  public float GetXPlotLength() {
    return m_fXPlotLength;
  }

  /**
   * Gets the plot Y length.
   * @return Plot Y length, in meters.
   */
  public float GetYPlotLength() {
    return m_fYPlotLength;
  }

  /**
   * Constructor.
   * @param sTarball  Detailed output file to manage.
   * @throws ModelException If there was a problem reading the file.
   */
  public DetailedOutputFileManager(DataVisualizerManager oManager, String sTarball) throws ModelException {
    super(oManager, sTarball);
    try {

      m_iLastTimestepRead = 0;

      //Put together the root temp directory - the application's directory
      //plus temp
      String sPath = System.getProperty("SORTIE_PATH");
      String sSeparator = System.getProperty("file.separator");
      if (sPath == null) {
        m_sTempRoot = sSeparator + "temp";
      }
      else {
        if (sPath.endsWith(sSeparator) == false) {
          sPath += sSeparator;
        }
        m_sTempRoot = sPath + "temp";
      }

      //Create a display name
      if (sTarball.length() > 20) {
        //Take the LAST 20
        String sText = sTarball.substring(sTarball.length() - 20);
        m_sFileDisplayName = "..." + sText;
      }
      else {
        m_sFileDisplayName = sTarball;
      }

      //Get the root filename
      m_sFileRoot = sTarball.substring(0, sTarball.indexOf(".gz.tar"));

      m_sTempDir = javawrapper.Tarball.ExtractTarball(m_sFilename, m_sTempRoot);

      //Create our parser
      //Create the parser this way - don't go through the SAXParserFactory!
      //Parsers created that way throw MalformedURLExceptions when asked to
      //read local files.
      m_oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");

      ParseParamFile();
      CountTimesteps();
      
      //Update area calculation
      if (m_fPlotArea < 0.0001) {
      	m_fPlotArea = (m_fXPlotLength * m_fYPlotLength)/10000;
      } else {
      	//This was a subplot and area is in square meters - transform to hectares
      	m_fPlotArea /= 10000;
      }

      //Create the legend
      String[] sSpeciesNames = new String[mp_sSpeciesNames.size()];
      for (int i = 0; i < mp_sSpeciesNames.size(); i++) {
        sSpeciesNames[i] = (String) mp_sSpeciesNames.elementAt(i);
      }

      String sFileName = new java.io.File(m_sFilename).getName();
      m_oLegend = new DetailedOutputLegend(this, sFileName, sSpeciesNames,
                                           m_iNumTimesteps);

    }
    catch (SAXException e) {
      throw(new ModelException(ErrorGUI.UNKNOWN, "JAVA",
      "There were problems constructing the XML parser."));
    }
  }

  /**
   * Redraws all the charts currently open for this detailed output file.  This
   * is accomplished by sending an UpdateChart() call to each data request
   * object.
   * @throws ModelException wrapping another exception.
   */
  public void UpdateCharts() throws ModelException {
    DataRequest oRequest;
    int i;
    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oRequest.UpdateChart(m_oLegend);
    }
  }

  /**
   * Adds a species name to the end of the species name list.  The position
   * within the vector becomes that name's species index code.
   * @param sName The species name to add.
   */
  public void AddSpeciesName(String sName) {
    mp_sSpeciesNames.add(sName);
  }

  /**
   * Creates a new DetailedTreeSettings object and adds it to the
   * tree settings vector.
   * @param sSpeciesName The species name string
   * @param iType The type number
   * @return The new object created
   * @throws ModelException if either the type or the species name is invalid.
   */
  public DetailedTreeSettings CreateNewTreeSettings(String
      sSpeciesName, int iType, int iDeadCode) throws
      ModelException {
    String sName;
    int iSpecies = -1, i;

    //Validate the type
    if (iType < 0 || iType >= m_iNumTypes) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
      "Invalid type encountered while parsing detailed output file - " +
          String.valueOf(iType)));
    }

    //Try to find the species
    for (i = 0; i < mp_sSpeciesNames.size(); i++) {
      sName = (String) mp_sSpeciesNames.elementAt(i);
      if (sName.equals(sSpeciesName)) {
        iSpecies = i;
        break;
      }
    }

    //Throw an error if we didn't find the name
    if (iSpecies == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
      "Invalid species name encountered while parsing detailed output file - " +
          sSpeciesName));
    }
    
    if (iDeadCode < OutputBehaviors.NOTDEAD || iDeadCode >= OutputBehaviors.NUMCODES) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
          "Invalid dead reason code - " + String.valueOf(iDeadCode)));
    }

    DetailedTreeSettings oData = new DetailedTreeSettings(iType, iSpecies, iDeadCode);

    mp_oTreeSettings.add(oData);

    return oData;
  }

  /**
   * Creates a new DetailedGridSettings object and adds it to the
   * grid settings vector.
   * @param sGridName The grid name string
   * @return The new object created
   */
  public DetailedGridSettings CreateNewGridSettings(String
      sGridName) {

    DetailedGridSettings oData = new DetailedGridSettings(sGridName);
    mp_oGridSettings.add(oData);
    return oData;
  }

  /**
   * Adds the X cell length to grid settings.
   * @param sGridName The grid name for which to add the cell length.
   * @param fLength The new X cell length in m.
   */
  public void AddGridXCellLength(String sGridName, float fLength) {

    //Go through our grids and find this one
    DetailedGridSettings oSettings;
    int i;
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals(sGridName)) {
        oSettings.SetXCellLength(fLength);
      }
    }
  }

  /**
   * Adds the Y cell length to grid settings.
   * @param sGridName The grid name for which to add the cell length.
   * @param fLength The new Y cell length in m.
   */
  public void AddGridYCellLength(String sGridName, float fLength) {

    //Go through our grids and find this one
    DetailedGridSettings oSettings;
    int i;
    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (oSettings.GetName().equals(sGridName)) {
        oSettings.SetYCellLength(fLength);
      }
    }
  }

  /**
   * Counts the number of timesteps in the tarball and places it in
   * m_iNumTimesteps.  The value is found by counting the number of
   * files.  The number of timesteps is the number of files - 2 (for the
   * parameter file and initial conditions).  This is compared to the
   * number of timesteps found in the header file and the lesser one is chosen.
   * @throws ModelException - wrapping other exceptions
   */
  private void CountTimesteps() throws ModelException {
    FileInputStream oFileStream = null;
    TarInputStream oTarballStream = null;
    try {
      TarEntry oTarEntry;

      int iCountedTimesteps;

      //Count the number of files in the tarball - it should be number of
      //timesteps plus 2 (for detailed output header and initial conditions)
      iCountedTimesteps = 0;

      oFileStream = new FileInputStream(m_sFilename);
      oTarballStream = new TarInputStream(oFileStream);
      oTarEntry = oTarballStream.getNextEntry();

      while (oTarEntry != null) {
        iCountedTimesteps++;
        oTarEntry = oTarballStream.getNextEntry();
      }

      if (iCountedTimesteps < m_iNumParFileTimesteps + 2) {
        m_iNumTimesteps = iCountedTimesteps - 2;
      }
      else {
        m_iNumTimesteps = m_iNumParFileTimesteps;
      }

    }
    catch (IOException e) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "DoDetailedOutputSetup",
                               m_sFilename));
    }
    finally {
      try {
        if (null != oTarballStream) {
          oTarballStream.close();
        }
        if (null != oFileStream) {
          oFileStream.close();
        }
      }
      catch (java.io.IOException e) {
        ; //do nothing
      }
    }
  }

  /**
   * Parses the parameter file portion of the tarball.  This function parses
   * the detailed output header file to find species information and tree setup
   * information.
   * @throws ModelException if the file cannot be parsed.
   */
  private void ParseParamFile() throws ModelException {
    InputStream oInputStream = null;
    String sFileToRead = "";
    try {
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new DetailedOutputFileSetupParseHandler(this);
      m_oParser.setContentHandler(oHandler);

      //Create the string which has the filename of the detailed output file
      sFileToRead = m_sFileRoot + ".xml.gz";
      //Trim off path and add temp directory
      sFileToRead = sFileToRead.substring(sFileToRead.lastIndexOf("\\") + 1);
      sFileToRead = m_sTempDir + sFileToRead;

      //Parse the file
      oInputStream = javawrapper.Tarball.UnzipFile(sFileToRead);
      InputSource oToParse = new InputSource(oInputStream);

      m_oParser.parse(oToParse);
    }
    catch (IOException e) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "DoDetailedOutputSetup",
                              sFileToRead));

    }
    catch (SAXException e) {
      throw(new ModelException(ErrorGUI.BAD_XML_FILE, "JAVA", e.getMessage()));
    }
    finally {
      if (oInputStream != null) {
        try {
          oInputStream.close();
        }
        catch (java.io.IOException e) {
          ; //do nothing
        }
      }
    }
  }

  /**
   * Gets the number of species that exist in the detailed output header file.
   * @return The number of species.
   */
  public int GetNumberOfSpecies() {
    return mp_sSpeciesNames.size();
  }

  /**
   * Gets the number of tree types.
   * @return The number of types.
   */
  public int GetNumberOfTypes() {
    return m_iNumTypes;
  }

  /**
   * When passed the name of a species, will give back the corresponding
   * species number code.
   * @param sName The name of the species.
   * @return The code.
   * @throws ModelException if the name is not recognized.
   */
  public int GetSpeciesCodeFromName(String sName) throws ModelException {
    int i;

    for (i = 0; i < mp_sSpeciesNames.size(); i++) {
      if (sName.equalsIgnoreCase( (String) mp_sSpeciesNames.elementAt(i))) {
        return i;
      }
    }
    throw(new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                             "Unrecognized species " + sName +
                             " passed to detailed output file manager for " +
                             m_sFilename));
  }

  /**
   * When passed the index of a species, will give back the corresponding
   * species name.
   * @param iIndex Species index.
   * @return Species name.
   * @throws ModelException if the index is unrecognized.
   */
  public String GetSpeciesNameFromCode(int iIndex) throws ModelException {
    if (iIndex < 0 || iIndex >= mp_sSpeciesNames.size()) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                               "Unrecognized species index" +
                               String.valueOf(iIndex) +
                               " passed to detailed output file manager for " +
                               m_sFilename));
    }

    return (String) mp_sSpeciesNames.elementAt(iIndex);
  }

  /**
   * Read the detailed output file for a particular timestep.
   * @param iTimestep Timestep for which to read the file.
   * @throws ModelException If the file cannot be read.
   */
  public void ReadFile(int iTimestep) throws ModelException {
    InputStream oInputStream = null;
    InputSource oToParse = null;
    String sFileToRead = "";
    try {

      m_iLastTimestepRead = iTimestep;

      //Tell the open data requests to clear all their old data -
      //and remove any requests that no longer have charts open
      DataRequest oDataRequest;
      int i;
      for (i = 0; i < mp_oDataRequests.size(); i++) {
        oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
        if (oDataRequest.m_oChartFrame != null &&
            oDataRequest.m_oChartFrame.isVisible() == false) {
          mp_oDataRequests.remove(i);
          i--;
        }
        else {
          oDataRequest.ClearData();
        }
      }

      //Create the string which has the filename of the detailed output file
      sFileToRead = m_sFileRoot + "_" + String.valueOf(iTimestep) + ".xml.gz";
      //Trim off path and add temp directory
      sFileToRead = sFileToRead.substring(sFileToRead.lastIndexOf("\\") + 1);
      sFileToRead = m_sTempDir + sFileToRead;

      //Now install the detailed output timestep handler
      DefaultHandler oHandler = new DetailedOutputTimestepParseHandler(this);
      m_oParser.setContentHandler(oHandler);

      //Parse the file
      try {
        oInputStream = javawrapper.Tarball.UnzipFile(sFileToRead);
        oToParse = new InputSource(oInputStream);
      }  catch (Exception e) {
        if (e instanceof java.io.FileNotFoundException) {
          if (m_oManager.GetModelState() == MainWindowStateSetter.PAUSED) {
            ; //fail silently to avoid interrupting the run
          } else {
            throw(e);
          }
        } else {
          throw(e);
        }
      }  
      
        m_oParser.parse(oToParse);
    }
    catch (SAXException e) {
      throw(new ModelException(ErrorGUI.BAD_XML_FILE, "JAVA",
          "There were problems parsing " + sFileToRead));
    }
    catch (IOException e) {
      throw(new ModelException(ErrorGUI.BAD_FILE,
          "DetailedOutputFileManager.ReadFile",
          sFileToRead));
    }
    catch (Exception e) {
      throw(new ModelException(ErrorGUI.UNKNOWN,
          "DetailedOutputFileManager.ReadFile",
          e.getMessage()));
    }
    finally {
      if (oInputStream != null) {
        try {
          oInputStream.close();
        }
        catch (java.io.IOException e) {
          ; //do nothing
        }
      }
    }
  }
  
  /**
   * Whether or not any open data requests want any tree float data members.
   * @return True if float data members are desired, false if not.
   */
  public boolean WantAnyTreeFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }
  
  /**
   * Whether or not any open data requests want any dead tree float data members.
   * @return True if float data members are desired, false if not.
   */
  public boolean WantAnyDeadTreeFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyDeadTreeFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid float data members.
   * @return True if float data members are desired, false if not.
   */
  public boolean WantAnyGridFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid package float data
   * members.
   * @return True if float data members are desired, false if not.
   */
  public boolean WantAnyGridPackageFloats() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridPackageFloats();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }


  /**
   * Whether or not any open data requests want any tree int data members.
   * @return True if int data members are desired, false if not.
   */
  public boolean WantAnyTreeInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }
  
  /**
   * Whether or not any open data requests want any dead tree int data members.
   * @return True if int data members are desired, false if not.
   */
  public boolean WantAnyDeadTreeInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyDeadTreeInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid int data members.
   * @return True if int data members are desired, false if not.
   */
  public boolean WantAnyGridInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid package int data
   * members.
   * @return True if int data members are desired, false if not.
   */
  public boolean WantAnyGridPackageInts() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridPackageInts();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any tree char data members.
   * @return True if char data members are desired, false if not.
   */
  public boolean WantAnyTreeChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }
  
  /**
   * Whether or not any open data requests want any dead tree char data members.
   * @return True if char data members are desired, false if not.
   */
  public boolean WantAnyDeadTreeChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyDeadTreeChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid char data members.
   * @return True if char data members are desired, false if not.
   */
  public boolean WantAnyGridChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid package char data
   * members.
   * @return True if char data members are desired, false if not.
   */
  public boolean WantAnyGridPackageChars() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridPackageChars();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any tree bool data members.
   * @return True if bool data members are desired, false if not.
   */
  public boolean WantAnyTreeBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyTreeBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }
  
  /**
   * Whether or not any open data requests want any dead tree bool data members.
   * @return True if bool data members are desired, false if not.
   */
  public boolean WantAnyDeadTreeBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyDeadTreeBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid bool data members.
   * @return True if bool data members are desired, false if not.
   */
  public boolean WantAnyGridBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Whether or not any open data requests want any grid package bool data
   * members.
   * @return True if bool data members are desired, false if not.
   */
  public boolean WantAnyGridPackageBools() {
    DataRequest oRequest;
    int i;
    boolean bWants = false;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      bWants = oRequest.WantAnyGridPackageBools();
      if (bWants) {
        break;
      }
    }

    return bWants;
  }

  /**
   * Accepts a piece of tree float data from the parser and passes it on to
   * open data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param fVal Value.
   */
  public void AddTreeFloatData(int iSpecies, int iType, int iCode, float fVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeFloats()) {
        oDataRequest.AddTreeFloatData(iSpecies, iType, iCode, fVal);
      }
    }
  }
  
  /**
   * Accepts a piece of dead tree float data from the parser and passes it on to
   * open data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param iDeadCode Dead code for this tree.
   * @param fVal Value.
   */
  public void AddDeadTreeFloatData(int iSpecies, int iType, int iCode, 
      int iDeadCode, float fVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyDeadTreeFloats()) {
        oDataRequest.AddDeadTreeFloatData(iSpecies, iType, iCode, iDeadCode, fVal);
      }
    }
  }

  /**
   * Accepts a piece of grid float data from the parser and passes it on to
   * open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param fVal Value.
   */
  public void AddGridFloatData(String sGridName, int iX, int iY, int iCode,
                               float fVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridFloats()) {
        oDataRequest.AddGridFloatData(sGridName, iX, iY, iCode, fVal);
      }
    }
  }

  /**
   * Accepts a piece of grid package float data from the parser and passes it
   * on to open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param fVal Value.
   */
  public void AddGridPackageFloatData(String sGridName, int iX, int iY, int iCode,
                               float fVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridPackageFloats()) {
        oDataRequest.AddGridPackageFloatData(sGridName, iX, iY, iCode, fVal);
      }
    }
  }

  /**
   * Accepts a piece of tree int data from the parser and passes it on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param iVal Value.
   */
  public void AddTreeIntData(int iSpecies, int iType, int iCode, int iVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeInts()) {
        oDataRequest.AddTreeIntData(iSpecies, iType, iCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of dead tree int data from the parser and passes it on to 
   * open data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param iDeadCode Dead code for this tree.
   * @param iVal Value.
   */
  public void AddDeadTreeIntData(int iSpecies, int iType, int iCode, 
      int iDeadCode, int iVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyDeadTreeInts()) {
        oDataRequest.AddDeadTreeIntData(iSpecies, iType, iCode, iDeadCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of grid int data from the parser and passes it on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   */
  public void AddGridIntData(String sGridName, int iX, int iY, int iCode,
                             int iVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridInts()) {
        oDataRequest.AddGridIntData(sGridName, iX, iY, iCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of grid package int data from the parser and passes it on
   * to open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   */
  public void AddGridPackageIntData(String sGridName, int iX, int iY, int iCode,
                             int iVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridPackageInts()) {
        oDataRequest.AddGridPackageIntData(sGridName, iX, iY, iCode, iVal);
      }
    }
  }

  /**
   * Accepts a piece of tree char data from the parser and passes it on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param sVal Value.
   */
  public void AddTreeCharData(int iSpecies, int iType, int iCode, String sVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeChars()) {
        oDataRequest.AddTreeCharData(iSpecies, iType, iCode, sVal);
      }
    }
  }
  
  /**
   * Accepts a piece of dead tree char data from the parser and passes it on to 
   * open data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param iDeadCode Dead code for this tree.
   * @param sVal Value.
   */
  public void AddDeadTreeCharData(int iSpecies, int iType, int iCode, 
      int iDeadCode, String sVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyDeadTreeChars()) {
        oDataRequest.AddDeadTreeCharData(iSpecies, iType, iCode, iDeadCode, sVal);
      }
    }
  }

  /**
   * Accepts a piece of grid char data from the parser and passes it on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param sVal Value.
   */
  public void AddGridCharData(String sGridName, int iX, int iY, int iCode,
                              String sVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridChars()) {
        oDataRequest.AddGridCharData(sGridName, iX, iY, iCode, sVal);
      }
    }
  }

  /**
   * Accepts a piece of grid package char data from the parser and passes it on
   * to open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param sVal Value.
   */
  public void AddGridPackageCharData(String sGridName, int iX, int iY, int iCode,
                              String sVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridPackageChars()) {
        oDataRequest.AddGridPackageCharData(sGridName, iX, iY, iCode, sVal);
      }
    }
  }

  /**
   * Accepts a piece of tree bool data from the parser to pass on to open data
   * requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param bVal Value.
   */
  public void AddTreeBoolData(int iSpecies, int iType, int iCode, boolean bVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyTreeBools()) {
        oDataRequest.AddTreeBoolData(iSpecies, iType, iCode, bVal);
      }
    }
  }
  
  /**
   * Accepts a piece of dead tree bool data from the parser to pass on to open 
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param iDeadCode Dead code for this tree.
   * @param bVal Value.
   */
  public void AddDeadTreeBoolData(int iSpecies, int iType, int iCode, 
      int iDeadCode, boolean bVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyDeadTreeBools()) {
        oDataRequest.AddDeadTreeBoolData(iSpecies, iType, iCode, iDeadCode, bVal);
      }
    }
  }

  /**
   * Accepts a piece of grid bool data from the parser to pass on to open data
   * requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param bVal Value.
   */
  public void AddGridBoolData(String sGridName, int iX, int iY, int iCode,
                              boolean bVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridBools()) {
        oDataRequest.AddGridBoolData(sGridName, iX, iY, iCode, bVal);
      }
    }
  }

  /**
   * Accepts a piece of grid package bool data from the parser to pass on to
   * open data requests.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value and passes it on to open
   * data requests.
   * @param bVal Value.
   */
  public void AddGridPackageBoolData(String sGridName, int iX, int iY, int iCode,
                              boolean bVal) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      if (oDataRequest.WantAnyGridPackageBools()) {
        oDataRequest.AddGridPackageBoolData(sGridName, iX, iY, iCode, bVal);
      }
    }
  }

  /**
   * Accepts a tree float data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddTreeFloatDataMemberCode(int iSpecies, int iType, String sLabel,
                                         int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeFloatDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid float data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridFloatDataMemberCode(String sGridName, String sLabel,
                                         int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridFloatDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid package float data member code from the parser to pass on
   * to open data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridPackageFloatDataMemberCode(String sGridName, String sLabel,
                                         int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridPackageFloatDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts an int data member code from the parser to pass on to open data
   * requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddTreeIntDataMemberCode(int iSpecies, int iType, String sLabel,
                                       int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeIntDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts an int data member code from the parser to pass on to open data
   * requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridIntDataMemberCode(String sGridName, String sLabel,
                                       int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridIntDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid package int data member code from the parser to pass on to
   * open data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridPackageIntDataMemberCode(String sGridName, String sLabel,
                                       int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridPackageIntDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a tree char data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddTreeCharDataMemberCode(int iSpecies, int iType, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeCharDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid char data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridCharDataMemberCode(String sGridName, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridCharDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid package char data member code from the parser to pass on
   * to open data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridPackageCharDataMemberCode(String sGridName, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridPackageCharDataMemberCode(sGridName, sLabel, iCode);
    }
  }


  /**
   * Accepts a tree bool data member code from the parser to pass on to open
   * data requests.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddTreeBoolDataMemberCode(int iSpecies, int iType, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddTreeBoolDataMemberCode(iSpecies, iType, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid bool data member code from the parser to pass on to open
   * data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridBoolDataMemberCode(String sGridName, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridBoolDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Accepts a grid package bool data member code from the parser to pass on
   * to open data requests.
   * @param sGridName Name of the grid for this data
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridPackageBoolDataMemberCode(String sGridName, String sLabel,
                                        int iCode) throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = (DataRequest) mp_oDataRequests.elementAt(i);
      oDataRequest.AddGridPackageBoolDataMemberCode(sGridName, sLabel, iCode);
    }
  }

  /**
   * Notifies all open data requests that a new package has ended.
   */
  public void EndPackage() throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = mp_oDataRequests.elementAt(i);
      oDataRequest.EndPackage();
    }
  }
  
  /**
   * Notifies all open data requests that a tree record has ended.
   */
  public void EndTree() throws SAXException {
    DataRequest oDataRequest;
    int i;

    for (i = 0; i < mp_oDataRequests.size(); i++) {
      oDataRequest = mp_oDataRequests.elementAt(i);
      oDataRequest.EndTree();
    }
  }
  
  /**
   * Get the total number of detailed tree settings.
   * @return the total number of detailed tree settings.
   */
  public int GetNumberTreeSettings() {return mp_oTreeSettings.size();}
  
  /**
   * Access a detailed tree output settings record.
   * @param i Index of the desired record.
   * @return The record at position i.
   */
  public DetailedTreeSettings GetTreeSetting(int i) {
    return mp_oTreeSettings.elementAt(i);    
  }

  /**
   * Gets the length of X cells for a grid.
   * @param sGridName Grid for which to get the X cell length.
   * @return The X cell length, in m, or 0 if the grid is not recognized.
   */
  public float GetGridXCellLength(String sGridName) {
    DetailedGridSettings oSettings;
    float fLength = 0;
    int i;

    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (sGridName.equals(oSettings.GetName())) {
        fLength = oSettings.GetXCellLength();
      }
    }
    return fLength;
  }

  /**
   * Gets the length of Y cells for a grid.
   * @param sGridName Grid for which to get the Y cell length.
   * @return The Y cell length, in m, or 0 if the grid is not recognized.
   */
  public float GetGridYCellLength(String sGridName) {
    DetailedGridSettings oSettings;
    float fLength = 0;
    int i;

    for (i = 0; i < mp_oGridSettings.size(); i++) {
      oSettings = (DetailedGridSettings) mp_oGridSettings.elementAt(
          i);
      if (sGridName.equals(oSettings.GetName())) {
        fLength = oSettings.GetYCellLength();
      }
    }
    return fLength;
  }

  /**
   * Deletes all files from the tarball in the temp folder
   */
  public void CleanUp() {
    javawrapper.Tarball.CleanUp(m_sFilename, m_sTempDir, m_sTempRoot);
  }
  
  /**
   * Adds one of the cells of this subplot. This adds the area of the cell to
   * the total, taking into account odd-sized end cells.
   * @param iX X cell
   * @param iY Y cell
   */
  public void AddSubplotCell(int iX, int iY) {
  	float fXLength, fYLength;
  	int iNumXCells = (int)java.lang.Math.ceil(m_fXPlotLength / m_fSubplotXCellLength),
  	    iNumYCells = (int)java.lang.Math.ceil(m_fYPlotLength / m_fSubplotYCellLength); 
  	if (iX < iNumXCells - 1) {
      fXLength = m_fSubplotXCellLength; 
    } else {
      fXLength = m_fXPlotLength - (iX * m_fSubplotXCellLength); 
    }   
    if (iY < iNumYCells - 1) {
      fYLength = m_fSubplotYCellLength; 
    } else {
    	fYLength = m_fYPlotLength - (iY * m_fSubplotYCellLength); 
    }
    m_fPlotArea += (fXLength * fYLength);
  }
}
