package javawrapper;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.table.TableModel;

/**
 * Makes a window for editing tree allometry functions.
 * 
 * Copyright: Copyright (c) Charles D. Canham 2008
 * Company: Cary Institute of Ecosystem Studies
 * 
 * @author Lora E. Murphy
 * @version 1.0
 * 
 * <br>
 * Edit history: <br>
 * ------------------ <br>
 * February 27, 2008: Created (LEM)
 */
public class AllometryFunctionEditor extends JDialog implements ActionListener,
        EnhancedTableWindow {
  
  
  
  /**Help ID string*/
  private String m_sHelpID = "windows.edit_allometry_functions_window";
  
  /** Tree population */
  private TreePopulation m_oPop;
  
  /** Allometry object */
  private Allometry m_oAllom;
  
  /** Table holding the allometry function settings for display */
  EnhancedTable m_oTable;
  
  public AllometryFunctionEditor(JFrame jParent, TreePopulation oPop,
      Allometry oAllom) {
    super(jParent, "Edit Allometry Functions", true);
    
    m_oPop = oPop;
    m_oAllom = oAllom;
    
    //Help ID
    m_oPop.m_oManager.GetHelpBroker().enableHelpKey(this.getRootPane(), m_sHelpID, null);
    
    Object[][] p_oTableData = null;
    Object[] p_oHeader;
    boolean[] p_bAppliesTo = new boolean[m_oPop.GetNumberOfSpecies()];
    int i;
    
    for (i = 0; i < p_bAppliesTo.length; i++) {
      p_bAppliesTo[i] = true;
    }
    
    // Assemble the allometry functions data
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatAdultHDFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatSaplingHDFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatSeedlingHDFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatAdultCRDFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatSaplingCRDFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatAdultCDHFunction);
    p_oTableData = m_oAllom.FormatDataForTable(p_oTableData, m_oAllom.mp_iWhatSaplingCDHFunction);
    
    p_oHeader = m_oAllom.FormatSpeciesHeaderRow(p_bAppliesTo, m_oPop);
    
    // Create the table from the data
    m_oTable = new EnhancedTable(p_oTableData, p_oHeader, this, 
        "Allometry Functions");
     
    // Create a table panel with an empty panel in CENTER so that the
    // table won't automatically fill the space
    JPanel jTablePanel = new JPanel(new BorderLayout());
    jTablePanel.setName("Table Panel");
    jTablePanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTablePanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jTablePanel.add(m_oTable, BorderLayout.LINE_START);
    jTablePanel.add(new JPanel(), BorderLayout.CENTER);

    // Assemble the window
    JPanel jContentPanel = (JPanel) getContentPane();
    jContentPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    getContentPane().setLayout(new java.awt.BorderLayout());
    getContentPane().add(jTablePanel, java.awt.BorderLayout.CENTER);
    getContentPane().add(new OKCancelButtonPanel(this,
                                                 m_oPop.m_oManager.GetHelpBroker(),
                                                 m_sHelpID),
                         java.awt.BorderLayout.PAGE_END);
  }
  

  /**
   * Controls actions for this window.
   * @param oEvent ActionEvent.
   */
  public void actionPerformed(ActionEvent oEvent) {
    String sCommand = oEvent.getActionCommand();
    if (sCommand.equals("Cancel")) {
      this.setVisible(false);
      this.dispose();
    } else if (sCommand.equals("OK")) {
      
      try {
      
        //Extract the data and feed it back to allometry
        Vector<TableData> p_oData = new Vector<TableData>(1);
        TableModel jModel = m_oTable.getModel();
        Object[][] p_oTableData = new Object[jModel.getRowCount()][
            jModel.getColumnCount()];
        int iRow, iCol;
        for (iRow = 0; iRow < jModel.getRowCount(); iRow++) {
          for (iCol = 0; iCol < jModel.getColumnCount(); iCol++) {
            p_oTableData[iRow][iCol] = jModel.getValueAt(iRow, iCol);
          }
        }
        Object[] p_oHeader = new Object[m_oTable.getColumnCount()];
        for (iCol = 0; iCol < p_oHeader.length; iCol++) {
          p_oHeader[iCol] = m_oTable.getColumnModel().getColumn(iCol).
              getHeaderValue();
        }

        p_oData.add(new TableData(p_oHeader, p_oTableData));
        m_oAllom.ReadDataFromDisplay(p_oData, m_oPop);
        this.setVisible(false);
        this.dispose();   
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }  
  }
  
  /**
   * Sets a table as last touched by the user.
   * @param oTable Table to be set as last touched.
   */
  public void SetLastTouched(EnhancedTable oTable) {
    oTable.SetLastTouched(true);
  }
}
