package javawrapper;

import java.io.BufferedWriter;
import java.util.Vector;

/**
* This class functions as an organizer for model-level behaviors.  Each object
* of this class manages a group of similar behavior objects.
*
* This is the behavior level of which the GUIManager is aware.  It keeps a
* known set of these objects around, which can respond to a common set of
* requests, and relies on them to manage the complexity of the individual
* behaviors which they own.
* <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>January 23, 2007: Added ChangeOfSpecies for updating behavior
* assignments (LEM)
* <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
*/

abstract class BehaviorTypeBase
    extends WorkerBase {

  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @param sName Name of this object to display to the user.
   * @param sXMLTag Parent XML tag for all of this object's data.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public BehaviorTypeBase(GUIManager oManager, String sName, String sXMLTag) {
    super(oManager, sName, sXMLTag);
    //m_iGridParseIndex = -1;
  }

  /**
   * Gets which species should have values for a given ModelVector.  This polls
   * all behaviors; if a behavior is enabled, and this ModelVector is on its
   * required data list, all species for which it is enabled are set to true.
   * @param p_oData The ModelVector to check.
   * @param oPop TreePopulation object.
   * @return A list of flags, one per species, that indicate for each whether
   * p_oData should have a value for that species.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean[] GetWhichSpeciesUsed(ModelVector p_oData, TreePopulation oPop) {
    ModelData oCompare;
    boolean[] p_bReturnFlags = new boolean[oPop.GetNumberOfSpecies()];
    String sName = p_oData.GetDescriptor();
    int i, j;
    for (i = 0; i < p_bReturnFlags.length; i++) {
      p_bReturnFlags[i] = false;
    }

    //Now go through each behavior
    if (mp_oChildBehaviors == null) {
      return p_bReturnFlags;
    }

    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled) {
        for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
          if (mp_oChildBehaviors[i].mp_oRequiredData.elementAt(j) instanceof
              ModelData) {
            oCompare = (ModelData) mp_oChildBehaviors[i].mp_oRequiredData.
                elementAt(j);
            if (oCompare.GetDescriptor().equals(sName)) {
              p_bReturnFlags = MergeBooleans(p_bReturnFlags,
                                             mp_oChildBehaviors[i].
                                             GetWhichSpeciesUsed(oPop));
              break;
            }
          }
        }
      }
    }
    return p_bReturnFlags;
  }

  /**
   * Determines if any of the child behaviors are currently enabled.
   * @return True if there are any child behaviors enabled.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean AnyBehaviorsEnabled() {
    int i;
    boolean bAnybodyEnabled = false;

    if (mp_oChildBehaviors != null) {
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        if (mp_oChildBehaviors[i].m_bIsEnabled) {
          bAnybodyEnabled = true;
          break;
        }
      }
    }

    return bAnybodyEnabled;
  }

  /**
   * Finds a behavior by its XML tag.
   * @param sXMLTag String The XML tag for which to find a behavior.
   * @return Behavior Behavior for the XML tag, or NULL if none of the
   * behaviors has that tag.
   */
  public Behavior GetBehaviorByXMLTag(String sXMLTag) {
    int j;
    if (mp_oChildBehaviors != null) {

      for (j = 0; j < mp_oChildBehaviors.length; j++) {

        if (mp_oChildBehaviors[j].GetParameterFileBehaviorName().equals(sXMLTag)) {
          return mp_oChildBehaviors[j];
        }
      }
    }
    return null;
  }

  /**
   * List of child behaviors a behavior group is managing.  They are kept in
   * the order in which they are arranged for the run.  Notice that this
   * array is not created in this class.  Also note that you can't count on the
   * order remaining constant, so to work with behaviors, use a
   * GetBehaviorBy* method.
   */
  protected Behavior[] mp_oChildBehaviors;

  /**
   * List of grid objects associated with these behaviors - notice this
   * array is not created in this class
   */
//  protected Grid[] mp_oGrids;

//  protected int
      /**When parsing grid maps, this is the index of the current map (or -1
       * if the current map is not owned by this behavior).*/
//      m_iGridParseIndex,
      /**When parsing packages, this is the index of the current package (or
       * -1 if the current map is not owned by this behavior).*/
//      m_iPackageIndex,
      /**When parsing grid maps, the current X grid cell*/
//      m_iCurrentXCell,
      /**When parsing grid maps, the current Y grid cell*/
//      m_iCurrentYCell;
  /**
   * Gets the list of grid objects which the currently enabled behaviors would
   * be expected to create - and thus would be available to output, etc.
   * @return The list of grids enabled, or null if none are enabled.
   * @throws ModelException passing through from called methods.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Grid[] GetEnabledGridObjects() throws ModelException {
    if (mp_oChildBehaviors == null) {
      return null;
    }

    Vector<Grid> oGrids = new Vector<Grid>(0);
    Grid oExistingGrid, oNewGrid;
    int i, j, k;
    boolean bAlreadyOnList;

    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled) {
        for (j = 0; j < mp_oChildBehaviors[i].GetNumberOfGrids(); j++) {
          oNewGrid = mp_oChildBehaviors[i].GetGrid(j);
          //Make sure each grid is not already on the list
          bAlreadyOnList = false;
          for (k = 0; k < oGrids.size(); k++) {
            oExistingGrid = (Grid) oGrids.elementAt(k);
            if (oExistingGrid.GetName().equals(oNewGrid.GetName())) {
              bAlreadyOnList = true;
              break;
            }
          }
          if (!bAlreadyOnList) {
            oGrids.add(oNewGrid);
          }
        }
      }
    }

    if (oGrids.size() == 0) {
      return null;
    }

    Grid[] p_oGrids = new Grid[oGrids.size()];
    for (i = 0; i < p_oGrids.length; i++) {
      p_oGrids[i] = (Grid) oGrids.elementAt(i);
    }

    return p_oGrids;
  }

  /**
   * Returns all grid objects, whether they are used or not.
   * @return All grid objects for a behavior grouping, or null if there are
   * none.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
/*  public Grid[] GetAllGridObjects() {
    return mp_oGrids;
  } */

  /**
   * Get all Behavior objects for this group.
   * @return The array of Behavior objects.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Behavior[] GetAllBehaviors() {
    return mp_oChildBehaviors;
  }

  /**
   * Writes behavior tags to the parameter file.  For all enabled behaviors,
   * writes them in order along with their appropriate applyTo tags.
   * @param out File stream to write to.
   * @param oPop Tree population object.
   * @throws ModelException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  void WriteBehaviorNodes(BufferedWriter out, TreePopulation oPop) throws
      ModelException {
    try {
      int i, j; //loop counters
      if (null != mp_oChildBehaviors) {
        for (i = 0; i < mp_oChildBehaviors.length; i++) {
          if (mp_oChildBehaviors[i].m_bIsEnabled) {

            //Write out the behavior tag
            out.write("<behavior><behaviorName>");
            out.write(mp_oChildBehaviors[i].GetParameterFileBehaviorName());
            out.write("</behaviorName><version>");
            out.write(String.valueOf(mp_oChildBehaviors[i].GetVersion()));
            out.write("</version>");

            //Now write the appliesTo tags
            for (j = 0; j < mp_oChildBehaviors[i].GetNumberOfCombos(); j++) {
              //When writing the species tags, replace spaces with underscores;
              //the XML parser can't handle spaces in attributes
              out.write("<applyTo species=\"");
              out.write(oPop.GetSpeciesNameFromCode(mp_oChildBehaviors[i].
                  GetSpeciesTypeCombo(j).
                  GetSpecies()).replace(' ', '_'));
              out.write("\" type=\"");
              out.write(TreePopulation.GetTypeNameFromCode(mp_oChildBehaviors[i].
                                                 GetSpeciesTypeCombo(j).
                                                 GetType()));
              out.write("\"/>");
            }

            //Now closing behavior tag
            out.write("</behavior>");
          }
        }
      }
    }
    catch (java.io.IOException e) {
      throw( new ModelException(ErrorGUI.BAD_FILE, "JAVA",
             "There was a problem writing the parameter file."));
    }
  }

  /**
   * Writes grid maps for any grid that has values.  If there are no grids,
   * or if there are no grid map values in any grid, nothing is written.
   * @param jOut File to write to.
   * @throws ModelException if there is a problem writing to the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
/*  public void WriteGridXML(BufferedWriter jOut) throws ModelException {

    if (mp_oGrids != null) {
      Plot oPlot = m_oManager.GetPlot();
      int i;
      for (i = 0; i < mp_oGrids.length; i++) {
        mp_oGrids[i].WriteXML(jOut, oPlot);
      }
    }
  } */

  /**
   * Clears all grid map values for this behavior's grids.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
/*  public void ClearGridMapValues() {
    if (mp_oGrids != null) {
      int i;
      for (i = 0; i < mp_oGrids.length; i++) {
        mp_oGrids[i].ClearMapValues();
      }
    }
  } */

  /**
   * Sets which species for a given type use a particular behavior.
   * <p>
   * This method would normally not be used, as this object manages its own
   * data and would make the decision of which behaviors are enabled and which
   * are not.  However, in the case of input of a tab-delimited parameter file,
   * the information must be fed to this class.
   * @param p_iVals Array of use flags, one per species.  A 0 value means that
   * the given species doesn't use the behavior, a non-zero value means they
   * do.
   * @param iType Tree type to which the species list applies.
   * @param oPop Tree population object for species/type validation.
   * @throws ModelException if the type number is invalid, or if not all
   * species are represented.
   * @param sKey Key string of behavior to be set.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected void SetUsesBehavior(Integer[] p_iVals, int iType,
                                 TreePopulation oPop, String sKey) throws
      ModelException {
    //Make sure the size of the array matches the number of species
    if (p_iVals.length != oPop.GetNumberOfSpecies()) {
      throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                "Data not provided for all species."));
    }

    //Find behavior
    Behavior oBeh = GetBehaviorByKey(sKey);
    int j;
    boolean bUsed;
    if (null != oBeh) {

      //Clear all combos with this type in it
      for (j = 0; j < oBeh.GetNumberOfCombos(); j++) {

        if (oBeh.GetSpeciesTypeCombo(j).GetType() == iType) {
          oBeh.DeleteSpeciesTypeCombo(j);
          j = -1; //We changed the size of the combo list - start over
        }
      }

      //Verify that one of the use flags is non-zero - otherwise there's
      //nothing do do here
      bUsed = false;
      for (j = 0; j < p_iVals.length; j++) {
        if (p_iVals[j].compareTo(new Integer(0)) > 0) {
          bUsed = true;
        }
      }
      if (bUsed) {
        oBeh.m_bIsEnabled = true;
        for (j = 0; j < p_iVals.length; j++) {
          if (p_iVals[j].compareTo(new Integer(0)) > 0) {
            oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(j, iType, oPop));
          }
        }
      }
      else {
        oBeh.m_bIsEnabled = false;
      }
    }
  }

  /**
   * Convenience method for getting a behavior.
   * @param sKey The short key name of the behavior.
   * @return The behavior if found, null if not.
   */
  public Behavior GetBehaviorByKey(String sKey) {
    int i;
    if (mp_oChildBehaviors == null) {
      return null;
    }
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].GetKey() == sKey) {
        return mp_oChildBehaviors[i];
      }
    }
    return null;
  }

  /**
   * Convenience method for getting a behavior.
   * @param sDescriptor The descriptive name of the behavior.
   * @return The behavior if found, null if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Behavior GetBehaviorByDisplayName(String sDescriptor) {
    int i;
    if (mp_oChildBehaviors == null) {
      return null;
    }
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].GetDescriptor() == sDescriptor) {
        return mp_oChildBehaviors[i];
      }
    }
    return null;
  }

  /**
   * Writes a set of species-specific float values to a parameter file.  The
   * data is only written if it is required for a behavior that is enabled; and
   * then only for the species required by any behavior for which it is
   * required.
   * @param out The file to write to.
   * @param p_oData The vector of data pieces.
   * @param oPop Tree population object to get species names
   * @throws java.io.IOException Passes on exceptions from FileWriter
   * @throws ModelException Passes on from TreePopulation
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteSpeciesSpecificValue(BufferedWriter out, ModelVector p_oData,
                                        TreePopulation oPop) throws
      java.io.IOException, ModelException {

    if (p_oData == null || p_oData.GetValue().size() == 0) {
      return;
    }

    //Determine which species for which this is enabled
    boolean[] p_bUse;
    if (p_oData.GetMustApplyToAllSpecies()) {
      p_bUse = new boolean[oPop.GetNumberOfSpecies()];
      for (int i = 0; i < p_bUse.length; i++) {
        p_bUse[i] = true;
      }
    }
    else {
      p_bUse = GetWhichSpeciesUsed(p_oData, oPop);
    }

    super.WriteSpeciesSpecificValue(out, p_oData, oPop, p_bUse);
  }

  /**
   * Writes the XML data to a parameter file for the behaviors owned by this
   * object.  Overridden so that only the data enabled by behaviors is written.
   * @param jOut File stream to write to.
   * @param oPop TreePopulation object.
   * @throws ModelException if there is a problem writing the file or
   * validating the data.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteXML(BufferedWriter jOut, TreePopulation oPop) throws
      ModelException {

    //Make sure a behavior is enabled - if not, return
    if (!AnyBehaviorsEnabled()) {
      return;
    }

    super.WriteXML(jOut, oPop);
  }

  /**
   * Determines whether a piece of data is required for any enabled behaviors
   * in the mp_oChildBehaviors list.
   * @param oData Piece of data to check.
   * @return True if an enabled behavior requires it; false otherwise.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean IsThisDataUsed(ModelData oData) {
    ModelData oCompare;
    String sName = oData.GetDescriptor();
    int i, j;
    boolean bIsUsed = false;

    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled) {
        for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
          if (mp_oChildBehaviors[i].mp_oRequiredData.elementAt(j) instanceof
              ModelData) {
            oCompare = (ModelData) mp_oChildBehaviors[i].mp_oRequiredData.
                elementAt(j);
            if (oCompare.GetDescriptor().equals(sName)) {
              bIsUsed = true;
              break;
            }
          }
          if (bIsUsed) {
            break;
          }
        }
      }
    }

    return bIsUsed;
  }

  /**
   * Writes a piece of data to an XML file.
   * @param jOut The file to write to.
   * @param oData The data being written.
   * @throws java.io.IOException Passes on exceptions from FileWriter
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteData(BufferedWriter jOut, ModelData oData) throws java.io.
      IOException {
    if (IsThisDataUsed(oData)) {
      WorkerBase.WriteDataToFile(jOut, oData);
    }
  }

  /**
   * Sets a chosen behavior as the next enabled behavior.  The behaviors are
   * listed in the order in which they appear in the run.  This finds the
   * behavior passed, and moves it in the list until it is immediately after
   * the last enabled behavior.  Then the behavior in question is set to
   * enabled.  This is meant to be used while loading a parameter file, so
   * the behavior name used is that which is in the parameter file.
   *
   * Created: 09-11-06, LEM
   *
   * @param sXMLName String The name of the behavior, from the parameter file,
   *  to make the next enabled.
   * @throws ModelException if the behavior is already enabled, or not enabled
   * yet in the list of enabled behaviors.  This is mainly an alert, because
   * it is a symptom of something else going wrong.
   */
  public void SetAsNextEnabled(String sXMLName) throws ModelException {

    //Get the behavior by its name
    Behavior oBehavior = GetBehaviorByXMLTag(sXMLName);

    if (oBehavior.m_bIsEnabled == true) {
      throw(new ModelException(ErrorGUI.BAD_COMMAND, "JAVA",
          "BehaviorTypeBase.SetAsNextEnabled: The \"" + sXMLName +
          "\" behavior is already enabled."));
    }

    String sTargetKey = oBehavior.GetKey();
    int i, iCurrPos = -1,  //our behavior's current position in the list
        iTargetPos = 0; //our behavior's target position in the list

    //Find this behavior's current position
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].GetKey().equals(sTargetKey)) {
        iCurrPos = i;
        break;
      }
    }

    if (iCurrPos < 0) {
      throw(new ModelException(ErrorGUI.BAD_COMMAND, "JAVA",
          "BehaviorTypeBase.SetAsNextEnabled: Couldn't find the \"" +
          sXMLName + "\" behavior."));
    }

    //If the behavior is already first, we're done
    if (iCurrPos == 0) {
      oBehavior.m_bIsEnabled = true;
      return;
    }

    //Find the position of the current last enabled behavior
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled == false) {
        iTargetPos = i;
        break;
      }
    }

    if (iTargetPos > iCurrPos) {
      //This should not happen - it means that the list of behaviors has
      //not been correctly constructed.  Throw an error.
      throw(new ModelException(ErrorGUI.BAD_COMMAND, "JAVA",
          "BehaviorTypeBase.SetAsNextEnabled: The behavior order is bad.  "+
          "Enabled behaviors not first as expected."));
    }

    //If our behavior is already in position, we're done
    if (iCurrPos == iTargetPos) {
      oBehavior.m_bIsEnabled = true;
      return;
    }

    //Create a space for our behavior by moving the ones in front of it to
    //the back
    for (i = iCurrPos - 1; i >= iTargetPos; i--) {
      mp_oChildBehaviors[i + 1] = mp_oChildBehaviors[i];
    }
    mp_oChildBehaviors[iTargetPos] = oBehavior;

    oBehavior.m_bIsEnabled = true;
  }

  /**
   * Triggered when there is a change in the species list.  This goes through
   * all behaviors and updates their assignments.
   * @param iOldNumSpecies says how many species there used to be.
   * @param p_iIndexer is an array, sized to the new number of species.  For
   * each bucket (representing the index number of a species on the new list),
   * the value is either the index of that same species in the old species
   * list, or -1 if the species is new.
   * @param p_sNewSpecies The new species list.
   * @throws ModelException if there's a problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 23, 2007: Created (LEM)
   */
  public void ChangeOfSpecies(int iOldNumSpecies, int[] p_iIndexer,
                              String[] p_sNewSpecies) throws ModelException {
    super.ChangeOfSpecies(iOldNumSpecies, p_iIndexer, p_sNewSpecies);

    TreePopulation oPop = m_oManager.GetTreePopulation();
    SpeciesTypeCombo oCombo;
    int i, j, iSpecies;

    //Make the index translator from old to new
    int[] p_iOldToNewTranslator = new int[oPop.GetNumberOfSpecies()];
    for (i = 0; i < p_iOldToNewTranslator.length; i++) {
      p_iOldToNewTranslator[i] = -1;

      //Does this species still exist in the new list?  If so, grab its index
      for (j = 0; j < p_sNewSpecies.length; j++) {
        if (p_sNewSpecies[j].equals(oPop.GetSpeciesNameFromCode(i))) {
          p_iOldToNewTranslator[i] = j;
        }
      }
    }

    //Go through all behaviors and update
    if (mp_oChildBehaviors != null) {
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].GetNumberOfCombos(); j++) {
          oCombo = mp_oChildBehaviors[i].GetSpeciesTypeCombo(j);
          iSpecies = oCombo.GetSpecies();

          if (p_iOldToNewTranslator[iSpecies] == -1) {
            mp_oChildBehaviors[i].DeleteSpeciesTypeCombo(j);
            j--;
          }
          else {
            oCombo.SetSpecies(p_iOldToNewTranslator[iSpecies]);
          }
        }
      }
    }
  }

  /**
   * Copies one species to another. This makes sure behavior assignments are
   * the same.
   * @param iSpeciesCopyFrom int Species to copy.
   * @param iSpeciesCopyTo int Species that is the copy.
   * @throws ModelException if there is a problem.
   */
  public void CopySpecies (int iSpeciesCopyFrom, int iSpeciesCopyTo) throws ModelException {
    super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);

    //First, remove the species that is to become the copy
    SpeciesTypeCombo oCombo;
    int i, j;

    //Go through all behaviors and update
    if (mp_oChildBehaviors != null) {
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].GetNumberOfCombos(); j++) {
          oCombo = mp_oChildBehaviors[i].GetSpeciesTypeCombo(j);
          if (iSpeciesCopyTo == oCombo.GetSpecies()) {
            mp_oChildBehaviors[i].DeleteSpeciesTypeCombo(j);
            j--;
          }
        }
      }

      //If any trees-needing behaviors lost all their combos, un-enable them
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        if (mp_oChildBehaviors[i].m_bMustHaveTrees &&
            0 == mp_oChildBehaviors[i].GetNumberOfCombos()) {
          mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
      }

      //Now copy the species
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].GetNumberOfCombos(); j++) {
          oCombo = mp_oChildBehaviors[i].GetSpeciesTypeCombo(j);
          if (iSpeciesCopyFrom == oCombo.GetSpecies()) {
            oCombo = (SpeciesTypeCombo)oCombo.clone();
            oCombo.SetSpecies(iSpeciesCopyTo);
            mp_oChildBehaviors[i].AddSpeciesTypeCombo(oCombo);
          }
        }
      }
    }
  }
}
