package javawrapper;

/**
 * Abstract class holding data member-based data, such as trees or grid values.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>October 29, 2010: Changed Floats to Floats
 */

public abstract class DataMemberData {

  /**Float data members*/
  protected Float[] mp_fFloats;
  /**Int data members*/
  protected Integer[] mp_iInts;
  /**Bool data members*/
  protected Boolean[] mp_bBools;
  /**Char data members - these may get truncated when read into C++*/
  protected String[] mp_sChars;

  /**
   * Constructor.  Creates the arrays.
   * @param iNumFloats Number of float data members
   * @param iNumInts Number of integer data members
   * @param iNumChars Number of char (String) data members
   * @param iNumBools Number of bool data members
   * @throws ModelException if the species or type numbers are invalid
   */
  public DataMemberData(int iNumFloats, int iNumInts, int iNumChars,
                        int iNumBools) throws ModelException {

    mp_iInts = new Integer[iNumInts];
    mp_fFloats = new Float[iNumFloats];
    mp_sChars = new String[iNumChars];
    mp_bBools = new Boolean[iNumBools];
  }

  /**
   * Sets an integer data member's value.
   * @param iIndex Index at which to set the value.
   * @param iValue The value to set.
   * @throws ModelException if the index is invalid.
   */
  public void SetValue(int iIndex, Integer iValue) throws ModelException {
    try {
      mp_iInts[iIndex] = iValue;
    }
    catch (java.lang.ArrayIndexOutOfBoundsException oE) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT,
                               "DataMemberData.SetValue - Java",
                            "Attempt to set int data member with bad index \""
                               + iIndex + "\"."));
    }
  }

  /**
   * Sets a float data member's value.
   * @param iIndex Index at which to set the value.
   * @param fValue The value to set.
   * @throws ModelException if the index is invalid.
   */
  public void SetValue(int iIndex, Float fValue) throws ModelException {
    try {
      mp_fFloats[iIndex] = fValue;
    }
    catch (java.lang.ArrayIndexOutOfBoundsException oE) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT,
                               "DataMemberData.SetValue - Java",
                        "Attempt to set float data member with bad index \""
                               + iIndex + "\"."));
    }
  }

  /**
   * Sets a boolean data member's value.
   * @param iIndex Index at which to set the value.
   * @param bValue The value to set.
   * @throws ModelException if the index is invalid.
   */
  public void SetValue(int iIndex, Boolean bValue) throws ModelException {
    try {
      mp_bBools[iIndex] = bValue;
    }
    catch (java.lang.ArrayIndexOutOfBoundsException oE) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT,
                               "DataMemberData.SetValue - Java",
                          "Attempt to set bool data member with bad index \""
                               + iIndex + "\"."));
    }
  }

  /**
   * Sets a char data member's value.
   * @param iIndex Index at which to set the value.
   * @param sValue The value to set.
   * @throws ModelException if the index is invalid.
   */
  public void SetValue(int iIndex, String sValue) throws ModelException {
    try {
      mp_sChars[iIndex] = sValue;
    }
    catch (java.lang.ArrayIndexOutOfBoundsException oE) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT,
                               "DataMemberData.SetValue - Java",
                        "Attempt to set char data member with bad index \""
                               + iIndex + "\"."));
    }
  }
}
