package javawrapper;

import java.util.Vector;

/**
 * Manages treefern establishment.
 * <p>Copyright: Copyright (c) 2009 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 *
 * <br>Edit history:
 * <br>------------------
 * <br>December 16, 2009: Created (LEM)
 */

public class EpiphyticEstablishmentBehaviors
    extends BehaviorTypeBase {

  /** Treefern Establishment - a in the seedling probability equation. */
  protected ModelVector mp_fTreeFernRecruitA = new ModelVector(
      "Tree Fern Establishment Seedling Prob \"a\"", "ep_epiphyticA", "ep_eaVal", 0,
      ModelVector.FLOAT);
  
  /** Treefern Establishment - b in the seedling probability equation. */
  protected ModelVector mp_fTreeFernRecruitB = new ModelVector(
      "Tree Fern Establishment Seedling Prob \"b\"", "ep_epiphyticB", "ep_ebVal", 0,
      ModelVector.FLOAT);
  
  /** Treefern Establishment - c in the seedling probability equation. */
  protected ModelVector mp_fTreeFernRecruitC = new ModelVector(
      "Tree Fern Establishment Seedling Prob \"c\"", "ep_epiphyticC", "ep_ecVal", 0,
      ModelVector.FLOAT);
  
  /** Treefern Establishment - m in the seedling probability equation. */
  protected ModelVector mp_fTreeFernRecruitM = new ModelVector(
      "Tree Fern Establishment Seedling Height \"m\"", "ep_epiphyticM", 
      "ep_emVal", 0, ModelVector.FLOAT);
  
  /** Treefern Establishment - n in the seedling probability equation */
  protected ModelVector mp_fTreeFernRecruitN = new ModelVector(
      "Tree Fern Establishment Seedling Height \"n\"", "ep_epiphyticN", 
      "ep_enVal", 0, ModelVector.FLOAT);
  
  /**Species-specific - snag age class 1 amount of light transmission*/
  protected ModelVector mp_fSnagClass1LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
                  "li_snag1LightExtinctionCoefficient", "li_s1lecVal", 0,
                  ModelVector.FLOAT, true);

  /**Species-specific - snag age class 2 light transmission coefficient*/
  protected ModelVector mp_fSnagClass2LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
                  "li_snag2LightExtinctionCoefficient", "li_s2lecVal", 0,
                  ModelVector.FLOAT, true);

  /**Species-specific - snag age class 3 light transmission coefficient*/
  protected ModelVector mp_fSnagClass3LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
                  "li_snag3LightExtinctionCoefficient", "li_s3lecVal", 0,
                  ModelVector.FLOAT, true);
  
  /**Treefern Establishment - Beam fraction of global radiation*/
  protected ModelFloat m_fBeamFractionOfGlobalRadiation = new ModelFloat(0,
      "Beam Fraction of Global Radiation", "li_beamFractGlobalRad");

  /**Treefern Establishment - Clear sky transmission coefficient*/
  protected ModelFloat m_fClearSkyTransmissionCoefficient = new ModelFloat(0,
      "Clear Sky Transmission Coefficient", "li_clearSkyTransCoeff");
  
  /**Treefern Establishment - Light extinction coefficient*/
  protected ModelVector mp_fLightTransmissionCoefficient = new ModelVector(
      "Amount Canopy Light Transmission (0-1)",
      "li_lightExtinctionCoefficient", "li_lecVal", 0,
      ModelVector.FLOAT, true);

  /**Treefern Establishment - Minimum sun angle in radians for GLI*/
  protected ModelFloat m_fMinSunAngle = new ModelFloat(0,
      "Minimum Solar Angle for GLI Calculations, in rad", "li_minSunAngle");
  
  /**Treefern Establishment - Number of azimuth divisions for GLI*/
  protected ModelInt m_iNumAziDiv = new ModelInt(0,
         "Number of Azimuth Sky Divisions for GLI Light Calculations",
                                                 "li_numAziGrids");

  /**Treefern Establishment - Number of altitude divisions for GLI*/
  protected ModelInt m_iNumAltDiv = new ModelInt(0,
         "Number of Altitude Sky Divisions for GLI Light Calculations",
                                                 "li_numAltGrids");
  
  /**Upper age limit of snag size class 1*/
  protected ModelInt m_iSnagAgeClass1 = new ModelInt(0,
      "Upper Age (Yrs) of Snag Light Transmission Class 1", "li_snagAgeClass1");

  /**Upper age limit of snag size class 2*/
  protected ModelInt m_iSnagAgeClass2 = new ModelInt(0,
      "Upper Age (Yrs) of Snag Light Transmission Class 2", "li_snagAgeClass2");
  
  /**Start of growing season as Julian day*/
  protected ModelInt m_iJulianDayGrowthStarts = new ModelInt(0,
      "First Day of Growing Season for GLI Light Calculations",
      "li_julianDayGrowthStarts");

  /**End of growing season as Julian day*/
  protected ModelInt m_iJulianDayGrowthEnds = new ModelInt(0,
      "Last Day of Growing Season for GLI Light Calculations",
      "li_julianDayGrowthEnds");
    
  /**
   * Constructor
   * @param oManager GUIManager object.
   * @throws ModelException won't.
   */
  public EpiphyticEstablishmentBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Epiphytic Establishment", "epiphyticEstablishment");

    m_sHelpID = "data.epiphytic_establishment_behaviors";

    //Set up our child behavior vector
    mp_oChildBehaviors = new Behavior[1];
    mp_oChildBehaviors[0] = new Behavior("Tree Fern Establishment",
                                         "Tree Fern Establishment",
                                         "Epiphytic Establishment",
                                          1.0);
    //Can only apply to saplings and adults
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SAPLING, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.ADULT, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SNAG, true);

    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeFernRecruitA);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeFernRecruitB);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeFernRecruitC);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeFernRecruitM);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeFernRecruitN);
    mp_oChildBehaviors[0].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(m_fMinSunAngle);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumAziDiv);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumAltDiv);
    mp_oChildBehaviors[0].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[0].AddRequiredData(m_iSnagAgeClass2);
    
    mp_oChildBehaviors[0].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[0].AddRequiredData(m_iJulianDayGrowthEnds);
    
    mp_oAllData.add(mp_fTreeFernRecruitA);
    mp_oAllData.add(mp_fTreeFernRecruitB);
    mp_oAllData.add(mp_fTreeFernRecruitC);
    mp_oAllData.add(mp_fTreeFernRecruitM);
    mp_oAllData.add(mp_fTreeFernRecruitN);
    mp_oAllData.add(mp_fLightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oAllData.add(m_fBeamFractionOfGlobalRadiation);
    mp_oAllData.add(m_fClearSkyTransmissionCoefficient);
    mp_oAllData.add(m_fMinSunAngle);
    mp_oAllData.add(m_iNumAziDiv);
    mp_oAllData.add(m_iNumAltDiv);
    mp_oAllData.add(m_iJulianDayGrowthStarts);
    mp_oAllData.add(m_iJulianDayGrowthEnds);
    mp_oAllData.add(m_iSnagAgeClass1);
    mp_oAllData.add(m_iSnagAgeClass2);
  }

  /**
   * Adds all species to the species seedling choice.
   * @param oPop Tree population
   */
  public void DoSetup(TreePopulation oPop) {
    ModelEnum oSeedlingSpecies;
    int iNumSpecies = oPop.GetNumberOfSpecies(), i, j;
    
    String[] p_sSpNames = new String[iNumSpecies];
    int[] p_iCodes = new int[iNumSpecies];
    j = iNumSpecies - 1;
    for (i = 0; i < iNumSpecies; i++) {
      p_iCodes[i] = j;
      p_sSpNames[i] = oPop.GetSpeciesNameFromCode(j);
      j--;
    }
    oSeedlingSpecies = new ModelEnum(p_iCodes, p_sSpNames, 
        "Tree Fern Establishment Species of New Seedlings", 
        "ep_epiphyticSeedlingSpecies");
    
    GetBehaviorByKey("Tree Fern Establishment").AddRequiredData(oSeedlingSpecies);
    mp_oAllData.add(oSeedlingSpecies);
  }

  /**
   * Validates the data in preparation for parameter file writing or some such.
   * @throws ModelException if any of the probabilities are not proportions,
   * or if the probabilities for a class don't add up to 1.
   * @param oPop Not used.
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    if (AnyBehaviorsEnabled()) {
      MakeSureRightSize(mp_fLightTransmissionCoefficient, oPop.GetNumberOfSpecies());
      MakeSureAllAreProportions(mp_fLightTransmissionCoefficient);
      MakeSureIsProportion(m_fBeamFractionOfGlobalRadiation);
      MakeSureGreaterThan(m_fClearSkyTransmissionCoefficient, 0);
      MakeSureIsBounded(m_iJulianDayGrowthStarts, 1, 365);
      MakeSureIsBounded(m_iJulianDayGrowthEnds, 1, 365);
      MakeSureGreaterThan(m_iNumAltDiv, 0);
      MakeSureGreaterThan(m_iNumAziDiv, 0);
      MakeSureGreaterThan(m_fMinSunAngle, 0);


      //If there are any snags enabled anywhere, make sure that the snag data is
      //filled out
      if (m_oManager.GetSnagAwareness()) {
        MakeSureAllAreProportions(mp_fSnagClass1LightTransmissionCoefficient);
        MakeSureAllAreProportions(mp_fSnagClass2LightTransmissionCoefficient);
        MakeSureAllAreProportions(mp_fSnagClass3LightTransmissionCoefficient);
        MakeSureGreaterThan(m_iSnagAgeClass1, 0);
        MakeSureGreaterThan(m_iSnagAgeClass2, 0);
        if (m_iSnagAgeClass2.GetValue() <= m_iSnagAgeClass1.GetValue()) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
              m_iSnagAgeClass2.GetDescriptor() +
              " must be greater than " +
              m_iSnagAgeClass1.GetDescriptor());
        }
      }
    }
    
  }
  
  /**
   * Checks to see if there are light parameters that could be grabbed as
   * defaults.
   */
  public void EndOfParameterFileRead() {
    LightBehaviors oLight = m_oManager.GetLightBehaviors();
    int i;
    if (mp_fLightTransmissionCoefficient.GetValue().size() == 0) {
      for (i = 0; i < oLight.mp_fLightTransmissionCoefficient.GetValue().size(); i++)
        mp_fLightTransmissionCoefficient.GetValue().add(oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(i));
    }
    
    if (mp_fSnagClass1LightTransmissionCoefficient.GetValue().size() == 0) {
      for (i = 0; i < oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().size(); i++)
        mp_fSnagClass1LightTransmissionCoefficient.GetValue().add(oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().elementAt(i));
    }
    if (mp_fSnagClass2LightTransmissionCoefficient.GetValue().size() == 0) {
      for (i = 0; i < oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().size(); i++)
        mp_fSnagClass2LightTransmissionCoefficient.GetValue().add(oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().elementAt(i));
    }
    if (mp_fSnagClass3LightTransmissionCoefficient.GetValue().size() == 0) {
      for (i = 0; i < oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().size(); i++)
        mp_fSnagClass3LightTransmissionCoefficient.GetValue().add(oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().elementAt(i));
    }
    
    if (0 == m_fBeamFractionOfGlobalRadiation.GetValue() &&
        oLight.m_fBeamFractionOfGlobalRadiation.GetValue() > 0) 
      m_fBeamFractionOfGlobalRadiation.SetValue(oLight.m_fBeamFractionOfGlobalRadiation.GetValue());
    
    if (0 == m_fClearSkyTransmissionCoefficient.GetValue() &&
        oLight.m_fClearSkyTransmissionCoefficient.GetValue() > 0) 
      m_fClearSkyTransmissionCoefficient.SetValue(oLight.m_fClearSkyTransmissionCoefficient.GetValue());
    
    if (0 == m_iJulianDayGrowthStarts.GetValue() &&
        oLight.m_iJulianDayGrowthStarts.GetValue() > 0) 
      m_iJulianDayGrowthStarts.SetValue(oLight.m_iJulianDayGrowthStarts.GetValue());
    
    if (0 == m_iJulianDayGrowthEnds.GetValue() && 
        oLight.m_iJulianDayGrowthEnds.GetValue() > 0) 
      m_iJulianDayGrowthEnds.SetValue(oLight.m_iJulianDayGrowthEnds.GetValue());
    
    if (0 == m_iSnagAgeClass1.GetValue() && 
        oLight.m_iSnagAgeClass1.GetValue() > 0) 
      m_iSnagAgeClass1.SetValue(oLight.m_iSnagAgeClass1.GetValue());
    
    if (0 == m_iSnagAgeClass2.GetValue() && 
        oLight.m_iSnagAgeClass2.GetValue() > 0) 
      m_iSnagAgeClass2.SetValue(oLight.m_iSnagAgeClass2.GetValue());
    
    if (0 == m_iNumAltDiv.GetValue()) {
      if (oLight.m_iNumGLIAltDiv.GetValue() > 0)
        m_iNumAltDiv.SetValue(oLight.m_iNumGLIAltDiv.GetValue());
      else if (oLight.m_iNumQuadratAltDiv.GetValue() > 0)
        m_iNumAltDiv.SetValue(oLight.m_iNumQuadratAltDiv.GetValue());
      else if (oLight.m_iNumMapAltDiv.GetValue() > 0)
        m_iNumAltDiv.SetValue(oLight.m_iNumMapAltDiv.GetValue());
      else if (oLight.m_iNumPointsAltDiv.GetValue() > 0)
        m_iNumAltDiv.SetValue(oLight.m_iNumPointsAltDiv.GetValue());
    }  
    
    if (0 == m_iNumAziDiv.GetValue()) {
      if (oLight.m_iNumGLIAziDiv.GetValue() > 0)
        m_iNumAziDiv.SetValue(oLight.m_iNumGLIAziDiv.GetValue());
      else if (oLight.m_iNumQuadratAziDiv.GetValue() > 0)
        m_iNumAziDiv.SetValue(oLight.m_iNumQuadratAziDiv.GetValue());
      else if (oLight.m_iNumMapAziDiv.GetValue() > 0)
        m_iNumAziDiv.SetValue(oLight.m_iNumMapAziDiv.GetValue());
      else if (oLight.m_iNumPointsAziDiv.GetValue() > 0)
        m_iNumAziDiv.SetValue(oLight.m_iNumPointsAziDiv.GetValue());
    }
    if (0 == m_fMinSunAngle.GetValue()) {
      if (oLight.m_fGLIMinSunAngle.GetValue() > 0)
        m_fMinSunAngle.SetValue(oLight.m_fGLIMinSunAngle.GetValue());
      else if (oLight.m_fQuadratMinSunAngle.GetValue() > 0)
        m_fMinSunAngle.SetValue(oLight.m_fQuadratMinSunAngle.GetValue());
      else if (oLight.m_fMapMinSunAngle.GetValue() > 0)
        m_fMinSunAngle.SetValue(oLight.m_fMapMinSunAngle.GetValue());
      else if (oLight.m_fPointsMinSunAngle.GetValue() > 0)
        m_fMinSunAngle.SetValue(oLight.m_fPointsMinSunAngle.GetValue());
    }
  }
  
  /**
   * Find light values for epiphytic establishment and reject those for light
   * behaviors.
   * @param sXMLTag XML tag of object to assign.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Ignored.
   * @param oData Data to assign.
   * @return true if assigned, or false if not.
   * @throws ModelException not thrown here.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        org.xml.sax.Attributes oAttributes,
                                        Object oData) throws
      ModelException {

    if (sXMLParentTag.equals(m_sXMLTag)) {
      return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag,
                                          oAttributes, oData);
    }
    return false;
  }
  
  /**
   * Find light values for epiphytic establishment and reject those for light
   * behaviors.
   * @param sXMLTag Parent XML tag of data vector whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param p_oData Vector of data values appropriate to the data type
   * @param p_sChildXMLTags The XML tags of the child elements
   * @param p_bAppliesTo Array of booleans saying which of the vector values
   * should be set.  This is important in the case of species-specifics - the
   * vector index is the species number but not all species are set.
   * @param oParentAttributes Attributes of parent tag.  May be useful when
   * overridding this for unusual tags.
   * @param p_oAttributes Attributes passed from parser.  This may be needed
   * when overriding this function.  Basic species-specific values are
   * already handled by this function.
   * @return true if the value was set successfully; false if the value could
   * not be found.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Vector<String> p_oData,
                                        String[] p_sChildXMLTags,
                                        boolean[] p_bAppliesTo,
                                        org.xml.sax.Attributes
                                        oParentAttributes,
                                        org.xml.sax.Attributes[] p_oAttributes) throws
      ModelException {

    if (sXMLParentTag.equals(m_sXMLTag)) {
      return super.SetVectorValueByXMLTag(sXMLTag, sXMLParentTag, p_oData,
                                          p_sChildXMLTags, p_bAppliesTo,
                                          oParentAttributes, p_oAttributes);
    }
    return false;
  }
}
