package javawrapper;

import java.io.BufferedWriter;
import java.util.Vector;
import org.xml.sax.Attributes;

/**
 * Manages output behaviors and data
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 11, 2006:  Added subplot support to detailed output (LEM)
 * <br>January 16, 2007: Made it so this dealt gracefully with species changes
 * in grid save settings (LEM)
 * <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
 * <br>January 20, 2011: Added support for dead trees (LEM)
 */

public class OutputBehaviors
    extends BehaviorTypeBase {
  
  /** Dead code for not dead - matches definition in C++ */
  public final static int NOTDEAD = 0;

  /** Dead code for harvest - matches definition in C++ */
  public final static int HARVEST = 1;

  /** Dead code for natural mortality - matches definition in C++ */
  public final static int NATURAL = 2;

  /** Dead code for disease - matches definition in C++ */
  public final static int DISEASE = 3; 

  /** Dead code for fire - matches definition in C++ */
  public final static int FIRE = 4;

  /** Dead code for insects - matches definition in C++ */
  public final static int INSECTS = 5;

  /** Dead code for storm - matches definition in C++ */
  public final static int STORM = 6;
  
  /** Total number of dead reason codes */
  public final static int NUMCODES = 7;  
  
  /**For short output - collection of TreeOutputSaveInfo objects - vector is
   * sized number of types*/
  protected Vector<TreeOutputSaveInfo> mp_oShortTreeSaveSettings = 
      new Vector<TreeOutputSaveInfo>(0);
  
  /**For short output - settings for dead trees - sized number of types by 
   * number of dead reason codes */
  protected TreeOutputSaveInfo[][] mp_oShortDeadTreeSaveSettings = new 
    TreeOutputSaveInfo[TreePopulation.GetNumberOfTypes()][NUMCODES];

  /**For detailed output - collection of DetailedTreeSettings objects*/
  protected Vector<DetailedTreeSettings> mp_oDetailedLiveTreeSaveSettings = new Vector<DetailedTreeSettings>(0);
  
  /**For detailed output - collection of DetailedTreeSettings objects*/
  protected Vector<DetailedTreeSettings> mp_oDetailedDeadTreeSaveSettings = new Vector<DetailedTreeSettings>(0);

  /**For detailed output - collection of DetailedGridSettings objects*/
  protected Vector<DetailedGridSettings> mp_oDetailedGridSaveSettings = new Vector<DetailedGridSettings>(0);

  /**For short output - the list of Subplot objects*/
  protected Vector<Subplot> mp_oShortOutputSubplots = new Vector<Subplot>(0);

  /**For detailed output - the list of Subplot objects*/
  protected Vector<Subplot> mp_oDetailedOutputSubplots = new Vector<Subplot>(0);

  /**Tree population*/
  protected TreePopulation m_oPop;

  /**File name for short output*/
  protected ModelString m_sShortOutputFilename = new ModelString("", 
  		"Filename for summary output", "so_filename");
  
  /**File name for detailed output*/
  protected ModelString m_sDetailedOutputFilename = new ModelString("", 
  		"Detailed output filename", "ou_filename");

  /**When parsing an XML file, this is the setting currently receiving data*/
  protected DetailedOutputSettings m_oCurrentDetailedSettings;

  /**When parsing an XML file, this is the subplot currently receiving data*/
  protected Subplot m_oCurrentSubplot;
  
  /**Subplot X cell length for detailed output subplots - defaults to match
   * plot*/
  protected ModelFloat m_fDetailedSubplotXCellLength = new ModelFloat(
  		Plot.CELL_LENGTH,	"Subplot cell length - X (m):", "ou_subplotXLength");
  
  /**Subplot Y cell length for detailed output subplots - defaults to match
   * plot*/
  protected ModelFloat m_fDetailedSubplotYCellLength = new ModelFloat(
  		Plot.CELL_LENGTH,	"Subplot cell length - Y (m):", "ou_subplotYLength");
  
  /**Subplot X cell length for short output subplots - defaults to match plot*/
  protected ModelFloat m_fShortSubplotXCellLength = new ModelFloat(
  		Plot.CELL_LENGTH,	"Subplot cell length - X (m):", "so_subplotXLength");
  
  /**Subplot Y cell length for short output subplots - defaults to match plot*/
  protected ModelFloat m_fShortSubplotYCellLength = new ModelFloat(
  		Plot.CELL_LENGTH,	"Subplot cell length - Y (m):", "so_subplotYLength");

  /**When parsing an XML file, this is the current short output type*/
  protected int m_iCurrentType;
  
  /**When parsing an XML file, this is the current short output dead code*/
  protected int m_iCurrentDeadCode;

  /**Flag for whether species change has occurred - just renaming doesn't
   * count*/
  private boolean m_bChangeOfSpecies = false;

  /** File extension for the detailed output file. */
  public final static String DETAILED_OUTPUT_EXTENSION = ".gz.tar";

  /** File extension for the summary output file. */
  public final static String SUMMARY_EXTENSION = ".out";

  /**
   * Constructor.
   * @param oManager GUIManager object.
   */
  public OutputBehaviors(GUIManager oManager) {
    super(oManager, "Output", "output");
    m_bAutoDisplayParameters = false;

    m_iCurrentType = -1;
    
    //Make our child behaviors
    mp_oChildBehaviors = new Behavior[2];
    mp_oChildBehaviors[0] = new Behavior("short output", "Summary Output",
                                         "short output", 1);
    mp_oChildBehaviors[0].m_bMustHaveTrees = false;
    mp_oChildBehaviors[0].m_bCanEditAutomatically = false;
    mp_oChildBehaviors[0].AddRequiredData(m_sShortOutputFilename);
    mp_oChildBehaviors[0].AddRequiredData(m_fShortSubplotXCellLength);
    mp_oChildBehaviors[0].AddRequiredData(m_fShortSubplotYCellLength);

    mp_oChildBehaviors[1] = new Behavior("detailed", "Detailed Output",
                                         "output",
                                         1);
    mp_oChildBehaviors[1].m_bMustHaveTrees = false;
    mp_oChildBehaviors[1].m_bCanEditAutomatically = false;
    mp_oChildBehaviors[1].AddRequiredData(m_sDetailedOutputFilename);
    mp_oChildBehaviors[1].AddRequiredData(m_fDetailedSubplotXCellLength);
    mp_oChildBehaviors[1].AddRequiredData(m_fDetailedSubplotYCellLength);

    mp_oAllData.add(m_sDetailedOutputFilename);
    mp_oAllData.add(m_sShortOutputFilename);
    mp_oAllData.add(m_fShortSubplotXCellLength);
    mp_oAllData.add(m_fShortSubplotYCellLength);
    mp_oAllData.add(m_fDetailedSubplotXCellLength);
    mp_oAllData.add(m_fDetailedSubplotYCellLength);
  }

  /**
   * Creates a subplot for short output.
   * @param sName Name of the subplot.
   * @return The new Subplot object.
   */
  public Subplot AddShortOutputSubplot(String sName) {
    Subplot oSubplot = new Subplot(sName);
    mp_oShortOutputSubplots.add(oSubplot);
    return oSubplot;
  }

  /**
   * Creates a subplot for detailed output.
   * @param sName Name of the subplot.
   * @return The new Subplot object.
   */
  public Subplot AddDetailedOutputSubplot(String sName) {
    Subplot oSubplot = new Subplot(sName);
    mp_oDetailedOutputSubplots.add(oSubplot);
    return oSubplot;
  }

  /**
   * Sets the short output filename.
   * @param sFilename Filename.
   */
  public void SetShortOutputFileName(String sFilename) {
    sFilename = sFilename.trim();
    //Make sure the file extension's on it
    if (sFilename.length() > 0 &&
        sFilename.toLowerCase().endsWith(SUMMARY_EXTENSION) == false) {
      sFilename += SUMMARY_EXTENSION;
    }
    //Check for ampersands
    sFilename = sFilename.replaceAll("&", "&amp;");

    m_sShortOutputFilename.SetValue(sFilename);
  }

  /**
   * Gets the short output filename.
   * @return Filename.
   */
  public String GetShortOutputFileName() {
    return m_sShortOutputFilename.GetValue();
  }

  /**
   * Sets the detailed output filename.
   * @param sFilename Filename.
   */
  public void SetDetailedOutputFileName(String sFilename) {
    sFilename = sFilename.trim();
    //Make sure the file extension's on it
    if (sFilename.length() > 0 &&
        sFilename.toLowerCase().endsWith(DETAILED_OUTPUT_EXTENSION) == false) {
      sFilename += DETAILED_OUTPUT_EXTENSION;
    }
    //Check for ampersands
    sFilename = sFilename.replaceAll("&", "&amp;");

    m_sDetailedOutputFilename.SetValue(sFilename);
  }

  /**
   * Gets the detailed output filename.
   * @return Filename.
   */
  public String GetDetailedOutputFileName() {
    return m_sDetailedOutputFilename.GetValue();
  }

  /**
   * Gets the number of tree settings for which there is detailed output info
   * (each setting is one tree species/type combo).
   * @return Number of settings.
   */
  public int GetNumberOfDetailedLiveTreeSettings() {
    return mp_oDetailedLiveTreeSaveSettings.size();
  }
  
  /**
   * Gets the number of tree settings for which there is detailed output info
   * (each setting is one tree species/type combo).
   * @return Number of settings.
   */
  public int GetNumberOfDetailedDeadTreeSettings() {
    return mp_oDetailedDeadTreeSaveSettings.size();
  }

  /**
   * Gets the number of grid settings for which there is detailed output info.
   * @return Number of settings.
   */
  public int GetNumberOfDetailedGridSettings() {
    return mp_oDetailedGridSaveSettings.size();
  }

  /**
   * Gets whether or not short output is being saved.
   * @return Save status.
   */
  public boolean SavingShortOutputData() {
    Behavior oBeh = GetBehaviorByKey("short output");
    return oBeh.m_bIsEnabled;
  }

  /**
   * Returns the detailed output tree setting at a specified index.
   * @param iIndex The index.
   * @return The DetailedTreeSettings object, or null if the index is invalid.
   */
  public DetailedTreeSettings GetDetailedLiveTreeSetting(int iIndex) {
    if (iIndex < 0 || iIndex > mp_oDetailedLiveTreeSaveSettings.size()) {
      return null;
    }
    else {
      return (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.elementAt(
          iIndex);
    }
  }
  
  /**
   * Returns the detailed output tree setting at a specified index.
   * @param iIndex The index.
   * @return The DetailedTreeSettings object, or null if the index is invalid.
   */
  public DetailedTreeSettings GetDetailedDeadTreeSetting(int iIndex) {
    if (iIndex < 0 || iIndex > mp_oDetailedDeadTreeSaveSettings.size()) {
      return null;
    }
    else {
      return (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.elementAt(
          iIndex);
    }
  }

  /**
   * Returns the detailed output grid setting at a specified index.
   * @param iIndex The index.
   * @return The DetailedGridSettings object, or null if the index is invalid.
   */
  public DetailedGridSettings GetDetailedGridSetting(int iIndex) {
    if (iIndex < 0 || iIndex > mp_oDetailedGridSaveSettings.size()) {
      return null;
    }
    else {
      return (DetailedGridSettings) mp_oDetailedGridSaveSettings.elementAt(
          iIndex);
    }
  }

  /**
   * Keeps the behavior enabled flag current.  Any change of save flags has
   * the potential to enable the behavior (by making the first save flag true)
   * or unenabling (by making the last save flag false).
   */
  protected void UpdateIsEnabled() {
    TreeOutputSaveInfo oData;
    int i, j;
    boolean bAnySaveFlags = false;
    for (i = 0; i < mp_oShortTreeSaveSettings.size(); i++) {
      for (j = 0; j < mp_oShortDeadTreeSaveSettings[i].length; j++) {
        if (mp_oShortDeadTreeSaveSettings[i][j].bSaveABA || 
            mp_oShortDeadTreeSaveSettings[i][j].bSaveADN || 
            mp_oShortDeadTreeSaveSettings[i][j].bSaveRBA || 
            mp_oShortDeadTreeSaveSettings[i][j].bSaveRDN) {
          bAnySaveFlags = true;
          break;
        }        
      }
      oData = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.elementAt(i);
      if (bAnySaveFlags || oData.bSaveABA || oData.bSaveADN || 
          oData.bSaveRBA || oData.bSaveRDN) {
        bAnySaveFlags = true;
        break;
      }      
    }

    GetBehaviorByKey("short output").m_bIsEnabled = bAnySaveFlags;
  }

  /**
   * Sets the save absolute density attribute for a type.
   * @param iType Tree type.
   * @param iDeadCode Dead code. Use NOTDEAD for live trees. 
   * @param bUse Whether or not to use this.
   * @throws ModelException If the type or dead code is not valid.
   */
  public void SetSaveAbsoluteDensity(int iType, int iDeadCode, boolean bUse) throws
      ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }
    if (iDeadCode < NOTDEAD || iDeadCode >= NUMCODES) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid dead code passed to output"));
    }

    if (NOTDEAD == iDeadCode) {
      TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
      elementAt(iType);
      oInfo.bSaveADN = bUse;
    } else {
      mp_oShortDeadTreeSaveSettings[iType][iDeadCode].bSaveADN = bUse; 
    }

    UpdateIsEnabled();
  }

  /**
   * Sets the save relative density attribute for a type.
   * @param iType Tree type.
   * @param bUse Whether or not to use this.
   * @throws ModelException If the type is not valid.
   */
  public void SetSaveRelativeDensity(int iType, boolean bUse) throws
      ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }

    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    oInfo.bSaveRDN = bUse;

    UpdateIsEnabled();
  }

  /**
   * Sets the save absolute basal area attribute for a type.  This is ignored
   * if it's not sapling or adult.
   * @param iType Tree type.
   * @param iDeadCode Dead code. Use NOTDEAD for live trees. 
   * @param bUse Whether or not to use this.
   * @throws ModelException If the type or dead code is not valid.
   */
  public void SetSaveAbsoluteBasalArea(int iType, int iDeadCode, boolean bUse) throws
      ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }
    if (iDeadCode < NOTDEAD || iDeadCode >= NUMCODES) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid dead code passed to output"));
    }

    if (iType == TreePopulation.SEEDLING) {
      return;
    }
    if (NOTDEAD == iDeadCode) {
      TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
      elementAt(iType);
      oInfo.bSaveABA = bUse;
    } else {
      mp_oShortDeadTreeSaveSettings[iType][iDeadCode].bSaveABA = bUse; 
    }

    UpdateIsEnabled();
  }

  /**
   * Sets the save relative basal area attribute for a type.  This is ignored
   * if it's not sapling or adult.
   * @param iType Tree type.
   * @param bUse Whether or not to use this.
   * @throws ModelException If the type is not valid.
   */
  public void SetSaveRelativeBasalArea(int iType, boolean bUse) throws
      ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }

    if (iType == TreePopulation.SEEDLING) {
      return;
    }

    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    oInfo.bSaveRBA = bUse;

    UpdateIsEnabled();
  }

  /**
   * Gets whether or not absolute basal area is saved for a tree type.
   * @param iType Tree type.
   * @param iDeadCode Dead code. Use NOTDEAD for live trees. 
   * @return Save flag.
   * @throws ModelException if the type or dead code is not valid.
   */
  public boolean GetSaveAbsoluteBasalArea(int iType, int iDeadCode) throws ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }
    if (iDeadCode < NOTDEAD || iDeadCode >= NUMCODES) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid dead code passed to output"));
    }
    if (NOTDEAD == iDeadCode) {
    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    return (oInfo.bSaveABA);
    } else {
      return mp_oShortDeadTreeSaveSettings[iType][iDeadCode].bSaveABA; 
    }
  }

  /**
   * Gets whether or not relative basal area is saved for a tree type.
   * @param iType Tree type.
   * @return Save flag.
   * @throws ModelException if the type is not valid.
   */
  public boolean GetSaveRelativeBasalArea(int iType) throws ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }

    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    return (oInfo.bSaveRBA);
  }

  /**
   * Gets whether or not absolute density is saved for a tree type.
   * @param iType Tree type.
   * @param iDeadCode Dead code. Use NOTDEAD for live trees. 
   * @return Save flag.
   * @throws ModelException if the type or dead code is not valid.
   */
  public boolean GetSaveAbsoluteDensity(int iType, int iDeadCode) throws ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }
    if (iDeadCode < NOTDEAD || iDeadCode >= NUMCODES) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid dead code passed to output"));
    }
    if (NOTDEAD == iDeadCode) {
    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    return (oInfo.bSaveADN);
    } else {
      return mp_oShortDeadTreeSaveSettings[iType][iDeadCode].bSaveADN; 
    }
  }

  /**
   * Gets whether or not relative density is saved for a tree type.
   * @param iType Tree type.
   * @return Save flag.
   * @throws ModelException if the type is not valid.
   */
  public boolean GetSaveRelativeDensity(int iType) throws ModelException {
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.ILLEGAL_OP, "JAVA",
                               "Invalid type passed to output"));
    }

    TreeOutputSaveInfo oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.
        elementAt(iType);
    return (oInfo.bSaveRDN);
  }

  /**
   * Overridden from base because output parameters are special.
   * @param jOut File to write to.
   * @param oPop TreePopulation object.
   * @throws ModelException if there is a problem writing the file.
   */
  public void WriteXML(BufferedWriter jOut, TreePopulation oPop) throws
      ModelException {

    try {
      WriteShortOutputXML(jOut);

      WriteDetailedOutputXML(jOut, oPop);
    }
    catch (java.io.IOException e) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "JAVA",
                            "There was a problem writing the parameter file."));
    }
  }

  /**
   * Writes the parameter file data for short output.  Does nothing if there
   * are no short output save settings.
   * @param jOut File to write
   * @throws java.io.IOException passed through from file writing
   */
  protected void WriteShortOutputXML(BufferedWriter jOut) throws java.io.
      IOException {
    //Make sure we have anything to save
    TreeOutputSaveInfo oInfo;
    Subplot oSubplot;
    Cell oCell;
    int i, j;
    UpdateIsEnabled();
    if (!GetBehaviorByKey("short output").m_bIsEnabled) {
      return;
    }

    jOut.write("<shortoutput>");

    //Write file name
    WriteData(jOut, m_sShortOutputFilename);
    
    //Write subplot cell size, if applicable
    if (8.0 != m_fShortSubplotXCellLength.GetValue() ||
    		8.0 != m_fShortSubplotYCellLength.GetValue()) {
    	WriteData(jOut, m_fShortSubplotXCellLength);
    	WriteData(jOut, m_fShortSubplotYCellLength);
    }

    //Write the tree types
    for (i = 0; i < mp_oShortTreeSaveSettings.size(); i++) {
      oInfo = (TreeOutputSaveInfo) mp_oShortTreeSaveSettings.elementAt(i);

      if (oInfo.bSaveABA || oInfo.bSaveADN || oInfo.bSaveRBA ||
          oInfo.bSaveRDN) {
        jOut.write("<so_treeTypeInfo type=\"" + TreePopulation.GetTypeNameFromCode(i) +
                   "\">");

        //Save relative basal area
        if (i > TreePopulation.SEEDLING) {
          jOut.write("<so_saveRBA save=\"");
          if (oInfo.bSaveRBA) jOut.write("true");
          else jOut.write("false");
          jOut.write("\"/>");
        }

        //Save absolute basal area
        if (i > TreePopulation.SEEDLING) {
          jOut.write("<so_saveABA save=\"");
          if (oInfo.bSaveABA) jOut.write("true");
          else jOut.write("false");
          jOut.write("\"/>");
        }

        //Save relative density
        jOut.write("<so_saveRDN save=\"");
        if (oInfo.bSaveRDN) jOut.write("true");
        else jOut.write("false");       
        jOut.write("\"/>");

        //Save absolute density
        jOut.write("<so_saveADN save=\"");
        if (oInfo.bSaveADN) jOut.write("true");
        else jOut.write("false");
        jOut.write("\"/>");

        jOut.write("</so_treeTypeInfo>");
      }
    }

    //Now write subplots
    for (i = 0; i < mp_oShortOutputSubplots.size(); i++) {
      oSubplot = (Subplot) mp_oShortOutputSubplots.elementAt(i);
      if (oSubplot.GetNumberOfCells() > 0) {
        jOut.write("<so_subplot>");
        jOut.write("<so_subplotName>" + oSubplot.GetSubplotName() +
                   "</so_subplotName>");
        jOut.write("<pointSet>");
        for (j = 0; j < oSubplot.GetNumberOfCells(); j++) {

          oCell = oSubplot.GetCell(j);
          jOut.write("<po_point x=\"" + oCell.GetX() + "\" y=\"" + oCell.GetY() +
                     "\"/>");
        }
        jOut.write("</pointSet>");

        jOut.write("</so_subplot>");
      }
    }
    
    //Now write dead 
    for (i = 0; i < mp_oShortDeadTreeSaveSettings.length; i++) {
      for (j = 0; j < mp_oShortDeadTreeSaveSettings[i].length; j++) {
        if (mp_oShortDeadTreeSaveSettings[i][j].bSaveABA || 
            mp_oShortDeadTreeSaveSettings[i][j].bSaveADN ) {
          String sReason = ""; 
          switch (j) {
          case OutputBehaviors.DISEASE:
            sReason = "disease";
            break; 
          case OutputBehaviors.FIRE:
            sReason = "fire";
            break; 
          case OutputBehaviors.HARVEST:
            sReason = "harvest";
            break; 
          case OutputBehaviors.INSECTS:
            sReason = "insects";
            break; 
          case OutputBehaviors.NATURAL:
            sReason = "natural";
            break;           
          }
          jOut.write("<so_deadTreeTypeInfo type=\"" + 
              TreePopulation.GetTypeNameFromCode(i) + "\" reason=\"" +
              sReason + "\">");

          //Save absolute basal area
          if (i > TreePopulation.SEEDLING) {
            jOut.write("<so_saveABA save=\"");
            if (mp_oShortDeadTreeSaveSettings[i][j].bSaveABA) jOut.write("true");
            else jOut.write("false");
            jOut.write("\"/>");
          }

          //Save absolute density
          jOut.write("<so_saveADN save=\"");
          if (mp_oShortDeadTreeSaveSettings[i][j].bSaveADN) jOut.write("true");
          else jOut.write("false");
          jOut.write("\"/>");

          jOut.write("</so_deadTreeTypeInfo>");
        }
      }
    }

    jOut.write("</shortoutput>");

  }

  /**
   * Writes the parameter file data for detailed output.  Does nothing if there
   * are no detailed output save settings.
   * @param jOut File to write
   * @param oPop TreePopulation object
   * @throws java.io.IOException passed through from file writing
   * @throws ModelException passed through from tree population
   */
  protected void WriteDetailedOutputXML(BufferedWriter jOut,
                                        TreePopulation oPop) throws
      java.io.
      IOException, ModelException {

    Behavior oBeh = GetBehaviorByKey("detailed");
    if (!oBeh.m_bIsEnabled) {
      return;
    }

    //Just an extra float-check
    if (mp_oDetailedLiveTreeSaveSettings.size() == 0 &&
        mp_oDetailedDeadTreeSaveSettings.size() == 0 &&        
        mp_oDetailedGridSaveSettings.size() == 0) {
      return;
    }

    DetailedTreeSettings oTreeSettings;
    DetailedGridSettings oGridSettings;
    Subplot oSubplot;
    Cell oCell;
    String sTemp;
    int i, j;

    jOut.write("<output>");

    //Write filename
    WriteData(jOut, m_sDetailedOutputFilename);
    
    //Write subplot cell size, if applicable
    if (8.0 != m_fDetailedSubplotXCellLength.GetValue() ||
    		8.0 != m_fDetailedSubplotYCellLength.GetValue()) {
    	WriteData(jOut, m_fDetailedSubplotXCellLength);
    	WriteData(jOut, m_fDetailedSubplotYCellLength);
    }

    //Write live tree settings
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.
          elementAt(i);

      jOut.write("<ou_treeInfo species=\"" +
                 oPop.GetSpeciesNameFromCode(oTreeSettings.GetSpecies()) +
                 "\" type=\"" + TreePopulation.GetTypeNameFromCode(oTreeSettings.GetType()) +
                 "\">");

      //Save frequency
      jOut.write("<ou_saveFreq>" +
                 String.valueOf(oTreeSettings.GetSaveFrequency()) +
                 "</ou_saveFreq>");

      //Ints
      for (j = 0; j < oTreeSettings.GetNumberOfInts(); j++) {
        jOut.write("<ou_int>" + oTreeSettings.GetInt(j).GetCodeName() +
                   "</ou_int>");
      }

      //Floats
      for (j = 0; j < oTreeSettings.GetNumberOfFloats(); j++) {
        jOut.write("<ou_float>" + oTreeSettings.GetFloat(j).GetCodeName() +
                   "</ou_float>");
      }

      //Chars
      for (j = 0; j < oTreeSettings.GetNumberOfChars(); j++) {
        jOut.write("<ou_char>" + oTreeSettings.GetChar(j).GetCodeName() +
                   "</ou_char>");
      }

      //Bools
      for (j = 0; j < oTreeSettings.GetNumberOfBools(); j++) {
        jOut.write("<ou_bool>" + oTreeSettings.GetBool(j).GetCodeName() +
                   "</ou_bool>");
      }

      jOut.write("</ou_treeInfo>");
    }
    
    //Write dead tree settings
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.
          elementAt(i);
      
      switch (oTreeSettings.GetDeadCode()) {
      case HARVEST:
        sTemp = "harvest";
        break;
      case NATURAL:
        sTemp = "natural";
        break;
      case DISEASE:
        sTemp = "disease";
        break;
      case FIRE:
        sTemp = "fire";
        break;
      case INSECTS:
        sTemp = "insects";
        break;
      case STORM:
        sTemp = "storm";
        break;
      default:
        throw (new ModelException(ErrorGUI.BAD_DATA, "Output Setup",
            "Unexpected dead code reason"));        
      }

      jOut.write("<ou_deadTreeInfo species=\"" +
                 oPop.GetSpeciesNameFromCode(oTreeSettings.GetSpecies()) +
        "\" type=\"" + TreePopulation.GetTypeNameFromCode(oTreeSettings.GetType()) +
        "\" code=\"" + sTemp + "\">");

      //Save frequency
      jOut.write("<ou_saveFreq>" +
                 String.valueOf(oTreeSettings.GetSaveFrequency()) +
                 "</ou_saveFreq>");

      //Ints
      for (j = 0; j < oTreeSettings.GetNumberOfInts(); j++) {
        jOut.write("<ou_int>" + oTreeSettings.GetInt(j).GetCodeName() +
                   "</ou_int>");
      }

      //Floats
      for (j = 0; j < oTreeSettings.GetNumberOfFloats(); j++) {
        jOut.write("<ou_float>" + oTreeSettings.GetFloat(j).GetCodeName() +
                   "</ou_float>");
      }

      //Chars
      for (j = 0; j < oTreeSettings.GetNumberOfChars(); j++) {
        jOut.write("<ou_char>" + oTreeSettings.GetChar(j).GetCodeName() +
                   "</ou_char>");
      }

      //Bools
      for (j = 0; j < oTreeSettings.GetNumberOfBools(); j++) {
        jOut.write("<ou_bool>" + oTreeSettings.GetBool(j).GetCodeName() +
                   "</ou_bool>");
      }

      jOut.write("</ou_deadTreeInfo>");
    }

    //Write grids stuff
    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oGridSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.
          elementAt(i);

      jOut.write("<ou_gridInfo gridName=\"" + oGridSettings.GetName() + "\">");

      //Save frequency
      jOut.write("<ou_saveFreq>" +
                 String.valueOf(oGridSettings.GetSaveFrequency()) +
                 "</ou_saveFreq>");

      //Ints
      for (j = 0; j < oGridSettings.GetNumberOfInts(); j++) {
        jOut.write("<ou_int>" + oGridSettings.GetInt(j).GetCodeName() +
                   "</ou_int>");
      }

      //Floats
      for (j = 0; j < oGridSettings.GetNumberOfFloats(); j++) {
        jOut.write("<ou_float>" + oGridSettings.GetFloat(j).GetCodeName() +
                   "</ou_float>");
      }

      //Chars
      for (j = 0; j < oGridSettings.GetNumberOfChars(); j++) {
        jOut.write("<ou_char>" + oGridSettings.GetChar(j).GetCodeName() +
                   "</ou_char>");
      }

      //Bools
      for (j = 0; j < oGridSettings.GetNumberOfBools(); j++) {
        jOut.write("<ou_bool>" + oGridSettings.GetBool(j).GetCodeName() +
                   "</ou_bool>");
      }

      //Package ints
      for (j = 0; j < oGridSettings.GetNumberOfPackageInts(); j++) {
        jOut.write("<ou_packageInt>" +
                   oGridSettings.GetPackageInt(j).GetCodeName() +
                   "</ou_packageInt>");
      }

      //Package floats
      for (j = 0; j < oGridSettings.GetNumberOfPackageFloats(); j++) {
        jOut.write("<ou_packageFloat>" +
                   oGridSettings.GetPackageFloat(j).GetCodeName() +
                   "</ou_packageFloat>");
      }

      //Package chars
      for (j = 0; j < oGridSettings.GetNumberOfPackageChars(); j++) {
        jOut.write("<ou_packageChar>" +
                   oGridSettings.GetPackageChar(j).GetCodeName() +
                   "</ou_packageChar>");
      }

      //Package bools
      for (j = 0; j < oGridSettings.GetNumberOfPackageBools(); j++) {
        jOut.write("<ou_packageBool>" +
                   oGridSettings.GetPackageBool(j).GetCodeName() +
                   "</ou_packageBool>");
      }

      jOut.write("</ou_gridInfo>");
    }

    //Now write subplots - only if there are tree settings
    if (mp_oDetailedLiveTreeSaveSettings.size() > 0) {
      for (i = 0; i < mp_oDetailedOutputSubplots.size(); i++) {
        oSubplot = (Subplot) mp_oDetailedOutputSubplots.elementAt(i);
        if (oSubplot.GetNumberOfCells() > 0) {
          jOut.write("<ou_subplot>");
          jOut.write("<ou_subplotName>" + oSubplot.GetSubplotName() +
                     "</ou_subplotName>");
          jOut.write("<pointSet>");
          for (j = 0; j < oSubplot.GetNumberOfCells(); j++) {

            oCell = oSubplot.GetCell(j);
            jOut.write("<po_point x=\"" + oCell.GetX() + "\" y=\"" + oCell.GetY() +
                       "\"/>");
          }
          jOut.write("</pointSet>");

          jOut.write("</ou_subplot>");
        }
      }
    }

    jOut.write("</output>");
  }

  /**
   * Validates the data before writing to a parameter file.
   * @throws ModelException if the filename is empty but there are save
   * settings.
   * @param oPop Not used.
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    BehaviorTypeBase[] p_oBehaviorGroups = m_oManager.GetAllBehaviorGroups();
    DetailedGridSettings oGridSettings;
    DetailedTreeSettings oTreeSettings;
    Grid[] oSetGrids;
    Behavior oBeh;
    int i, j, k;
    boolean bEnabled;

    //If no behaviors are enabled, exit
    boolean bAnybodyEnabled = false;
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled) {
        bAnybodyEnabled = true;
      }
    }

    if (!bAnybodyEnabled) {
      return;
    }

    // Short output
    oBeh = GetBehaviorByKey("short output");

    if (oBeh.m_bIsEnabled && m_sShortOutputFilename.GetValue().length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "An output file name is required."));
    }

    //Detailed output

    //Verify that the grids to save are all still enabled.  It's possible for
    //a user to, say, save the substrate grid, then remove the substrate
    //behavior from the run.  This will then un-save the substrate grid.
    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oGridSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.
          elementAt(i);
      bEnabled = false;
      if (null != p_oBehaviorGroups) {
        for (j = 0; j < p_oBehaviorGroups.length; j++) {
          oSetGrids = p_oBehaviorGroups[j].GetEnabledGridObjects();
          if (null != oSetGrids) {
            for (k = 0; k < oSetGrids.length; k++) {
              if (oSetGrids[k].GetName().equals(oGridSettings.GetName())) {
                bEnabled = true;
                break;
              }
            }
          }
        }
      }

      if (false == bEnabled) {
        mp_oDetailedGridSaveSettings.remove(i);
        i--;
      }
    }

    //Verify that the tree data members to save are all still enabled, just
    //like the grids
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.
          elementAt(i);

      for (j = 0; j < oTreeSettings.GetNumberOfBools(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetBool(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveBool(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfFloats(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetFloat(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveFloat(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfInts(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetInt(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveInt(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfChars(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetChar(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveChar(j);
          j--;
        }
      }
    }
    
    //Verify that the tree data members to save are all still enabled, just
    //like the grids
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.
          elementAt(i);

      for (j = 0; j < oTreeSettings.GetNumberOfBools(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetBool(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveBool(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfFloats(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetFloat(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveFloat(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfInts(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetInt(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveInt(j);
          j--;
        }
      }

      for (j = 0; j < oTreeSettings.GetNumberOfChars(); j++) {
        if (false ==
            VerifyTreeMember(oTreeSettings.GetChar(j), oTreeSettings.GetSpecies(),
                             oTreeSettings.GetType())) {
          oTreeSettings.RemoveChar(j);
          j--;
        }
      }
    }

    //Make sure all detailed tree settings actually have settings in them -
    //if not, delete them
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.
          elementAt(i);
      if (oTreeSettings.GetNumberOfFloats() == 0 &&
          oTreeSettings.GetNumberOfInts() == 0 &&
          oTreeSettings.GetNumberOfBools() == 0 &&
          oTreeSettings.GetNumberOfChars() == 0) {
        mp_oDetailedLiveTreeSaveSettings.remove(i);
        i--;
      }
    }
    
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.
          elementAt(i);
      if (oTreeSettings.GetNumberOfFloats() == 0 &&
          oTreeSettings.GetNumberOfInts() == 0 &&
          oTreeSettings.GetNumberOfBools() == 0 &&
          oTreeSettings.GetNumberOfChars() == 0) {
        mp_oDetailedDeadTreeSaveSettings.remove(i);
        i--;
      }
    }

    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oGridSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.
          elementAt(i);
      if (oGridSettings.GetNumberOfFloats() == 0 &&
          oGridSettings.GetNumberOfInts() == 0 &&
          oGridSettings.GetNumberOfBools() == 0 &&
          oGridSettings.GetNumberOfChars() == 0 &&
          oGridSettings.GetNumberOfPackageFloats() == 0 &&
          oGridSettings.GetNumberOfPackageInts() == 0 &&
          oGridSettings.GetNumberOfPackageBools() == 0 &&
          oGridSettings.GetNumberOfPackageChars() == 0) {
        mp_oDetailedGridSaveSettings.remove(i);
        i--;
      }
    }

    oBeh = GetBehaviorByKey("detailed");
    if (mp_oDetailedLiveTreeSaveSettings.size() == 0 &&
        mp_oDetailedDeadTreeSaveSettings.size() == 0 &&
        mp_oDetailedGridSaveSettings.size() == 0) {
      oBeh.m_bIsEnabled = false;
    }

    //Detailed output
    if (oBeh.m_bIsEnabled && m_sDetailedOutputFilename.GetValue().length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "A detailed output file name is required."));
    }
  }

  /**
   * Verify that a tree member is correct.
   * @param oDataMember DataMember Data member to verify
   * @param iSpecies int Species for which to verify the data member
   * @param iType int Tree type for which to verify the data member
   * @throws ModelException Passing through from called functions
   * @return boolean True if the data member is valid for the tree species and
   * type, false if not
   */
  private boolean VerifyTreeMember(DataMember oDataMember, int iSpecies,
                                   int iType) throws ModelException {
    BehaviorTypeBase[] p_oBehaviorGroups = m_oManager.GetAllBehaviorGroups();
    Behavior[] p_oBehaviors;
    DataMember oBehDataMember;
    SpeciesTypeCombo oCombo;
    int i, j, k, m;

    //If this is a tree population data member, it's OK
    if (oDataMember.GetCodeName().equals("X") ||
        oDataMember.GetCodeName().equals("Y") ||
        oDataMember.GetCodeName().equals("Diam10") ||
        oDataMember.GetCodeName().equals("Height") ||
        oDataMember.GetCodeName().equals("DBH") ||
        oDataMember.GetCodeName().equals("Crown Radius") ||
        oDataMember.GetCodeName().equals("Crown Depth") ||
        oDataMember.GetCodeName().equals("Age")) {
      return true;
    }
    
    //First - find the behavior associated with this data member
    for (i = 0; i < p_oBehaviorGroups.length; i++) {
      p_oBehaviors = p_oBehaviorGroups[i].GetAllBehaviors();
      if (p_oBehaviors != null) {
        for (j = 0; j < p_oBehaviors.length; j++) {
          if (p_oBehaviors[j].m_bIsEnabled) {
            for (k = 0; k < p_oBehaviors[j].mp_oNewTreeDataMembers.size(); k++) {
              oBehDataMember = (DataMember) p_oBehaviors[j].
                  mp_oNewTreeDataMembers.elementAt(k);
              if (oBehDataMember.GetCodeName().equals(oDataMember.GetCodeName())) {

                //We found the data member - so see if this behavior is applied
                //to the species and type of the data member
                for (m = 0; m < p_oBehaviors[j].GetNumberOfCombos(); m++) {
                  oCombo = p_oBehaviors[j].GetSpeciesTypeCombo(m);

                  if (iType == oCombo.GetType() &&
                      iSpecies == oCombo.GetSpecies()) {
                    return true;
                  }
                }
              }
            }
          }
        }
      }
    }

    return false;
  }

  /**
   * Performs setup.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException Doesn't throw this exception.
   */
  public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {
    m_oPop = oPop;
    int i, j;

    //Make a tree settings object for each tree type
    if (mp_oShortTreeSaveSettings.size() == 0) {
      for (i = 0; i < TreePopulation.GetNumberOfTypes(); i++) {
        mp_oShortTreeSaveSettings.add(i, new TreeOutputSaveInfo());
      }
    }
    
    for (i = 0; i < TreePopulation.GetNumberOfTypes(); i++) {
      for (j = 0; j < NUMCODES; j++) {
        if (null == mp_oShortDeadTreeSaveSettings[i][j]) {
          mp_oShortDeadTreeSaveSettings[i][j] = new TreeOutputSaveInfo();
          mp_oShortDeadTreeSaveSettings[i][j].bIsDead = true;
        }
      }
    }

    //Check to see if this is being called as a change of species - and if
    //so, update the grids
    if (m_bChangeOfSpecies && mp_oDetailedGridSaveSettings.size() > 0) {

      //Check each display name in the grid settings. If it still exists,
      //change the code name to match. If it doesn't still exist, delete that
      //setting.
      DetailedGridSettings oSettings;
      Grid[] p_oGrids = m_oManager.GetAllGrids();
      DataMember[] p_oGridDataMembers, p_oGridPackageDataMembers;
      DataMember oDataMember;
      int k, m;
      boolean bFoundGrid, bFoundDataMember;
      for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
        oSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.elementAt(i);
        //Get the grid for these settings
        bFoundGrid = false;
        for (j = 0; j < p_oGrids.length; j++) {
          if (p_oGrids[j].GetName().equals(oSettings.GetName())) {
            bFoundGrid = true;
            p_oGridDataMembers = p_oGrids[j].GetDataMembers();
            p_oGridPackageDataMembers = p_oGrids[j].GetPackageDataMembers();

            if (null != p_oGridDataMembers) {
              //Check the floats
              for (k = 0; k < oSettings.GetNumberOfFloats(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetFloat(k);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemoveFloat(k);
                  k--;
                }
              }

              //Check the ints
              for (k = 0; k < oSettings.GetNumberOfInts(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetInt(k);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemoveInt(k);
                  k--;
                }
              }

              //Check the bools
              for (k = 0; k < oSettings.GetNumberOfBools(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetBool(k);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemoveBool(k);
                  k--;
                }
              }

              //Check the chars
              for (k = 0; k < oSettings.GetNumberOfChars(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetChar(k);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemoveChar(k);
                  k--;
                }
              }
            }

            if (null != p_oGridPackageDataMembers) {
              //Check the package floats
              for (k = 0; k < oSettings.GetNumberOfPackageFloats(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetPackageFloat(k);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridPackageDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridPackageDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridPackageDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemovePackageFloat(k);
                  k--;
                }
              }

              //Check the package ints
              for (k = 0; k < oSettings.GetNumberOfPackageInts(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetPackageInt(k);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridPackageDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridPackageDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridPackageDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemovePackageInt(k);
                  k--;
                }
              }

              //Check the package bools
              for (k = 0; k < oSettings.GetNumberOfPackageBools(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetPackageBool(k);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridPackageDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridPackageDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridPackageDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemovePackageBool(k);
                  k--;
                }
              }

              //Check the package chars
              for (k = 0; k < oSettings.GetNumberOfPackageChars(); k++) {
                bFoundDataMember = false;
                oDataMember = oSettings.GetPackageChar(k);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (oDataMember.GetDisplayName().equals(p_oGridPackageDataMembers[m].
                      GetDisplayName()) &&
                      oDataMember.GetType() == p_oGridPackageDataMembers[m].GetType()) {
                    bFoundDataMember = true;
                    oDataMember.SetCodeName(p_oGridPackageDataMembers[m].
                                            GetCodeName());
                  }
                }
                if (bFoundDataMember == false) {
                  oSettings.RemovePackageChar(k);
                  k--;
                }
              }
            }
          }
        }
        if (false == bFoundGrid) {
          mp_oDetailedGridSaveSettings.remove(i);
          i--;
        }
      }
    }
    m_bChangeOfSpecies = false;
  }

  /**
   * Displays the output setup options.
   * @param oWindow Main application window.
   */
  void DisplayWindow(MainWindow oWindow) {
    OutputSetup oOutputFrame = new OutputSetup(this, oWindow);
    oOutputFrame.pack();
    oOutputFrame.setLocationRelativeTo(oWindow);
    oOutputFrame.setVisible(true);
  }

  /**
   * This has its own display window; it won't contribute data to the display.
   * @param oPop TreePopulation object.
   * @return null.
   */
  public Vector<TableData> FormatDataForDisplay(TreePopulation oPop) {
    return null;
  }

  /**
   * Gets the tree population.
   * @return Tree population.
   */
  public TreePopulation GetTreePopulation() {
    return m_oPop;
  }

  /**
   * Deletes all detailed output tree settings.
   */
  public void ClearDetailedLiveTreeSettings() {
    mp_oDetailedLiveTreeSaveSettings.clear();

    //Make sure the detailed output behavior is not enabled if there is also
    //no grids
    Behavior oBeh = GetBehaviorByKey("detailed");
    if (mp_oDetailedGridSaveSettings.size() == 0 &&
        mp_oDetailedDeadTreeSaveSettings.size() == 0) {
      oBeh.m_bIsEnabled = false;
    }

    //Clear the behavior's apply to settings
   // oBeh.ClearSpeciesTypeCombos();
  }

  /**
   * Deletes all detailed output grid settings.
   */
  public void ClearDetailedGridSettings() {
    mp_oDetailedGridSaveSettings.clear();

    //Make sure the detailed output behavior is not enabled if there is also no grids
    Behavior oBeh = GetBehaviorByKey("detailed");
    if (mp_oDetailedLiveTreeSaveSettings.size() == 0 &&
        mp_oDetailedDeadTreeSaveSettings.size() == 0) {
      oBeh.m_bIsEnabled = false;
    }
  }
  
  /**
   * Deletes all detailed output tree settings for dead trees.
   */
  public void ClearDetailedDeadTreeSettings() {
    mp_oDetailedDeadTreeSaveSettings.clear();

    //Make sure the detailed output behavior is not enabled if there is also
    //no grids
    Behavior oBeh = GetBehaviorByKey("detailed");
    if (mp_oDetailedGridSaveSettings.size() == 0 &&
        mp_oDetailedLiveTreeSaveSettings.size() == 0) {
      oBeh.m_bIsEnabled = false;
    }

    //Clear the behavior's apply to settings
   // oBeh.ClearSpeciesTypeCombos();
  }

  /**
   * Adds a new detailed output tree settings object.  This will overwrite any
   * existing settings for that species and type.
   * @param oTreeSettings The new settings object.
   * @throws ModelException if either the type or the species name is invalid.
   */
  public void AddDetailedTreeSettings(DetailedTreeSettings oTreeSettings) throws
      ModelException {

    if (null == oTreeSettings) {
      return;
    }

    DetailedTreeSettings oData;
    int i,
        iSpecies = oTreeSettings.GetSpecies(),
        iCode = oTreeSettings.GetDeadCode(),
        iType = oTreeSettings.GetType();

    //Validate the type
    if (iType < 0 || iType >= TreePopulation.GetNumberOfTypes()) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Invalid type passed to output  - " +
                               String.valueOf(iType)));
    }

    //Validate the species
    if (iSpecies < 0 || iSpecies >= m_oPop.GetNumberOfSpecies()) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Invalid species passed to output - " +
                               String.valueOf(iSpecies)));
    }
    
    //Validate the dead reason code
    if (iCode < NOTDEAD || iCode >= NUMCODES) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
          "Invalid dead reason code passed to output - " +
          String.valueOf(iCode)));
    }

    //Check to see if this type/species combo exists already
    if (NOTDEAD == iCode) {
      for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
        oData = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.elementAt(i);
        if (iSpecies == oData.GetSpecies() && iType == oData.GetType()) {
          //Remove this one
          mp_oDetailedLiveTreeSaveSettings.remove(i);
          i--;
        }
      }
      mp_oDetailedLiveTreeSaveSettings.add(oTreeSettings);
    } else {
      for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
        oData = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.elementAt(i);
        if (iSpecies == oData.GetSpecies() && iType == oData.GetType() && 
            iCode == oData.GetDeadCode()) {
          //Remove this one
          mp_oDetailedDeadTreeSaveSettings.remove(i);
          i--;
        }
      }
      mp_oDetailedDeadTreeSaveSettings.add(oTreeSettings);
    }

    //Make sure the detailed output behavior is enabled
    Behavior oBeh = GetBehaviorByKey("detailed");
    oBeh.m_bIsEnabled = true;

    //Add this species/type combo
//    oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(oTreeSettings.GetSpecies(),
//                                                  oTreeSettings.GetType(),
//                                                  m_oPop));
  }

  /**
   * Adds a new detailed output grid settings object.  This will overwrite any
   * existing settings for that grid.
   * @param oTreeSettings The new settings object.
   */
  public void AddDetailedGridSettings(DetailedGridSettings oTreeSettings) {
    DetailedGridSettings oData;
    int i;

    if (null == oTreeSettings) {
      return;
    }

    //Check to see if this type/species combo exists already
    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oData = (DetailedGridSettings) mp_oDetailedGridSaveSettings.elementAt(i);
      if (oTreeSettings.GetName() == oData.GetName()) {
        //Remove this one
        mp_oDetailedGridSaveSettings.remove(i);
        i--;
      }
    }

    mp_oDetailedGridSaveSettings.add(oTreeSettings);

    //Make sure the detailed output behavior is enabled
    Behavior oBeh = GetBehaviorByKey("detailed");
    oBeh.m_bIsEnabled = true;
  }

  /**
   * Sets a data object's value.  Overriden from the base class to capture
   * output settings.
   * @param sXMLTag XML tag of data object whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Attributes of XML tag
   * @param oData Data value appropriate to the data type
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Attributes oAttributes,
                                        Object oData) throws
      ModelException {

    if (sXMLTag.equals("ou_float")) {

      //Check for outdated substrate grid names - this was added for
      //backwards compatibility for changes made for 6.05
      String sValue = String.valueOf(oData);
      if (sValue.equals("gapmoss")) {
        oData = "scarsoil";
      }
      else if (sValue.equals("canmoss")) {
        oData = "ffmoss"; //arbitrarily chosen between this and fflitter
      }
      else if (sValue.equals("organic")) {
        oData = "tipup";
      }
      else if (sValue.equals("mossylog")) {
        oData = "declog";
      }

      //Detailed output float value - tree or grid

      //If it's a grid, get its display name as well as its code name
      if (m_oCurrentDetailedSettings instanceof DetailedGridSettings) {
        String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
        Grid[] p_oGrids = m_oManager.GetAllGrids();
        for (int i = 0; i < p_oGrids.length; i++) {
          if (sGridName.equals(p_oGrids[i].GetName())) {
            DataMember[] p_oDataMembers = p_oGrids[i].GetDataMembers();
            if (null != p_oDataMembers) {
              for (int j = 0; j < p_oDataMembers.length; j++) {
                if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                  m_oCurrentDetailedSettings.AddFloat(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
                }
              }
            }
          }
        }
      } else {
        m_oCurrentDetailedSettings.AddFloat(String.valueOf(oData), String.valueOf(oData));
      }
      return true;
    }
    else if (sXMLTag.equals("ou_int")) {
      //Detailed output int value - tree or grid
      //If it's a grid, get its display name as well as its code name
      if (m_oCurrentDetailedSettings instanceof DetailedGridSettings) {
        String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
        Grid[] p_oGrids = m_oManager.GetAllGrids();
        for (int i = 0; i < p_oGrids.length; i++) {
          if (sGridName.equals(p_oGrids[i].GetName())) {
            DataMember[] p_oDataMembers = p_oGrids[i].GetDataMembers();
            if (null != p_oDataMembers) {
              for (int j = 0; j < p_oDataMembers.length; j++) {
                if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                  m_oCurrentDetailedSettings.AddInt(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
                }
              }
            }
          }
        }
      } else {
        m_oCurrentDetailedSettings.AddInt(String.valueOf(oData), String.valueOf(oData));
      }
      return true;
    }
    else if (sXMLTag.equals("ou_saveFreq")) {
      //Detailed output save frequency - tree or grid
      m_oCurrentDetailedSettings.SetSaveFrequency(
                         new Integer(String.valueOf(oData)).intValue());
      return true;
    }
    else if (sXMLTag.equals("ou_bool")) {
      //Detailed output bool value - tree or grid
      //If it's a grid, get its display name as well as its code name
      if (m_oCurrentDetailedSettings instanceof DetailedGridSettings) {
        String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
        Grid[] p_oGrids = m_oManager.GetAllGrids();
        for (int i = 0; i < p_oGrids.length; i++) {
          if (sGridName.equals(p_oGrids[i].GetName())) {
            DataMember[] p_oDataMembers = p_oGrids[i].GetDataMembers();
            if (null != p_oDataMembers) {
              for (int j = 0; j < p_oDataMembers.length; j++) {
                if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                  m_oCurrentDetailedSettings.AddBool(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
                }
              }
            }
          }
        }
      } else {
        //Trap for deprecated boolean tree data member "dead"
        if (String.valueOf(oData).equals("dead")) {
          m_oCurrentDetailedSettings.AddInt(String.valueOf(oData), String.valueOf(oData));
        } else {
          m_oCurrentDetailedSettings.AddBool(String.valueOf(oData), String.valueOf(oData));
        }
      }
      return true;
    }
    else if (sXMLTag.equals("ou_char")) {
      //Detailed output char value - tree or grid
      //If it's a grid, get its display name as well as its code name
      if (m_oCurrentDetailedSettings instanceof DetailedGridSettings) {
        String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
        Grid[] p_oGrids = m_oManager.GetAllGrids();
        for (int i = 0; i < p_oGrids.length; i++) {
          if (sGridName.equals(p_oGrids[i].GetName())) {
            DataMember[] p_oDataMembers = p_oGrids[i].GetDataMembers();
            if (null != p_oDataMembers) {
              for (int j = 0; j < p_oDataMembers.length; j++) {
                if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                  m_oCurrentDetailedSettings.AddChar(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
                }
              }
            }
          }
        }
      } else {
        m_oCurrentDetailedSettings.AddChar(String.valueOf(oData), String.valueOf(oData));
      }
      return true;
    }
    else if (sXMLTag.equals("ou_packageFloat")) {
      //Check for outdated substrate grid names - this was added for
      //backwards compatibility for changes made for 6.05
      String sValue = String.valueOf(oData);
      if (sValue.equals("gapmoss")) {
        oData = "scarsoil";
      }
      else if (sValue.equals("organic")) {
        oData = "tipup";
      }

      //Detailed output package float value for grid
      String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
      Grid[] p_oGrids = m_oManager.GetAllGrids();
      for (int i = 0; i < p_oGrids.length; i++) {
        if (sGridName.equals(p_oGrids[i].GetName())) {
          DataMember[] p_oDataMembers = p_oGrids[i].GetPackageDataMembers();
          if (null != p_oDataMembers) {
            for (int j = 0; j < p_oDataMembers.length; j++) {
              if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                ((DetailedGridSettings)m_oCurrentDetailedSettings).AddPackageFloat(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
              }
            }
          }
        }
      }
      return true;
    }
    else if (sXMLTag.equals("ou_packageInt")) {
      //Detailed output package int value for grid
      String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
      Grid[] p_oGrids = m_oManager.GetAllGrids();
      for (int i = 0; i < p_oGrids.length; i++) {
        if (sGridName.equals(p_oGrids[i].GetName())) {
          DataMember[] p_oDataMembers = p_oGrids[i].GetPackageDataMembers();
          if (null != p_oDataMembers) {
            for (int j = 0; j < p_oDataMembers.length; j++) {
              if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                ((DetailedGridSettings)m_oCurrentDetailedSettings).AddPackageInt(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
              }
            }
          }
        }
      }
      return true;
    }
    else if (sXMLTag.equals("ou_packageBool")) {
      //Detailed output package bool value for grid
      String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
      Grid[] p_oGrids = m_oManager.GetAllGrids();
      for (int i = 0; i < p_oGrids.length; i++) {
        if (sGridName.equals(p_oGrids[i].GetName())) {
          DataMember[] p_oDataMembers = p_oGrids[i].GetPackageDataMembers();
          if (null != p_oDataMembers) {
            for (int j = 0; j < p_oDataMembers.length; j++) {
              if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                ((DetailedGridSettings)m_oCurrentDetailedSettings).AddPackageBool(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
              }
            }
          }
        }
      }
      return true;
    }
    else if (sXMLTag.equals("ou_packageChar")) {
      //Detailed output package char value for grid
      String sGridName = ((DetailedGridSettings) m_oCurrentDetailedSettings).GetName();
      Grid[] p_oGrids = m_oManager.GetAllGrids();
      for (int i = 0; i < p_oGrids.length; i++) {
        if (sGridName.equals(p_oGrids[i].GetName())) {
          DataMember[] p_oDataMembers = p_oGrids[i].GetPackageDataMembers();
          if (null != p_oDataMembers) {
            for (int j = 0; j < p_oDataMembers.length; j++) {
              if (p_oDataMembers[j].GetCodeName().equals(oData)) {
                ((DetailedGridSettings)m_oCurrentDetailedSettings).AddPackageChar(String.valueOf(oData), p_oDataMembers[j].GetDisplayName());
              }
            }
          }
        }
      }
      return true;
    }

    else if (sXMLTag.equals("so_subplotName")) {
      //Subplot name - create the subplot
      m_oCurrentSubplot = AddShortOutputSubplot( (String) oData);
      return true;
    }

    else if (sXMLTag.equals("ou_subplotName")) {
      //Subplot name - create the subplot
      m_oCurrentSubplot = AddDetailedOutputSubplot( (String) oData);
      return true;
    }


    return super.SetSingleValueByXMLTag(sXMLTag, sXMLTag, oAttributes, oData);
  }

  /**
   * Sets a data vector's value.  Overridden from the base class.  Due to
   * the vagaries of parameter file parsing, our values may come in through
   * here instead of in single values because there could be repeats.
   * @param sXMLTag Parent XML tag of data vector whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param p_oData Vector of data values appropriate to the data type
   * @param p_sChildXMLTags The XML tags of the child elements
   * @param p_bAppliesTo Array of booleans saying which of the vector values
   * should be set.  This is important in the case of species-specifics - the
   * vector index is the species number but not all species are set.
   * @param oParentAttributes Attributes of parent tag.  May be useful when
   * overridding this for unusual tags.
   * @param p_oAttributes Attributes passed from parser.  This may be needed
   * when overriding this function.  Basic species-specific values are
   * already handled by this function.
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)  If a match to a data object is made via XML tag, but the found
   * object is not a ModelVector, this returns false.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Vector<String> p_oData,
                                        String[] p_sChildXMLTags,
                                        boolean[] p_bAppliesTo,
                                        Attributes oParentAttributes,
                                        Attributes[] p_oAttributes) throws
      ModelException {

    int i;
    boolean bReturn = false;
    for (i = 0; i < p_sChildXMLTags.length; i++) {
      if (p_sChildXMLTags[i] != null) {
        if (SetSingleValueByXMLTag(p_sChildXMLTags[i], sXMLParentTag,
                                   p_oAttributes[i], p_oData.elementAt(i))) {
          bReturn = true;
        }
      }
    }

    return bReturn;
  }

  /**
   * Accepts an XML parent tag (empty, no data) from the parser.  This function
   * watches for the following output tags:
   * <ul>
   * <li>ou_treeInfo
   * <li>ou_gridInfo
   * <li>so_treeTypeInfo
   * <li>so_saveRBA
   * <li>so_saveABA
   * <li>so_saveRDN
   * <li>so_saveADN
   * <li>po_point
   * </ul>
   * @param sXMLTag The XML tag.
   * @param oAttributes The attributes of this object.
   * @throws ModelException if data is missing or invalid.
   */
  public void ReadXMLParentTag(String sXMLTag, Attributes oAttributes) throws
      ModelException {

    if (sXMLTag.equals("ou_treeInfo")) {

      //This is detailed output tree info

      //Extract species and type
      int iSpecies = m_oPop.GetSpeciesCodeFromName(oAttributes.getValue(
          "species")),
          iType = TreePopulation.GetTypeCodeFromName(oAttributes.getValue("type"));

      if (iSpecies == -1 || iType == -1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Attributes missing from tag \"ou_treeInfo\" "
                                 + "- \"species\" and/or \"type\""));
      }

      m_oCurrentDetailedSettings = new DetailedTreeSettings(iType, iSpecies);
      AddDetailedTreeSettings( (DetailedTreeSettings)
                              m_oCurrentDetailedSettings);

    }
    else if (sXMLTag.equals("ou_deadTreeInfo")) {

      //This is detailed output dead tree info
      
      //Extract species and type
      int iSpecies = m_oPop.GetSpeciesCodeFromName(oAttributes.getValue(
          "species")),
          iType = TreePopulation.GetTypeCodeFromName(oAttributes.getValue("type")),
          iCode;
      
      String sTemp = oAttributes.getValue("code");
      if (sTemp.equals("harvest")) {
        iCode = HARVEST;
      } else if (sTemp.equals("natural")) {
        iCode = NATURAL;
      } else if (sTemp.equals("disease")) {
        iCode = DISEASE;
      } else if (sTemp.equals("fire")) {
        iCode = FIRE;
      } else if (sTemp.equals("insects")) {
        iCode = INSECTS;
      } else if (sTemp.equals("storm")) {
        iCode = STORM;
      } else {
        throw (new ModelException(ErrorGUI.BAD_DATA, "Output Setup",
            "Unexpected dead code reason" + sTemp));        
      }

      if (iSpecies == -1 || iType == -1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Attributes missing from tag \"ou_treeInfo\" "
                                 + "- \"species\" and/or \"type\""));
      }

      m_oCurrentDetailedSettings = new DetailedTreeSettings(iType, iSpecies, iCode);
      AddDetailedTreeSettings( (DetailedTreeSettings)
                              m_oCurrentDetailedSettings);

    }
    else if (sXMLTag.equals("ou_gridInfo")) {

      //This is detailed output grid info

      //Extract the grid name
      String sGridName = oAttributes.getValue("gridName");
      if (sGridName == null) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                 "Attribute missing from tag \"ou_gridInfo\" - \"gridName\""));
      }

      m_oCurrentDetailedSettings = new DetailedGridSettings(sGridName);
      AddDetailedGridSettings( (DetailedGridSettings)
                              m_oCurrentDetailedSettings);
    }
    else if (sXMLTag.equals("so_treeTypeInfo")) {

      //This is short output tree info

      //Extract type
      m_iCurrentType = TreePopulation.GetTypeCodeFromName(oAttributes.getValue("type"));

      if (m_iCurrentType == -1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                 "Attribute missing from tag \"so_treeTypeInfo\" - \"type\""));
      }
      m_iCurrentDeadCode = NOTDEAD;
    }
    else if (sXMLTag.equals("so_deadTreeTypeInfo")) {

      //This is short output tree info

      //Extract type
      m_iCurrentType = TreePopulation.GetTypeCodeFromName(oAttributes.getValue("type"));

      if (m_iCurrentType == -1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                 "Attribute missing from tag \"so_treeTypeInfo\" - \"type\""));
      }
      String sReason = oAttributes.getValue("reason");
      if (sReason.equalsIgnoreCase("harvest")) m_iCurrentDeadCode = HARVEST;
      else if (sReason.equalsIgnoreCase("natural")) m_iCurrentDeadCode = NATURAL;
      else if (sReason.equalsIgnoreCase("disease")) m_iCurrentDeadCode = DISEASE;
      else if (sReason.equalsIgnoreCase("fire")) m_iCurrentDeadCode = FIRE;
      else if (sReason.equalsIgnoreCase("insects"))  m_iCurrentDeadCode = INSECTS;
      else if (sReason.equalsIgnoreCase("storm")) m_iCurrentDeadCode = STORM;
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "Unrecognized dead reason code attribute \" + " + sReason + 
        "\" for tag \"so_deadTreeTypeInfo\"."));
      }      
    }
    else if (sXMLTag.equals("so_saveRBA")) {
      //Short output - save relative basal area
      //Get whether or not to save
      String sSave = oAttributes.getValue("save");
      boolean bSave;
      if (sSave == null) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" missing from XML"
                                 + "tag \"so_saveRBA\" in input file."));
      }
      if (sSave.equals("true")) {
        bSave = true;
      }
      else if (sSave.equals("false")) {
        bSave = false;
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" of XML tag " +
                                 "\"so_saveRBA\" in input file has unrecognized"
                                 + " value \"" + sSave + "\"."));
      }
      SetSaveRelativeBasalArea(m_iCurrentType, bSave);
    }
    else if (sXMLTag.equals("so_saveABA")) {
      //Short output - save absolute basal area
      //Get whether or not to save
      String sSave = oAttributes.getValue("save");
      boolean bSave;
      if (sSave == null) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" missing from XML"
                                 + " tag \"so_saveABA\" in input file."));
      }
      if (sSave.equals("true")) {
        bSave = true;
      }
      else if (sSave.equals("false")) {
        bSave = false;
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" of XML tag " +
                                 "\"so_saveABA\" in input file has unrecognized"
                                 + " value \"" + sSave + "\"."));
      }
      SetSaveAbsoluteBasalArea(m_iCurrentType, m_iCurrentDeadCode, bSave);
    }
    else if (sXMLTag.equals("so_saveRDN")) {
      //Short output - save relative density
      //Get whether or not to save
      String sSave = oAttributes.getValue("save");
      boolean bSave;
      if (sSave == null) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" missing from XML"
                                 + " tag \"so_saveRDN\" in input file."));
      }
      if (sSave.equals("true")) {
        bSave = true;
      }
      else if (sSave.equals("false")) {
        bSave = false;
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" of XML tag " +
                                 "\"so_saveRDN\" in input file has unrecognized"
                                 + " value \"" + sSave + "\"."));
      }
      SetSaveRelativeDensity(m_iCurrentType, bSave);
    }
    else if (sXMLTag.equals("so_saveADN")) {
      //Short output - save absolute density
      //Get whether or not to save
      String sSave = oAttributes.getValue("save");
      boolean bSave;
      if (sSave == null) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" missing from XML"
                                 + " tag \"so_saveADN\" in input file."));
      }
      if (sSave.equals("true")) {
        bSave = true;
      }
      else if (sSave.equals("false")) {
        bSave = false;
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "Output - attribute \"save\" of XML tag " +
                                 "\"so_saveADN\" in input file has unrecognized"
                                 + " value \"" + sSave + "\"."));
      }
      SetSaveAbsoluteDensity(m_iCurrentType, m_iCurrentDeadCode, bSave);
    }
    else if (sXMLTag.equals("po_point") && m_oCurrentSubplot != null) {
      Plot oPlot = m_oManager.GetPlot();

      int iX = new Integer( (String) oAttributes.getValue("x")).
          intValue(),
          iY = new Integer( (String) oAttributes.getValue("y")).
          intValue();
      m_oCurrentSubplot.AddCell(iX, iY, oPlot);
    }
    super.ReadXMLParentTag(sXMLTag, oAttributes);
  }

  /**
   * Checks detailed output settings upon change of species.
   * @param iOldNumSpecies says how many species there used to be.
   * @param p_iIndexer is an array, sized to the new number of species.  For
   * each bucket (representing the index number of a species on the new list),
   * the value is either the index of that same species in the old species
   * list, or -1 if the species is new.
   * @param p_sNewSpecies The new species list.
   */
  public void ChangeOfSpecies(int iOldNumSpecies, int[] p_iIndexer,
                              String[] p_sNewSpecies) throws ModelException {
    DetailedTreeSettings oTree;
    int i, j, iOldSpecies;
    boolean bFound;
    
    super.ChangeOfSpecies(iOldNumSpecies, p_iIndexer, p_sNewSpecies);

    //Go through all the trees and check for species that have been removed
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTree = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.elementAt(i);
      iOldSpecies = oTree.GetSpecies();

      //Look for this species in the new species list - change the species if
      //we find it
      bFound = false;
      for (j = 0; j < p_iIndexer.length; j++) {
        if (p_iIndexer[j] == iOldSpecies) {
          //Yep, this species is still on the list - change the species
          //number
          oTree.SetSpecies(j);
          mp_oDetailedLiveTreeSaveSettings.remove(i);
          mp_oDetailedLiveTreeSaveSettings.add(i, oTree);
          bFound = true;
          break;
        }
      }

      if (false == bFound) {
        //The species has been deleted - remove this tree setting
        mp_oDetailedLiveTreeSaveSettings.remove(i);
        i--;
      }
    }
    
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTree = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.elementAt(i);
      iOldSpecies = oTree.GetSpecies();

      //Look for this species in the new species list - change the species if
      //we find it
      bFound = false;
      for (j = 0; j < p_iIndexer.length; j++) {
        if (p_iIndexer[j] == iOldSpecies) {
          //Yep, this species is still on the list - change the species
          //number
          oTree.SetSpecies(j);
          mp_oDetailedDeadTreeSaveSettings.remove(i);
          mp_oDetailedDeadTreeSaveSettings.add(i, oTree);
          bFound = true;
          break;
        }
      }

      if (false == bFound) {
        //The species has been deleted - remove this tree setting
        mp_oDetailedDeadTreeSaveSettings.remove(i);
        i--;
      }
    }

    m_bChangeOfSpecies = true;
  }

  /**
  * Updates output settings when species are copied.
  * @param iSpeciesCopyFrom int Species to copy.
  * @param iSpeciesCopyTo int Species that is the copy.
  * @throws ModelException if there is a problem.
  */
  public void CopySpecies (int iSpeciesCopyFrom, int iSpeciesCopyTo) throws ModelException {
    super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);

    Grid[] p_oGrids = m_oManager.GetAllGrids();
    DetailedGridSettings oGridSettings;
    DetailedTreeSettings oTreeSettings, oNewTreeSettings;
    DataMember[] p_oGridDataMembers, p_oGridPackageDataMembers;
    DataMember oDataMember;
    String sCopyFrom = m_oManager.GetTreePopulation().GetSpeciesNameFromCode(iSpeciesCopyFrom).replace('_', ' '),
           sCopyTo = m_oManager.GetTreePopulation().GetSpeciesNameFromCode(iSpeciesCopyTo).replace('_', ' '),
           sCopyToName;
    int i, j, k, m;
    boolean bFoundGrid;
    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oGridSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.elementAt(i);

      //Delete any settings that are for the species that will become a copy
      //Floats
      for (j = 0; j < oGridSettings.GetNumberOfFloats(); j++) {
        oDataMember = oGridSettings.GetFloat(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemoveFloat(j);
          j--;
        }
      }

      //Ints
      for (j = 0; j < oGridSettings.GetNumberOfInts(); j++) {
        oDataMember = oGridSettings.GetInt(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemoveInt(j);
          j--;
        }
      }

      //Bools
      for (j = 0; j < oGridSettings.GetNumberOfBools(); j++) {
        oDataMember = oGridSettings.GetBool(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemoveBool(j);
          j--;
        }
      }

      //Chars
      for (j = 0; j < oGridSettings.GetNumberOfChars(); j++) {
        oDataMember = oGridSettings.GetChar(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemoveChar(j);
          j--;
        }
      }

      //Package Floats
      for (j = 0; j < oGridSettings.GetNumberOfPackageFloats(); j++) {
        oDataMember = oGridSettings.GetPackageFloat(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemovePackageFloat(j);
          j--;
        }
      }

      //Package Ints
      for (j = 0; j < oGridSettings.GetNumberOfPackageInts(); j++) {
        oDataMember = oGridSettings.GetPackageInt(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemovePackageInt(j);
          j--;
        }
      }

      //Package Chars
      for (j = 0; j < oGridSettings.GetNumberOfPackageChars(); j++) {
        oDataMember = oGridSettings.GetPackageChar(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemovePackageChar(j);
          j--;
        }
      }

      //Package Bools
      for (j = 0; j < oGridSettings.GetNumberOfPackageBools(); j++) {
        oDataMember = oGridSettings.GetPackageBool(j);
        if (oDataMember.GetDisplayName().indexOf(sCopyTo) > -1) {
          oGridSettings.RemovePackageBool(j);
          j--;
        }
      }
    }

    //Now float any settings for the species that is being copied
    for (i = 0; i < mp_oDetailedGridSaveSettings.size(); i++) {
      oGridSettings = (DetailedGridSettings) mp_oDetailedGridSaveSettings.elementAt(i);

      //Find the grid so we know what the data members look like for a copied
      //species
      bFoundGrid = false;
      for (j = 0; j < p_oGrids.length; j++) {
        if (p_oGrids[j].GetName().equals(oGridSettings.GetName())) {
          bFoundGrid = true;
          p_oGridDataMembers = p_oGrids[j].GetDataMembers();
          p_oGridPackageDataMembers = p_oGrids[j].GetPackageDataMembers();

          if (null != p_oGridDataMembers) {

            //Floats
            for (k = 0; k < oGridSettings.GetNumberOfFloats(); k++) {
              oDataMember = oGridSettings.GetFloat(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (p_oGridDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddFloat(p_oGridDataMembers[m].GetCodeName(), p_oGridDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Ints
            for (k = 0; k < oGridSettings.GetNumberOfInts(); k++) {
              oDataMember = oGridSettings.GetInt(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (p_oGridDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddInt(p_oGridDataMembers[m].GetCodeName(), p_oGridDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Bools
            for (k = 0; k < oGridSettings.GetNumberOfBools(); k++) {
              oDataMember = oGridSettings.GetBool(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (p_oGridDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddBool(p_oGridDataMembers[m].GetCodeName(), p_oGridDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Chars
            for (k = 0; k < oGridSettings.GetNumberOfChars(); k++) {
              oDataMember = oGridSettings.GetChar(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridDataMembers.length; m++) {
                  if (p_oGridDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddChar(p_oGridDataMembers[m].GetCodeName(), p_oGridDataMembers[m].GetDisplayName());
                  }
                }
              }
            }
          }

          //Repeat with packages
          if (null != p_oGridPackageDataMembers) {

            //Floats
            for (k = 0; k < oGridSettings.GetNumberOfPackageFloats(); k++) {
              oDataMember = oGridSettings.GetPackageFloat(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (p_oGridPackageDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddPackageFloat(p_oGridPackageDataMembers[m].GetCodeName(), p_oGridPackageDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Ints
            for (k = 0; k < oGridSettings.GetNumberOfPackageInts(); k++) {
              oDataMember = oGridSettings.GetPackageInt(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (p_oGridPackageDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddPackageInt(p_oGridPackageDataMembers[m].GetCodeName(), p_oGridPackageDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Bools
            for (k = 0; k < oGridSettings.GetNumberOfPackageBools(); k++) {
              oDataMember = oGridSettings.GetPackageBool(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (p_oGridPackageDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddPackageBool(p_oGridPackageDataMembers[m].GetCodeName(), p_oGridPackageDataMembers[m].GetDisplayName());
                  }
                }
              }
            }

            //Chars
            for (k = 0; k < oGridSettings.GetNumberOfPackageChars(); k++) {
              oDataMember = oGridSettings.GetPackageChar(k);
              if (oDataMember.GetDisplayName().indexOf(sCopyFrom) > -1) {
                //Go through the grid's data members and find the data member
                //for the species that is becoming a copy, so we can add it

                //We need to guess at what the name will be
                sCopyToName = oDataMember.GetDisplayName().replaceAll(sCopyFrom, sCopyTo);
                for (m = 0; m < p_oGridPackageDataMembers.length; m++) {
                  if (p_oGridPackageDataMembers[m].GetDisplayName().equals(sCopyToName)) {
                    oGridSettings.AddPackageChar(p_oGridPackageDataMembers[m].GetCodeName(), p_oGridPackageDataMembers[m].GetDisplayName());
                  }
                }
              }
            }
          }
        }
      }

      if (false == bFoundGrid) {
        mp_oDetailedGridSaveSettings.remove(i);
        i--;
      }
      if (0 == oGridSettings.GetNumberOfBools() &&
          0 == oGridSettings.GetNumberOfFloats() &&
          0 == oGridSettings.GetNumberOfInts() &&
          0 == oGridSettings.GetNumberOfChars() &&
          0 == oGridSettings.GetNumberOfPackageBools() &&
          0 == oGridSettings.GetNumberOfPackageFloats() &&
          0 == oGridSettings.GetNumberOfPackageInts() &&
          0 == oGridSettings.GetNumberOfPackageChars()) {
        mp_oDetailedGridSaveSettings.remove(i);
        i--;
      }
    }

    //Update tree settings

    //Go through all the trees and remove any instances of the species that is
    //becoming a copy; if we find the species being copied, copy it
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.elementAt(i);
      if (oTreeSettings.GetSpecies() == iSpeciesCopyTo) {
        mp_oDetailedLiveTreeSaveSettings.remove(i);
        i--;
      }
    }
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.elementAt(i);
      if (oTreeSettings.GetSpecies() == iSpeciesCopyTo) {
        mp_oDetailedDeadTreeSaveSettings.remove(i);
        i--;
      }
    }
    for (i = 0; i < mp_oDetailedLiveTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedLiveTreeSaveSettings.
          elementAt(i);
      if (oTreeSettings.GetSpecies() == iSpeciesCopyFrom) {
        oNewTreeSettings = (DetailedTreeSettings) oTreeSettings.clone();
        oNewTreeSettings.SetSpecies(iSpeciesCopyTo);
        mp_oDetailedLiveTreeSaveSettings.add(oNewTreeSettings);
      }
    }
    for (i = 0; i < mp_oDetailedDeadTreeSaveSettings.size(); i++) {
      oTreeSettings = (DetailedTreeSettings) mp_oDetailedDeadTreeSaveSettings.
          elementAt(i);
      if (oTreeSettings.GetSpecies() == iSpeciesCopyFrom) {
        oNewTreeSettings = (DetailedTreeSettings) oTreeSettings.clone();
        oNewTreeSettings.SetSpecies(iSpeciesCopyTo);
        mp_oDetailedDeadTreeSaveSettings.add(oNewTreeSettings);
      }
    }
  }

  /**
   * Removes cells from subplots that are outside the plot.
   * @param fOldX float Old plot X length.
   * @param fOldY float Old plot Y length.
   * @param fNewX float New plot X length.
   * @param fNewY float New plot Y length.
   * @throws ModelException won't.
   */
  public void ChangeOfPlotResolution(float fOldX, float fOldY, float fNewX,
                                   float fNewY) throws ModelException {

    //If the plot has shrunk, check for cells now outside the plot
    if (fOldX > fNewX || fOldY > fNewY) {
      Subplot oSubplot;
      Cell oCell;
      float fCellLength = (float)8.0;
      int iNewMaxX = (int) java.lang.Math.ceil(fNewX / fCellLength),
          iNewMaxY = (int) java.lang.Math.ceil(fNewY / fCellLength),
          i, j;

      for (i = 0; i < mp_oDetailedOutputSubplots.size(); i++) {
        oSubplot = (Subplot) mp_oDetailedOutputSubplots.elementAt(i);
        for (j = 0; j < oSubplot.GetNumberOfCells(); j++) {
          oCell = (Cell) oSubplot.GetCell(j);
          if (oCell.GetX() >= iNewMaxX || oCell.GetY() >= iNewMaxY) {
            oSubplot.RemoveCell(j);
            j--;
          }
        }
        //If there's no cells left remove this subplot
        if (oSubplot.GetNumberOfCells() == 0) {
          mp_oDetailedOutputSubplots.remove(i);
          i--;
        }
      }

      for (i = 0; i < mp_oShortOutputSubplots.size(); i++) {
        oSubplot = (Subplot) mp_oShortOutputSubplots.elementAt(i);
        for (j = 0; j < oSubplot.GetNumberOfCells(); j++) {
          oCell = (Cell) oSubplot.GetCell(j);
          if (oCell.GetX() >= iNewMaxX || oCell.GetY() >= iNewMaxY) {
            oSubplot.RemoveCell(j);
            j--;
          }
        }
        //If there's no cells left remove this subplot
        if (oSubplot.GetNumberOfCells() == 0) {
          mp_oShortOutputSubplots.remove(i);
          i--;
        }
      }
    }
  }
}
