package javawrapper;


import java.io.FileWriter;
import java.io.File;


/**
 * Tests certain behaviors of the DisturbanceBehavior class.
 * 
 * Copyright: Copyright (c) Charles D. Canham 2007
 * Company: Institute of Ecosystem Studies
 * 
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestOtherDisturbanceBehaviors extends ModelTestCase {

	/**
	 * This test makes sure that lambdas are reset correctly if species are
	 * changed.
	 */
	public void testDoSetup() {
		try {
			Behavior oBeh;
			Float[] p_fFloat;
			ModelVector p_oData;
			ModelData oData;
			String sDescriptor;
			int i;

			// First - set up the way it is
			GUIManager oManager = new GUIManager(null);
			oManager.ClearCurrentData();
			oManager.GetTreePopulation().SetSpeciesNames(
					new String[] { "ACRU", "ACSA" });
			DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
			oDist.DoSetup(oManager.GetTreePopulation());

			// Set some lambda values
			p_fFloat = new Float[] { new Float(0.01), new Float(0.02) };
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
				oData = (ModelData) oBeh.GetRequiredData(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					p_oData = (ModelVector) oBeh.GetRequiredData(i);
					oDist.SetVectorValues(p_oData, p_fFloat);
					break;
				}
			}

			p_fFloat[0] = new Float(0.11);
			p_fFloat[1] = new Float(0.12);
			for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
				oData = (ModelData) oBeh.GetRequiredData(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					p_oData = (ModelVector) oBeh.GetRequiredData(i + 1);
					oDist.SetVectorValues(p_oData, p_fFloat);
					break;
				}
			}

			// Make sure there are ACRU and ACSA lambda neighbors in the whole
			// data list
			boolean bFoundACRU = false, bFoundACSA = false;

			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
					} else if (sDescriptor.indexOf("acsa") > -1) {
						if (bFoundACSA) {
							fail("Found ACSA lambda more than once.");
						}
						bFoundACSA = true;
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);
			assertTrue(bFoundACSA);

			bFoundACRU = false;
			bFoundACSA = false;

			// Now check the individual behaviors
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			if (oBeh == null) {
				fail("Can't find Competition Harvest behavior.");
			}
			for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
				oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
					} else if (sDescriptor.indexOf("acsa") > -1) {
						if (bFoundACSA) {
							fail("Found ACSA lambda more than once.");
						}
						bFoundACSA = true;
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);
			assertTrue(bFoundACSA);

			// ************************
			// Now remove a species
			// ************************
			oManager.GetTreePopulation().SetSpeciesNames(new String[] { "ACRU" });
			// Make sure there are ACRU lambda neighbors only in the whole
			// data list
			bFoundACRU = false;
			bFoundACSA = false;

			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
						// Make sure it has the values we set for it
						p_oData = (ModelVector) oData;
						assertEquals(0.01, ((Float) p_oData.GetValue().elementAt(0))
								.floatValue(), 0.001);
					} else if (sDescriptor.indexOf("acsa") > -1) {
						fail("Found ACSA lambda in all data when it should be gone.");
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);

			bFoundACRU = false;
			bFoundACSA = false;

			// Now check the individual behaviors
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			if (oBeh == null) {
				fail("Can't find Competition Harvest behavior.");
			}
			for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
				oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
					} else if (sDescriptor.indexOf("acsa") > -1) {
						fail("Found ACSA lambda in behavior when it should be gone.");
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);

			// ************************
			// Reset our species back to normal
			// ************************
			oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
			// Make sure there are ACRU and ACSA lambda neighbors in the whole
			// data list
			bFoundACRU = false;
			bFoundACSA = false;

			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
					} else if (sDescriptor.indexOf("acsa") > -1) {
						if (bFoundACSA) {
							fail("Found ACSA lambda more than once.");
						}
						bFoundACSA = true;
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);
			assertTrue(bFoundACSA);

			bFoundACRU = false;
			bFoundACSA = false;

			// Now check the individual behaviors
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			if (oBeh == null) {
				fail("Can't find Competition Harvest behavior.");
			}
			for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
				oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
				sDescriptor = oData.GetDescriptor().toLowerCase();
				if (sDescriptor.indexOf("lambda") > -1) {
					if (sDescriptor.indexOf("acru") > -1) {
						if (bFoundACRU) {
							fail("Found ACRU lambda more than once.");
						}
						bFoundACRU = true;
					} else if (sDescriptor.indexOf("acsa") > -1) {
						if (bFoundACSA) {
							fail("Found ACSA lambda more than once.");
						}
						bFoundACSA = true;
					} else {
						fail("Found some lambda that I can't figure out what the hell it is.");
					}
				}
			}

			// Make sure we found everything
			assertTrue(bFoundACRU);
			assertTrue(bFoundACSA);
			
			System.out.println("Disturbance testDoSetup succeeded.");
			
		} catch (ModelException oErr) {
			fail("Caught error with message " + oErr.getMessage());
		}
	}

	/**
	 * Tests that the correct parameters are displayed for each behavior.
	 */
	public void testFormatDataForDisplay() {
		try {
			GUIManager oManager = new GUIManager(null);
			TreePopulation oPop;
			DisturbanceBehaviors oDist = null;
			String[] p_sExpected;
			Behavior oBeh;

			// Case: competition harvest is enabled.
			oManager.InputXMLParameterFile(WriteXMLValidCompetitionHarvestFile1());
			oDist = oManager.GetDisturbanceBehaviors();
			oPop = oManager.GetTreePopulation();
			p_sExpected = new String[] {
					"Competition Harvest: Max Radius of Competitive Effects (m)",
					"Competition Harvest: Amount of Harvest Per Species (0 - 1)",
					"Competition Harvest: DBH Effect of Targets (alpha)",
					"Competition Harvest: Distance Effect of Targets (beta)",
					"Competition Harvest: Size Sensitivity (gamma)",
					"Competition Harvest: C", "Competition Harvest: D",
					"Competition Harvest: Target DBH Divisor",
					"Competition Harvest: Minimum DBH to Harvest",
					"Competition Harvest: Maximum DBH to Harvest",
					"Competition Harvest: Harvest Type",
					"Competition Harvest: Amount to Harvest",
					"Competition Harvest: Min Years Between Fixed BA Harvests",
					"Competition Harvest: Fixed Interval Harvest Interval (yr)",
					"Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
					"Competition Harvest: Filename for List of Harvested Trees",
					"Competition Harvest: Species 1 Target Lambda",
					"Competition Harvest: Species 3 Target Lambda" };
			TestTable(oDist.FormatDataForDisplay(oPop), p_sExpected);

			// Set the behavior to apply to species 2 and 3 and check again
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			oBeh.ClearSpeciesTypeCombos();
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT,
					oPop));
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT,
					oPop));
			p_sExpected = new String[] {
					"Competition Harvest: Max Radius of Competitive Effects (m)",
					"Competition Harvest: Amount of Harvest Per Species (0 - 1)",
					"Competition Harvest: DBH Effect of Targets (alpha)",
					"Competition Harvest: Distance Effect of Targets (beta)",
					"Competition Harvest: Size Sensitivity (gamma)",
					"Competition Harvest: C", "Competition Harvest: D",
					"Competition Harvest: Target DBH Divisor",
					"Competition Harvest: Minimum DBH to Harvest",
					"Competition Harvest: Maximum DBH to Harvest",
					"Competition Harvest: Harvest Type",
					"Competition Harvest: Amount to Harvest",
					"Competition Harvest: Min Years Between Fixed BA Harvests",
					"Competition Harvest: Fixed Interval Harvest Interval (yr)",
					"Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
					"Competition Harvest: Filename for List of Harvested Trees",
					"Competition Harvest: Species 2 Target Lambda",
					"Competition Harvest: Species 3 Target Lambda" };
			TestTable(oDist.FormatDataForDisplay(oPop), p_sExpected);

			// Set the behavior to apply to all species and check again
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			oBeh.ClearSpeciesTypeCombos();
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT,
					oPop));
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT,
					oPop));
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT,
					oPop));
			p_sExpected = new String[] {
					"Competition Harvest: Max Radius of Competitive Effects (m)",
					"Competition Harvest: Amount of Harvest Per Species (0 - 1)",
					"Competition Harvest: DBH Effect of Targets (alpha)",
					"Competition Harvest: Distance Effect of Targets (beta)",
					"Competition Harvest: Size Sensitivity (gamma)",
					"Competition Harvest: C", "Competition Harvest: D",
					"Competition Harvest: Target DBH Divisor",
					"Competition Harvest: Minimum DBH to Harvest",
					"Competition Harvest: Maximum DBH to Harvest",
					"Competition Harvest: Harvest Type",
					"Competition Harvest: Amount to Harvest",
					"Competition Harvest: Min Years Between Fixed BA Harvests",
					"Competition Harvest: Fixed Interval Harvest Interval (yr)",
					"Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
					"Competition Harvest: Filename for List of Harvested Trees",
					"Competition Harvest: Species 1 Target Lambda",
					"Competition Harvest: Species 2 Target Lambda",
					"Competition Harvest: Species 3 Target Lambda" };
			TestTable(oDist.FormatDataForDisplay(oPop), p_sExpected);
			
		  //Case:  Generalized harvest regime is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidCompetitionHarvestFile1());
      oPop = oManager.GetTreePopulation();
      oDist = oManager.GetDisturbanceBehaviors();
      oBeh = oDist.GetBehaviorByKey("Competition Harvest");
      oBeh.ClearSpeciesTypeCombos();
      oBeh = oDist.GetBehaviorByKey("Generalized Harvest Regime");
      oBeh.ClearSpeciesTypeCombos();
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oBeh = oManager.GetAnalysisBehaviors().GetBehaviorByKey("tree biomass calculator");       
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      p_sExpected = new String[] {
          "Gen Harvest Regime Harvest Probability \"A\"", 
          "Gen Harvest Regime Harvest Probability \"B\"", 
          "Gen Harvest Regime Harvest Probability \"M\"",  
          "Gen Harvest Regime Remove Amount \"Alpha\"", 
          "Gen Harvest Regime Remove Amount \"Mu\"", 
          "Gen Harvest Regime Gamma Scale Parameter", 
          "Gen Harvest Acceptable Deviation From Cut Target",
          "Gen Harvest Regime Cut Preference \"Alpha\"", 
          "Gen Harvest Regime Cut Preference \"Beta\"",
          "Gen Harvest Regime Cut Preference \"Gamma\"", 
          "Gen Harvest Regime Cut Preference \"Mu\"", 
          "Gen Harvest Regime Cut Preference \"A\"", 
          "Gen Harvest Regime Cut Preference \"B\"", 
          "Gen Harvest Regime Cut Preference \"C\""
      };
      TestTable(oDist.FormatDataForDisplay(oPop), p_sExpected);      
			
		//Case:  all are enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoDisturbanceXMLFile());
      oPop = oManager.GetTreePopulation();
      oDist = oManager.GetDisturbanceBehaviors();
      int i;
      for (i = 0; i < oDist.mp_oChildBehaviors.length; i++) {
        if (oDist.mp_oChildBehaviors[i].GetKey().equals("Harvest") == false &&
            oDist.mp_oChildBehaviors[i].GetKey().indexOf("Episodic Mortality") == -1 &&
            oDist.mp_oChildBehaviors[i].GetKey().indexOf("Harvest Interface") == -1) {
        oDist.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oDist.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, 3, oPop));        
        }
      }
      p_sExpected = new String[] {
          "Competition Harvest: Max Radius of Competitive Effects (m)",
          "Competition Harvest: Amount of Harvest Per Species (0 - 1)",
          "Competition Harvest: DBH Effect of Targets (alpha)",
          "Competition Harvest: Distance Effect of Targets (beta)",
          "Competition Harvest: Size Sensitivity (gamma)",
          "Competition Harvest: C", "Competition Harvest: D",
          "Competition Harvest: Target DBH Divisor",
          "Competition Harvest: Minimum DBH to Harvest",
          "Competition Harvest: Maximum DBH to Harvest",
          "Competition Harvest: Harvest Type",
          "Competition Harvest: Amount to Harvest",
          "Competition Harvest: Min Years Between Fixed BA Harvests",
          "Competition Harvest: Fixed Interval Harvest Interval (yr)",
          "Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
          "Competition Harvest: Filename for List of Harvested Trees",
          "Competition Harvest: Species 1 Target Lambda",
          "Minimum DBH for Storm Damage, in cm",
          "Storm Damage Intercept (a) for Medium Damage",
          "Storm Damage Intercept (a) for Heavy Damage", 
          "Storm Intensity Coefficient (b)",
          "Storm DBH Coefficient (d)",
          "Storm Medium Damage Survival Prob Intercept (a)",
          "Storm Medium Damage Survival Prob DBH Coeff. (b)",
          "Storm Heavy Damage Survival Prob Intercept (a)",
          "Storm Heavy Damage Survival Prob DBH Coeff. (b)",
          "Windstorm - Minimum DBH for Windstorm Mortality",
          "Windstorm - Mortality Intercept (a)",
          "Windstorm - Storm Intensity Coefficient (c)",
          "Windstorm - DBH Exponent (b)",
          "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
          "Random Browse - Annual Browse Probability (0-1)",
          "Random Browse - Browse Probability Standard Deviation", 
          "Storm Direct Killer - \"a\"",
          "Storm Direct Killer - \"b\"",
          "Insect Infestation Initial Rate",
          "Insect Infestation Max Rate",
          "Insect Infestation X0",
          "Insect Infestation Xb",
          "Insect Infestation Min DBH",
          "Insect Infestation First Timestep",
          "Return Interval for Severity Storm Class 0 - 0.1",
          "Return Interval for Severity Storm Class 0.1 - 0.2",
          "Return Interval for Severity Storm Class 0.2 - 0.3",
          "Return Interval for Severity Storm Class 0.3 - 0.4",
          "Return Interval for Severity Storm Class 0.4 - 0.5",
          "Return Interval for Severity Storm Class 0.5 - 0.6",
          "Return Interval for Severity Storm Class 0.6 - 0.7",
          "Return Interval for Severity Storm Class 0.7 - 0.8",
          "Return Interval for Severity Storm Class 0.8 - 0.9",
          "Return Interval for Severity Storm Class 0.9 - 1.0",
          "Storm - Sea Surface Temperature Cyclicity Period (Years)",
          "Storm - Storm Cyclicity Sine Curve d",
          "Storm - Storm Cyclicity Sine Curve f",
          "Storm - Storm Cyclicity Sine Curve g",
          "Storm - Storm Cyclicity Trend Function Slope (m)",
          "Storm - Storm Cyclicity Trend Function Intercept (i)",
          "Windstorm - Severity for 1 Year Return Interval Storm",
          "Windstorm - Severity for 5 Year Return Interval Storm",
          "Windstorm - Severity for 10 Year Return Interval Storm",
          "Windstorm - Severity for 20 Year Return Interval Storm",
          "Windstorm - Severity for 40 Year Return Interval Storm",
          "Windstorm - Severity for 80 Year Return Interval Storm",
          "Windstorm - Severity for 160 Year Return Interval Storm",
          "Windstorm - Severity for 320 Year Return Interval Storm",
          "Windstorm - Severity for 640 Year Return Interval Storm",
          "Windstorm - Severity for 1280 Year Return Interval Storm",
          "Windstorm - Severity for 2560 Year Return Interval Storm",
          "Windstorm - Sea Surface Temperature Cyclicity Period (Years)",
          "Windstorm - Storm Cyclicity Sine Curve d",
          "Windstorm - Storm Cyclicity Sine Curve f", 
          "Windstorm - Storm Cyclicity Sine Curve g",
          "Windstorm - Storm Cyclicity Trend Function Slope (m)",
          "Windstorm - Storm Cyclicity Trend Function Intercept (i)",
          "Standard Deviation (lognormal or normal)",
          "Selection Harvest Cut Range 1 Lower DBH (cm)",
          "Selection Harvest Cut Range 1 Upper DBH (cm)",
          "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 2 Lower DBH (cm)",
          "Selection Harvest Cut Range 2 Upper DBH (cm)",
          "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 3 Lower DBH (cm)",
          "Selection Harvest Cut Range 3 Upper DBH (cm)",
          "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 4 Lower DBH (cm)",
          "Selection Harvest Cut Range 4 Upper DBH (cm)",
          "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
          "Selection Harvest Initial Age",
          "Windstorm - Timestep to Start Storms",
          "Number of Years Damaged Trees Take to Heal",
          "Number of Years Storm-Damaged Snags Last",
          "Plot Storm Susceptibility Pattern",
          "Storm Damage Application",
          "Stochastic Pattern Damage Distribution",
          "Gen Harvest Regime Harvest Probability \"A\"", 
          "Gen Harvest Regime Harvest Probability \"B\"", 
          "Gen Harvest Regime Harvest Probability \"M\"",  
          "Gen Harvest Regime Remove Amount \"Alpha\"", 
          "Gen Harvest Regime Remove Amount \"Mu\"", 
          "Gen Harvest Regime Gamma Scale Parameter", 
          "Gen Harvest Acceptable Deviation From Cut Target",
          "Gen Harvest Regime Cut Preference \"Alpha\"", 
          "Gen Harvest Regime Cut Preference \"Beta\"",
          "Gen Harvest Regime Cut Preference \"Gamma\"", 
          "Gen Harvest Regime Cut Preference \"Mu\"", 
          "Gen Harvest Regime Cut Preference \"A\"", 
          "Gen Harvest Regime Cut Preference \"B\"", 
          "Gen Harvest Regime Cut Preference \"C\"",
          "Random Browse - Probability PDF",
        };  
      TestTable(oDist.FormatDataForDisplay(oPop), p_sExpected);

			System.out.println("FormatDataForDisplay succeeded for disturbance.");
		} catch (ModelException oErr) {
			fail("Caught error with message " + oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		}

	}

	/**
	 * Tests the reading of the parameter file.
	 */
	public void testReadParFile() {
		GUIManager oManager = null;
		String sFileName = null;
		try {
			ModelData oData;
			ModelVector oVector;
			int i, iIndex = 0;

			oManager = new GUIManager(null);

			oManager.ClearCurrentData();
			sFileName = WriteXMLValidCompetitionHarvestFile1();
			oManager.InputXMLParameterFile(sFileName);
			DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();

			assertEquals(
					((Float) oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(0))
							.floatValue(), 10.0, 0.001);
			assertNull(oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(1));
			assertEquals(
					((Float) oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(2))
							.floatValue(), 12.0, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvProportion.GetValue()
					.elementAt(0)).floatValue(), 0.25, 0.001);
			//This value is not in the parameter file but this gets a default,
			//so...
			assertEquals(((Float) oDist.mp_fCompHarvProportion.GetValue()
					.elementAt(1)).floatValue(), 1.0, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvProportion.GetValue()
					.elementAt(2)).floatValue(), 0.75, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(0))
					.floatValue(), 2.17031683, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(1))
					.floatValue(), 2.81, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(2))
					.floatValue(), 1.33, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(0))
					.floatValue(), 0.69994199, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(1))
					.floatValue(), 0.5, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(2))
					.floatValue(), 0.09, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(0))
					.floatValue(), -0.43, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(1))
					.floatValue(), -0.36, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(2))
					.floatValue(), -0.4, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvC.GetValue().elementAt(0))
					.floatValue(), 0.698, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvC.GetValue().elementAt(1))
					.floatValue(), 0.457, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvC.GetValue().elementAt(2))
					.floatValue(), 0.469, 0.001);

			assertEquals(((Float) oDist.mp_fCompHarvD.GetValue().elementAt(0))
					.floatValue(), -0.00163, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvD.GetValue().elementAt(1))
					.floatValue(), -0.00126, 0.001);
			assertEquals(((Float) oDist.mp_fCompHarvD.GetValue().elementAt(2))
					.floatValue(), -0.00163, 0.001);

			assertEquals(oDist.m_fCompHarvQ.GetValue(), 100.0, 0.001);
			assertEquals(oDist.m_fCompHarvMinDBH.GetValue(), 20.0, 0.001);
			assertEquals(oDist.m_fCompHarvMaxDBH.GetValue(), 50.0, 0.001);
			assertEquals(oDist.m_iCompHarvHarvType.GetValue(), 1);
			assertEquals(oDist.m_fCompHarvCutAmount.GetValue(), 0.0406, 0.001);
			assertEquals(oDist.m_iCompHarvMinInterval.GetValue(), 2);
			assertEquals(oDist.m_iCompHarvInterval.GetValue(), 3);
			assertEquals(oDist.m_fCompHarvBAThreshold.GetValue(), 0.2, 0.001);
			assertEquals(oDist.m_sCompHarvFilename.GetValue(),
					"core_model_tester1.txt");

			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
					iIndex = i;
					break;
				}
			}
			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertEquals(((Number) oVector.GetValue().elementAt(0)).floatValue(),
					0.66401082, 0.001);
			assertEquals(((Number) oVector.GetValue().elementAt(1)).floatValue(),
					0.71, 0.001);
			assertEquals(((Number) oVector.GetValue().elementAt(2)).floatValue(),
					0.08, 0.001);

			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex + 1);
			assertEquals(0, oVector.GetValue().size());

			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex + 2);
			assertEquals(((Number) oVector.GetValue().elementAt(0)).floatValue(),
					(float) 0.41812471, 0.01);
			assertEquals(((Number) oVector.GetValue().elementAt(1)).floatValue(),
					(float) 0.24, 0.01);
			assertEquals(((Number) oVector.GetValue().elementAt(2)).floatValue(),
					(float) 0.05, 0.01);

			new File(sFileName).delete();
			
			System.out.println("Disturbance testReadParFile succeeded.");

		} catch (ModelException oErr) {
			fail("Disturbance validation failed with message " + oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		}
	}

	/**
	 * Makes sure that competition harvest lambda values are managed correctly 
	 * when species change.
	 */
	public void testChangeOfSpecies() {
		GUIManager oManager = null;
		String sFileName = null;
		try {
			DisturbanceBehaviors oDist;
			 ModelData oData; 
			 ModelVector oVector; 
			 int i, iIndex = 0, iCount;
			  
			 oManager = new GUIManager(null);
			  
			 oManager.ClearCurrentData(); 
			 sFileName = WriteXMLValidCompetitionHarvestFile1();
			 oManager.InputXMLParameterFile(sFileName); 
			 oDist = oManager.GetDisturbanceBehaviors(); 
			 for (i = 0; i < oDist.mp_oAllData.size(); i++) { 
			 	 oData = (ModelData) oDist.mp_oAllData.elementAt(i); 
			 	 if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
			     iIndex = i; 
			     break; 
			   } 
			 } 
			 oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex); 
			 assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1")); 
			 assertEquals(((Number) oVector.GetValue().elementAt(0)).floatValue(),
						0.66401082, 0.001);
				assertEquals(((Number) oVector.GetValue().elementAt(1)).floatValue(),
						0.71, 0.001);
				assertEquals(((Number) oVector.GetValue().elementAt(2)).floatValue(),
						0.08, 0.001);
				
				oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex + 1);
				assertEquals(0, oVector.GetValue().size());

				oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex + 2);
				assertEquals(((Number) oVector.GetValue().elementAt(0)).floatValue(),
						(float) 0.41812471, 0.01);
				assertEquals(((Number) oVector.GetValue().elementAt(1)).floatValue(),
						(float) 0.24, 0.01);
				assertEquals(((Number) oVector.GetValue().elementAt(2)).floatValue(),
						(float) 0.05, 0.01);
			  
			 //Change the species 
			 String[] sNewSpecies = new String[] { "Species 2", "Species 3", "Species 4"};
			 oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies); 
			 oDist = oManager.GetDisturbanceBehaviors();
			 
			 //Count the total number of lambdas 
			 iCount = 0; 
			 for (i = 0; i < oDist.mp_oAllData.size(); i++) { 
				 oData = (ModelData) oDist.mp_oAllData.elementAt(i); 
				 if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
			     iCount++; 
			   } 
			 } 
			 assertEquals(3, iCount);
			  
			 for (i = 0; i < oDist.mp_oAllData.size(); i++) { 
				 oData = (ModelData) oDist.mp_oAllData.elementAt(i); 
				 if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
			     iIndex = i; 
			     break; 
			   } 
			 } 
			 oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex); 
			 assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2") && 
		        -1 != oVector.GetDescriptor().indexOf("Target Lambda")); 
			  
			 iIndex++; 
			 oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex); 
			 assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") && 
					        -1 != oVector.GetDescriptor().indexOf("Target Lambda"));
			  
			 iIndex++; 
			 oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex); 
			 assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") && 
					        -1 != oVector.GetDescriptor().indexOf("Target Lambda"));
			  
			 iIndex++; 
			 if (iIndex < oDist.mp_oAllData.size()) { 
				 oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex); 
				 assertTrue(-1 == oVector.GetDescriptor().indexOf("Target Lambda")); 
			 }
			 
			 System.out.println("Disturbance testChangeOfSpecies succeeded.");
		}
		catch (ModelException oErr) {
		 fail("Disturbance validation failed with message " + oErr.getMessage());
		}
		catch (java.io.IOException oE) {
		 fail("Caught IOException. Message: " + oE.getMessage());
		}
		finally {
			new File(sFileName).delete();
		}
	}

	/**
	 * Makes sure that competition harvest lambda values are managed correctly 
	 * when species are copied.
	 */
	public void testCopySpecies() {
		GUIManager oManager = null;
		String sFileName = null;
		try {
			DisturbanceBehaviors oDist;
			Behavior oBeh;
			ModelData oData;
			ModelVector oVector;
			SpeciesTypeCombo oCombo;
			Float fVal;
			int i, iIndex = 0;

			oManager = new GUIManager(null);

			oManager.ClearCurrentData();
			sFileName = WriteXMLValidCompetitionHarvestFile1();
			oManager.InputXMLParameterFile(sFileName);
			oDist = oManager.GetDisturbanceBehaviors();
			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
					iIndex = i;
					break;
				}
			}

			// Verify how the data was read in before species copy
			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			assertTrue(oBeh.m_bIsEnabled);
			assertEquals(2, oBeh.GetNumberOfCombos());
			oCombo = oBeh.GetSpeciesTypeCombo(0);
			assertEquals(TreePopulation.ADULT, oCombo.GetType());
			assertEquals(0, oCombo.GetSpecies());
			oCombo = oBeh.GetSpeciesTypeCombo(1);
			assertEquals(TreePopulation.ADULT, oCombo.GetType());
			assertEquals(2, oCombo.GetSpecies());

			// Lambdas
			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
					iIndex = i;
					break;
				}
			}
			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
			fVal = (Float) oVector.GetValue().elementAt(0);
			assertEquals(0.66401082, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(1);
			assertEquals(0.71, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(2);
			assertEquals(0.08, fVal.floatValue(), 0.001);
			iIndex++;
			
			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
			assertEquals(0, oVector.GetValue().size());
			iIndex++;

			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3"));
			fVal = (Float) oVector.GetValue().elementAt(0);
			assertEquals(0.41812471, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(1);
			assertEquals(0.24, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(2);
			assertEquals(0.05, fVal.floatValue(), 0.001);
			
			// Copy the species
			oManager.GetTreePopulation().DoCopySpecies("Species_1",	"Species_2");

			oBeh = oDist.GetBehaviorByKey("Competition Harvest");
			assertTrue(oBeh.m_bIsEnabled);
			assertEquals(3, oBeh.GetNumberOfCombos());
			oCombo = oBeh.GetSpeciesTypeCombo(0);
			assertEquals(TreePopulation.ADULT, oCombo.GetType());
			assertEquals(0, oCombo.GetSpecies());
			oCombo = oBeh.GetSpeciesTypeCombo(1);
			assertEquals(TreePopulation.ADULT, oCombo.GetType());
			assertEquals(2, oCombo.GetSpecies());
			oCombo = oBeh.GetSpeciesTypeCombo(2);
			assertEquals(TreePopulation.ADULT, oCombo.GetType());
			assertEquals(1, oCombo.GetSpecies());
			
			fVal = (Float) oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(0); 
			assertEquals(fVal.floatValue(), 10.0, 0.001);
			fVal = (Float) oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), 10.0, 0.001);
			fVal = (Float) oDist.mp_fCompHarvMaxRadius.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), 12.0, 0.001);

			fVal = (Float) oDist.mp_fCompHarvProportion.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), 0.25, 0.001);
			fVal = (Float) oDist.mp_fCompHarvProportion.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), 0.25, 0.001);
			fVal = (Float) oDist.mp_fCompHarvProportion.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), 0.75, 0.001);

			fVal = (Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), 2.17031683, 0.001);
			fVal = (Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), 2.17031683, 0.001);
			fVal = (Float) oDist.mp_fCompHarvAlpha.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), 1.33, 0.001);

			fVal = (Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), 0.69994199, 0.001);
			fVal = (Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), 0.69994199, 0.001);
			fVal = (Float) oDist.mp_fCompHarvBeta.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), 0.09, 0.001);

			fVal = (Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), -0.43, 0.001);
			fVal = (Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), -0.43, 0.001);
			fVal = (Float) oDist.mp_fCompHarvGamma.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), -0.4, 0.001);

			fVal = (Float) oDist.mp_fCompHarvC.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), 0.698, 0.001);
			fVal = (Float) oDist.mp_fCompHarvC.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), 0.698, 0.001);
			fVal = (Float) oDist.mp_fCompHarvC.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), 0.469, 0.001);

			fVal = (Float) oDist.mp_fCompHarvD.GetValue().elementAt(0);
			assertEquals(fVal.floatValue(), -0.00163, 0.001);
			fVal = (Float) oDist.mp_fCompHarvD.GetValue().elementAt(1);
			assertEquals(fVal.floatValue(), -0.00163, 0.001);
			fVal = (Float) oDist.mp_fCompHarvD.GetValue().elementAt(2);
			assertEquals(fVal.floatValue(), -0.00163, 0.001);
			
			// Lambdas
			for (i = 0; i < oDist.mp_oAllData.size(); i++) {
				oData = (ModelData) oDist.mp_oAllData.elementAt(i);
				if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
					iIndex = i;
					break;
				}
			}
			
			
			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
			fVal = (Float) oVector.GetValue().elementAt(0);
			assertEquals(0.66401082, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(1);
			assertEquals(0.66401082, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(2);
			assertEquals(0.08, fVal.floatValue(), 0.001);
			iIndex++;
			
			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
			fVal = (Float) oVector.GetValue().elementAt(0);
			assertEquals(0.66401082, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(1);
			assertEquals(0.66401082, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(2);
			assertEquals(0.08, fVal.floatValue(), 0.001);
			iIndex++;

			oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
			assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3"));
			fVal = (Float) oVector.GetValue().elementAt(0);
			assertEquals(0.41812471, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(1);
			assertEquals(0.41812471, fVal.floatValue(), 0.001);
			fVal = (Float) oVector.GetValue().elementAt(2);
			assertEquals(0.05, fVal.floatValue(), 0.001);
			
			System.out.println("Disturbance testCopySpecies succeeded.");
			
			System.out.println("Disturbance test copy species was successful.");
		} catch (ModelException oErr) {
			fail("Disturbance test copy species failed with message " + oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}
	}
	
	/**
   * Makes sure that competition harvest lambda values are managed correctly 
   * when species are copied with very short names ("A").
   */
  public void testCopySpecies2() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      DisturbanceBehaviors oDist;
      Behavior oBeh;
      ModelData oData;
      ModelVector oVector;
      SpeciesTypeCombo oCombo;
      Float fVal;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidCompetitionHarvestFile1();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      
      //Create a new species
      String[] sNewSpecies = new String[] {
          "Species 1",
          "Species 2",
          "Species 3",
          "A"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Species_1", "A");
      
      for (i = 0; i < oDist.mp_oAllData.size(); i++) {
        oData = (ModelData) oDist.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      // Verify the data
      oBeh = oDist.GetBehaviorByKey("Competition Harvest");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(3, oBeh.GetNumberOfCombos());
      oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(TreePopulation.ADULT, oCombo.GetType());
      assertEquals(0, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(TreePopulation.ADULT, oCombo.GetType());
      assertEquals(2, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(TreePopulation.ADULT, oCombo.GetType());
      assertEquals(3, oCombo.GetSpecies());

      // Lambdas
      for (i = 0; i < oDist.mp_oAllData.size(); i++) {
        oData = (ModelData) oDist.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("target lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.66401082, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.71, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.08, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.66401082, fVal.floatValue(), 0.001);
      iIndex++;
      
      oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals(0, oVector.GetValue().size());
      iIndex++;

      oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41812471, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.24, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.05, fVal.floatValue(), 0.001);
      iIndex++;
      
      oVector = (ModelVector) oDist.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf(" A "));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.66401082, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.71, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.08, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.66401082, fVal.floatValue(), 0.001);
      iIndex++;
     
      
      System.out.println("Disturbance testCopySpecies succeeded.");
      
      System.out.println("Disturbance test copy species was successful.");
    } catch (ModelException oErr) {
      fail("Disturbance test copy species failed with message " + oErr.getMessage());
    } catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    } finally {
      new File(sFileName).delete();
    }
  }

	/**
	 * Tests ValidateData().
	 */
	 public void testValidateData() { 
		 GUIManager oManager = null; 
		 String sFileName = null; 
		 TreePopulation oPop = null; 
		 DisturbanceBehaviors oDist;
		 try {
			 oManager = new GUIManager(null);	 
			 //Valid file 1
			 oManager.ClearCurrentData();
			 sFileName = WriteXMLValidCompetitionHarvestFile2();
			 oManager.InputXMLParameterFile(sFileName);
			 oManager.GetDisturbanceBehaviors().ValidateData(oManager.GetTreePopulation());
			 new File(sFileName).delete();
	  } catch (ModelException oErr) {
	  	fail("Disturbance validation failed with message " + oErr.getMessage()); 
	  } catch (java.io.IOException oE) {
	    fail("Caught IOException. Message: " + oE.getMessage()); 
	  }
	  
    //Harvest type is fixed interval but the interval value is less than 1
	  try { 
	    oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.m_iCompHarvHarvType.SetValue("Fixed Interval");
	    oDist.m_iCompHarvInterval.SetValue(-2);
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch interval less than 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
    
    //Harvest type is fixed BA with proportion to cut but cut amount is not a 
	  //proportion between 0 and 1
	  try { 
	    oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.m_iCompHarvHarvType.SetValue("Fixed BA %");
	    oDist.m_fCompHarvCutAmount.SetValue(15);
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch cut proportion not between "+
	    		"0 and 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
    
	  //Species proportions to cut do not add up to 1 (unless they are all 1)
	  try { 
	  	oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.mp_fCompHarvProportion.GetValue().remove(0);
	    oDist.mp_fCompHarvProportion.GetValue().add(0, new Float(0.4));
	    oDist.mp_fCompHarvProportion.GetValue().remove(2);
	    oDist.mp_fCompHarvProportion.GetValue().add(2, new Float(0.3));
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch interval less than 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
    
	  //Harvest type is fixed BA but the minimum interval is less than 1
	  try { 
	    oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.m_iCompHarvHarvType.SetValue("Fixed BA Amt");
	    oDist.m_iCompHarvMinInterval.SetValue(-2);
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch interval less than 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
	  
    //Harvest type is fixed BA but the minimum interval is less than 1
	  try { 
	    oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.m_iCompHarvHarvType.SetValue("Fixed BA %");
	    oDist.m_iCompHarvMinInterval.SetValue(-2);
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch interval less than 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
    
	  //Any value in max crowding radius is not greater than 0
	  try { 
	  	oManager = new GUIManager(null);
	    oManager.ClearCurrentData(); 
	    sFileName = WriteXMLValidCompetitionHarvestFile1();
	    oManager.InputXMLParameterFile(sFileName);
	    oPop = oManager.GetTreePopulation();
	    oDist = oManager.GetDisturbanceBehaviors();
	    oDist.mp_fCompHarvProportion.GetValue().remove(0);
	    oDist.mp_fCompHarvProportion.GetValue().add(0, new Float(-2));
	    oDist.ValidateData(oPop);
	    fail("Disturbance validation failed to catch crowding radius less than 1."); 
	  }
	  catch (ModelException oErr) { ; } 
	  catch (java.io.IOException oE) {
	  	fail("Caught IOException. Message: " + oE.getMessage()); 	
	  } finally { 
	  	new File(sFileName).delete();
	  }
	  
	  //Generalized harvest regime enabled but not dimension analysis
	  try { 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData(); 
      sFileName = WriteXMLValidCompetitionHarvestFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDist = oManager.GetDisturbanceBehaviors();
      Behavior oBeh = oDist.GetBehaviorByKey("Generalized Harvest Regime");
      oBeh.ClearSpeciesTypeCombos();
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oDist.ValidateData(oPop);
      fail("Disturbance validation failed to catch dimension analysis not +" +
      		"applied to all generalized harvest regime trees."); 
    }
    catch (ModelException oErr) { ; } 
    catch (java.io.IOException oE) {
      fail("Caught IOException. Message: " + oE.getMessage());  
    } finally { 
      new File(sFileName).delete();
    }
    try { 
      oManager.ClearCurrentData(); 
      sFileName = WriteXMLValidCompetitionHarvestFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDist = oManager.GetDisturbanceBehaviors();
      Behavior oBeh = oDist.GetBehaviorByKey("Generalized Harvest Regime");
      oBeh.ClearSpeciesTypeCombos();
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oBeh = oManager.GetAnalysisBehaviors().GetBehaviorByKey("tree biomass calculator");       
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oDist.ValidateData(oPop);
    } catch (ModelException oErr) { 
      fail("Expecting a pass. Message: " + oErr.getMessage()); 
    } catch (java.io.IOException oE) {
      fail("Caught IOException. Message: " + oE.getMessage());  
    } finally { 
      new File(sFileName).delete();
    }
    //Generalized harvest regime not applied to all species
    try { 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData(); 
      sFileName = WriteXMLValidCompetitionHarvestFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDist = oManager.GetDisturbanceBehaviors();
      Behavior oBeh = oDist.GetBehaviorByKey("Generalized Harvest Regime");
      oBeh.ClearSpeciesTypeCombos();
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oBeh = oManager.GetAnalysisBehaviors().GetBehaviorByKey("tree biomass calculator");       
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, TreePopulation.ADULT, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(2, TreePopulation.ADULT, oPop));
      oDist.ValidateData(oPop);
      fail("Disturbance validation failed to catch generalized harvest " +
      		"regime not applied to all species."); 
    }
    catch (ModelException oErr) { ; } 
    catch (java.io.IOException oE) {
      fail("Caught IOException. Message: " + oE.getMessage());  
    } finally { 
      new File(sFileName).delete();
    }
	  
	  System.out.println("Disturbance ValidateData testing succeeded."); 
	}
	 
	 /** 
	  * Writes a valid competition harvest file. 
	  * @return The file name. 
	  * @throws IOException if there is a problem writing the file.
	 */
	protected String WriteXMLValidCompetitionHarvestFile1()
			throws java.io.IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
		oOut.write("<paramFile fileCode=\"06010101\">");
		oOut.write("<plot>");
		oOut.write("<timesteps>8</timesteps>");
		oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
		oOut.write("<randomSeed>1</randomSeed>");
		oOut.write("<plot_lenX>200.0</plot_lenX>");
		oOut.write("<plot_lenY>200.0</plot_lenY>");
		oOut.write("<plot_latitude>55.37</plot_latitude>");
		oOut.write("</plot>");
		oOut.write("<trees>");
		oOut.write("<tr_speciesList>");
		oOut.write("<tr_species speciesName=\"Species_1\" />");
		oOut.write("<tr_species speciesName=\"Species_2\" />");
		oOut.write("<tr_species speciesName=\"Species_3\" />");
		oOut.write("</tr_speciesList>");
		oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
		oOut.write("<tr_minAdultDBH>");
		oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
		oOut.write("</tr_minAdultDBH>");
		oOut.write("<tr_maxSeedlingHeight>");
		oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
		oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
		oOut.write("<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>");
		oOut.write("</tr_maxSeedlingHeight>");
		oOut.write("</trees>");
		oOut.write("<allometry>");
		oOut.write("<tr_canopyHeight>");
		oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_3\">39.48</tr_chVal>");
		oOut.write("</tr_canopyHeight>");
		oOut.write("<tr_stdAsympCrownRad>");
		oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
		oOut.write("</tr_stdAsympCrownRad>");
		oOut.write("<tr_stdCrownRadExp>");
		oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
		oOut.write("</tr_stdCrownRadExp>");
		oOut.write("<tr_conversionDiam10ToDBH>");
		oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
		oOut.write("</tr_conversionDiam10ToDBH>");
		oOut.write("<tr_interceptDiam10ToDBH>");
		oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
		oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
		oOut.write("<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>");
		oOut.write("</tr_interceptDiam10ToDBH>");
		oOut.write("<tr_stdAsympCrownHt>");
		oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
		oOut.write("</tr_stdAsympCrownHt>");
		oOut.write("<tr_stdCrownHtExp>");
		oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
		oOut.write("</tr_stdCrownHtExp>");
		oOut.write("<tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
		oOut.write("</tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_slopeOfAsymHeight>");
		oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
		oOut.write("</tr_slopeOfAsymHeight>");
		oOut.write("<tr_whatSeedlingHeightDiam>");
		oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
		oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
		oOut.write("<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>");
		oOut.write("</tr_whatSeedlingHeightDiam>");
		oOut.write("<tr_whatSaplingHeightDiam>");
		oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
		oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
		oOut.write("<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>");
		oOut.write("</tr_whatSaplingHeightDiam>");
		oOut.write("<tr_whatAdultHeightDiam>");
		oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
		oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
		oOut.write("<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>");
		oOut.write("</tr_whatAdultHeightDiam>");
		oOut.write("<tr_whatAdultCrownRadDiam>");
		oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
		oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
		oOut.write("<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>");
		oOut.write("</tr_whatAdultCrownRadDiam>");
		oOut.write("<tr_whatAdultCrownHeightHeight>");
		oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
		oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
		oOut.write("<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>");
		oOut.write("</tr_whatAdultCrownHeightHeight>");
		oOut.write("<tr_whatSaplingCrownRadDiam>");
		oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
		oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
		oOut.write("<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>");
		oOut.write("</tr_whatSaplingCrownRadDiam>");
		oOut.write("<tr_whatSaplingCrownHeightHeight>");
		oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
		oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
		oOut.write("<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>");
		oOut.write("</tr_whatSaplingCrownHeightHeight>");
		oOut.write("</allometry>");
		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>Competition Harvest</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");
		oOut.write("<disturbanceOther>");
		oOut.write("<di_compHarvMaxCrowdingRadius>");
		oOut.write("<di_chmcrVal species=\"Species_1\">10</di_chmcrVal>");
		oOut.write("<di_chmcrVal species=\"Species_3\">12</di_chmcrVal>");
		oOut.write("</di_compHarvMaxCrowdingRadius>");
		oOut.write("<di_compHarvAlpha>");
		oOut.write("<di_chaVal species=\"Species_1\">2.17031683</di_chaVal>");
		oOut.write("<di_chaVal species=\"Species_2\">2.81</di_chaVal>");
		oOut.write("<di_chaVal species=\"Species_3\">1.33</di_chaVal>");
		oOut.write("</di_compHarvAlpha>");
		oOut.write("<di_compHarvBeta>");
		oOut.write("<di_chbVal species=\"Species_1\">0.69994199</di_chbVal>");
		oOut.write("<di_chbVal species=\"Species_2\">0.5</di_chbVal>");
		oOut.write("<di_chbVal species=\"Species_3\">0.09</di_chbVal>");
		oOut.write("</di_compHarvBeta>");
		oOut.write("<di_compHarvGamma>");
		oOut.write("<di_chgVal species=\"Species_1\">-0.43</di_chgVal>");
		oOut.write("<di_chgVal species=\"Species_2\">-0.36</di_chgVal>");
		oOut.write("<di_chgVal species=\"Species_3\">-0.4</di_chgVal>");
		oOut.write("</di_compHarvGamma>");
		oOut.write("<di_compHarvSpecies_1NeighborLambda>");
		oOut.write("<di_nlVal species=\"Species_1\">0.66401082</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_2\">0.71</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_3\">0.08</di_nlVal>");
		oOut.write("</di_compHarvSpecies_1NeighborLambda>");
		oOut.write("<di_compHarvSpecies_3NeighborLambda>");
		oOut.write("<di_nlVal species=\"Species_1\">0.41812471</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_2\">0.24</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_3\">0.05</di_nlVal>");
		oOut.write("</di_compHarvSpecies_3NeighborLambda>");
		oOut.write("<di_compHarvCrowdingSlope>");
		oOut.write("<di_chcsVal species=\"Species_1\">0.698</di_chcsVal>");
		oOut.write("<di_chcsVal species=\"Species_2\">0.457</di_chcsVal>");
		oOut.write("<di_chcsVal species=\"Species_3\">0.469</di_chcsVal>");
		oOut.write("</di_compHarvCrowdingSlope>");
		oOut.write("<di_compHarvCrowdingSteepness>");
		oOut.write("<di_chcstVal species=\"Species_1\">-0.00163</di_chcstVal>");
		oOut.write("<di_chcstVal species=\"Species_2\">-0.00126</di_chcstVal>");
		oOut.write("<di_chcstVal species=\"Species_3\">-0.00163</di_chcstVal>");
		oOut.write("</di_compHarvCrowdingSteepness>");
		oOut.write("<di_compHarvProportion>");
		oOut.write("<di_chpVal species=\"Species_1\">0.25</di_chpVal>");
		oOut.write("<di_chpVal species=\"Species_3\">0.75</di_chpVal>");
		oOut.write("</di_compHarvProportion>");
		oOut.write("<di_compHarvDbhDivisor>100</di_compHarvDbhDivisor>");
		oOut.write("<di_compHarvMinHarvDBH>20</di_compHarvMinHarvDBH>");
		oOut.write("<di_compHarvMaxHarvDBH>50</di_compHarvMaxHarvDBH>");
		oOut.write("<di_compHarvTypeHarvest>1</di_compHarvTypeHarvest>");
		oOut.write("<di_compHarvCutAmount>0.0406</di_compHarvCutAmount>");
		oOut.write("<di_compHarvBAThreshold>0.2</di_compHarvBAThreshold>");
		oOut.write("<di_compHarvMinInterval>2</di_compHarvMinInterval>");
		oOut.write("<di_compHarvInterval>3</di_compHarvInterval>");
		oOut
				.write("<di_compHarvHarvestedListFile>core_model_tester1.txt</di_compHarvHarvestedListFile>");
		oOut.write("</disturbanceOther>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}
	
	/** 
	  * Writes a valid competition harvest file with extra tweaks to help catch
	  * validation problems.
	  * @return The file name. 
	  * @throws IOException if there is a problem writing the file.
	 */
	protected String WriteXMLValidCompetitionHarvestFile2()
			throws java.io.IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
		oOut.write("<paramFile fileCode=\"06010101\">");
		oOut.write("<plot>");
		oOut.write("<timesteps>8</timesteps>");
		oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
		oOut.write("<randomSeed>1</randomSeed>");
		oOut.write("<plot_lenX>200.0</plot_lenX>");
		oOut.write("<plot_lenY>200.0</plot_lenY>");
		oOut.write("<plot_latitude>55.37</plot_latitude>");
		oOut.write("</plot>");
		oOut.write("<trees>");
		oOut.write("<tr_speciesList>");
		oOut.write("<tr_species speciesName=\"Species_1\" />");
		oOut.write("<tr_species speciesName=\"Species_2\" />");
		oOut.write("<tr_species speciesName=\"Species_3\" />");
		oOut.write("</tr_speciesList>");
		oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
		oOut.write("<tr_minAdultDBH>");
		oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
		oOut.write("</tr_minAdultDBH>");
		oOut.write("<tr_maxSeedlingHeight>");
		oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
		oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
		oOut.write("<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>");
		oOut.write("</tr_maxSeedlingHeight>");
		oOut.write("</trees>");
		oOut.write("<allometry>");
		oOut.write("<tr_canopyHeight>");
		oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_3\">39.48</tr_chVal>");
		oOut.write("</tr_canopyHeight>");
		oOut.write("<tr_stdAsympCrownRad>");
		oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
		oOut.write("</tr_stdAsympCrownRad>");
		oOut.write("<tr_stdCrownRadExp>");
		oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
		oOut.write("</tr_stdCrownRadExp>");
		oOut.write("<tr_conversionDiam10ToDBH>");
		oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
		oOut.write("</tr_conversionDiam10ToDBH>");
		oOut.write("<tr_interceptDiam10ToDBH>");
		oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
		oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
		oOut.write("<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>");
		oOut.write("</tr_interceptDiam10ToDBH>");
		oOut.write("<tr_stdAsympCrownHt>");
		oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
		oOut.write("</tr_stdAsympCrownHt>");
		oOut.write("<tr_stdCrownHtExp>");
		oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
		oOut.write("</tr_stdCrownHtExp>");
		oOut.write("<tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
		oOut.write("</tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_slopeOfAsymHeight>");
		oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
		oOut.write("</tr_slopeOfAsymHeight>");
		oOut.write("<tr_whatSeedlingHeightDiam>");
		oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
		oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
		oOut.write("<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>");
		oOut.write("</tr_whatSeedlingHeightDiam>");
		oOut.write("<tr_whatSaplingHeightDiam>");
		oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
		oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
		oOut.write("<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>");
		oOut.write("</tr_whatSaplingHeightDiam>");
		oOut.write("<tr_whatAdultHeightDiam>");
		oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
		oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
		oOut.write("<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>");
		oOut.write("</tr_whatAdultHeightDiam>");
		oOut.write("<tr_whatAdultCrownRadDiam>");
		oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
		oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
		oOut.write("<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>");
		oOut.write("</tr_whatAdultCrownRadDiam>");
		oOut.write("<tr_whatAdultCrownHeightHeight>");
		oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
		oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
		oOut.write("<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>");
		oOut.write("</tr_whatAdultCrownHeightHeight>");
		oOut.write("<tr_whatSaplingCrownRadDiam>");
		oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
		oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
		oOut.write("<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>");
		oOut.write("</tr_whatSaplingCrownRadDiam>");
		oOut.write("<tr_whatSaplingCrownHeightHeight>");
		oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
		oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
		oOut.write("<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>");
		oOut.write("</tr_whatSaplingCrownHeightHeight>");
		oOut.write("</allometry>");
		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>Competition Harvest</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");
		oOut.write("<disturbanceOther>");
		oOut.write("<di_compHarvMaxCrowdingRadius>");
		oOut.write("<di_chmcrVal species=\"Species_1\">10</di_chmcrVal>");
		oOut.write("<di_chmcrVal species=\"Species_3\">12</di_chmcrVal>");
		oOut.write("</di_compHarvMaxCrowdingRadius>");
		oOut.write("<di_compHarvAlpha>");
		oOut.write("<di_chaVal species=\"Species_1\">2.17031683</di_chaVal>");
		oOut.write("<di_chaVal species=\"Species_2\">2.81</di_chaVal>");
		oOut.write("<di_chaVal species=\"Species_3\">1.33</di_chaVal>");
		oOut.write("</di_compHarvAlpha>");
		oOut.write("<di_compHarvBeta>");
		oOut.write("<di_chbVal species=\"Species_1\">0.69994199</di_chbVal>");
		oOut.write("<di_chbVal species=\"Species_2\">0.5</di_chbVal>");
		oOut.write("<di_chbVal species=\"Species_3\">0.09</di_chbVal>");
		oOut.write("</di_compHarvBeta>");
		oOut.write("<di_compHarvGamma>");
		oOut.write("<di_chgVal species=\"Species_1\">-0.43</di_chgVal>");
		oOut.write("<di_chgVal species=\"Species_2\">-0.36</di_chgVal>");
		oOut.write("<di_chgVal species=\"Species_3\">-0.4</di_chgVal>");
		oOut.write("</di_compHarvGamma>");
		oOut.write("<di_compHarvSpecies_1NeighborLambda>");
		oOut.write("<di_nlVal species=\"Species_1\">0.66401082</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_2\">0.71</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_3\">0.08</di_nlVal>");
		oOut.write("</di_compHarvSpecies_1NeighborLambda>");
		oOut.write("<di_compHarvSpecies_3NeighborLambda>");
		oOut.write("<di_nlVal species=\"Species_1\">0.41812471</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_2\">0.24</di_nlVal>");
		oOut.write("<di_nlVal species=\"Species_3\">0.05</di_nlVal>");
		oOut.write("</di_compHarvSpecies_3NeighborLambda>");
		oOut.write("<di_compHarvCrowdingSlope>");
		oOut.write("<di_chcsVal species=\"Species_1\">0.698</di_chcsVal>");
		oOut.write("<di_chcsVal species=\"Species_2\">0.457</di_chcsVal>");
		oOut.write("<di_chcsVal species=\"Species_3\">0.469</di_chcsVal>");
		oOut.write("</di_compHarvCrowdingSlope>");
		oOut.write("<di_compHarvCrowdingSteepness>");
		oOut.write("<di_chcstVal species=\"Species_1\">-0.00163</di_chcstVal>");
		oOut.write("<di_chcstVal species=\"Species_2\">-0.00126</di_chcstVal>");
		oOut.write("<di_chcstVal species=\"Species_3\">-0.00163</di_chcstVal>");
		oOut.write("</di_compHarvCrowdingSteepness>");
		oOut.write("<di_compHarvProportion>");
		oOut.write("<di_chpVal species=\"Species_1\">0.25</di_chpVal>");
		oOut.write("<di_chpVal species=\"Species_3\">0.75</di_chpVal>");
		oOut.write("</di_compHarvProportion>");
		oOut.write("<di_compHarvDbhDivisor>100</di_compHarvDbhDivisor>");
		oOut.write("<di_compHarvMinHarvDBH>0</di_compHarvMinHarvDBH>");
		oOut.write("<di_compHarvMaxHarvDBH>50</di_compHarvMaxHarvDBH>");
		oOut.write("<di_compHarvTypeHarvest>1</di_compHarvTypeHarvest>");
		oOut.write("<di_compHarvCutAmount>0.0406</di_compHarvCutAmount>");
		oOut.write("<di_compHarvBAThreshold>0.2</di_compHarvBAThreshold>");
		oOut.write("<di_compHarvMinInterval>2</di_compHarvMinInterval>");
		oOut.write("<di_compHarvInterval>3</di_compHarvInterval>");
		oOut
				.write("<di_compHarvHarvestedListFile>core_model_tester1.txt</di_compHarvHarvestedListFile>");
		oOut.write("</disturbanceOther>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}


  /**
   * Writes a file with no disturbance settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNoDisturbanceXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);
  
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");
  
    oOut.close();
    return sFileName;
  }
}